/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// This file defines additional functions, types, variables and constants to be used for initializing the EM
// The definitions are TI specific and are likely different from one application to another

#ifndef _my_em_init_h_
#define _my_em_init_h_

/* importing some of the Open EM defined symbols */
#include "my_event_machine.h"
#include "ti_em_init.h"

#ifdef TI_EM_C6670
#define MY_EM_CORE_NUM (4)
#else
#define MY_EM_CORE_NUM (TI_EM_CORE_NUM)
#endif

#define MY_EM_INIT_CORE_IDX (0)

#define MY_EM_HW_SEM_IDX (3)

#define MY_EM_PROC_QUEUE_NUM (32)
#define MY_EM_PROC_QUEUE_TYPE (EM_QUEUE_TYPE_PARALLEL)
#define MY_EM_PROC_EVENT_TYPE (TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_C)

#define MY_EM_PUBLIC_EVENT_DSC_SIZE     64u

#define MY_EM_POOL_NUM (2)

#define MY_EM_SCHEDULER_THREAD_NUM (1)

#define MY_EM_POOL_IDX (0)
#define MY_EM_EVENT_NUM (2048)
#define MY_EM_EVENT_BUF_SIZE (32*1024+CACHE_L2_LINESIZE)
#define MY_EM_BUF_MODE (TI_EM_BUF_MODE_TIGHT)
#define MY_EM_COH_MODE (TI_EM_COH_MODE_ON)

#define MY_EM_EXIT_POOL_IDX (1)
#define MY_EM_EXIT_EVENT_NUM (MY_EM_CORE_NUM)

#define MY_EM_AP_PRIVATE_FREE_QUEUE_IDX (1022)
#define MY_EM_CD_PRIVATE_FREE_QUEUE_IDX (1023)
#define MY_EM_HW_QUEUE_BASE_IDX (1024)
#define MY_EM_FREE_QUEUE_BASE_IDX (2048)
#define MY_EM_DMA_QUEUE_BASE_IDX (0)

#define MY_EM_PRELOAD_SIZE_A (256)
#define MY_EM_PRELOAD_SIZE_B (2*1024)
#define MY_EM_PRELOAD_SIZE_C (64*1024)
#define MY_EM_PRELOAD_EVENT_NUM (2) // At least 2
#define MY_EM_PRELOAD_EVENT_BUF_SIZE (MY_EM_PRELOAD_SIZE_C)
#define MY_EM_PRELOAD_FREE_QUEUE_BASE_IDX (MY_EM_FREE_QUEUE_BASE_IDX + MY_EM_POOL_NUM)

extern em_queue_t my_em_svProcQueueHdlTbl[];
extern em_queue_t my_em_svSinkQueueHdl;

int my_em_initQueues (
    void
);
int my_em_initGlobal (
    void
);
int my_em_initLocal (
    void
);
int my_em_exitGlobal (
    void
);

size_t my_em_getGlobalSizeFast (
);
size_t my_em_getGlobalSizeSlow (
);
size_t my_em_getLobalSize (
);

#endif /*_my_em_init_h_*/
