/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// This file defines additional functions, types, variables and constants to be used for initializing the EM
// The definitions are TI specific and are likely the same for all applications

#ifndef _ti_em_init_h_
#define _ti_em_init_h_

/* importing some of the Open EM defined symbols */
#include "my_event_machine.h"

#include <ti/drv/qmss/qmss_drv.h>
#include <stdint.h>

#define TI_EM_EVENT_NUM2 (MY_EM_EVENT_NUM + MY_EM_EXIT_EVENT_NUM)

typedef struct ti_em_pool_config2_t_
{
    uint32_t event_num;
    uint8_t *buffer_ptr;
    uint32_t buffer_size;
    uint32_t ps_word_num;
    uint32_t free_policy;
    ti_em_pool_config_t pool_config;
} ti_em_pool_config2_t;

typedef struct ti_em_pl_pool_config2_t_
{
    uint32_t core_idx;
    uint32_t event_num;
    uint8_t *buffer_ptr;
    uint32_t buffer_size;
    uint32_t ps_word_num;
    uint32_t free_queue_idx;
} ti_em_pl_pool_config2_t;

typedef struct ti_em_hw_config2_t_
{
    uint32_t ap_private_free_queue_idx;
    uint32_t cd_private_free_queue_idx;
    uint32_t hw_queue_base_idx;
    uint32_t hw_sem_idx;
    uint32_t dma_queue_base_idx;
    uint32_t preload_size_a;
    uint32_t preload_size_b;
    uint32_t preload_size_c;
} ti_em_hw_config2_t;

em_status_t ti_em_init_global2 (
    uint32_t region_num,
    Qmss_MemRegInfo * region_config_tbl,
    uint32_t pdsp_num,
    Qmss_PdspCfg * pdsp_config_tbl,
    uint32_t pool_num,
    ti_em_pool_config2_t * pool_config2_tbl,
    uint32_t core_num,
    ti_em_pl_pool_config2_t * pl_pool_config2_tbl,
    ti_em_hw_config2_t hw_config2
);

em_status_t ti_em_init_local2 (
    void
);

em_status_t ti_em_exit_global2 (
    void
);

#endif /*_ti_em_init_h_*/
