/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "my_init.h"
#include "my_em_init.h"
#include "my_util.h"
#include "my_receive.h"

/* importing some of the Open EM defined symbols */
#include "my_event_machine.h"

int
main (
    void
)
{
    uint32_t token;
    if (DNUM == MY_EM_INIT_CORE_IDX)
        token = my_waitForProcessBarrier ();

    /*
     * initialization 
     */
    if (my_init () < 0)
        return -1;

    if (DNUM == MY_EM_INIT_CORE_IDX)
        my_endProcessBarrier (token);

    /*
     * on master core 
     */
    if (DNUM == MY_EM_INIT_CORE_IDX)
    {

        /*
         * initialize fft shared variables 
         */
        if (my_initFft () < 0)
            return -1;


        /*
         * initialize jobs 
         */
        if (my_initJobs () < 0)
            return -1;
        /*
         * allocates events, initialize event buffers,
         * * send events 
         */
        if (my_sourceJobs () < 0)
            return -1;

        /*
         * synchronisation barrier 
         */
        my_waitAtCoreBarrier ();

        /*
         * request first event from scheduler 
         */
        ti_em_preschedule ();

        /*
         * call event dispatcher 
         */
        while (my_svRunningJobNum > 0)
            ti_em_dispatch_once ();

        /*
         * compute job statictics 
         */
        if (my_statsJobs () < 0)
        {
            printf ("Debug(Core %d): my_statsJobs failed.\n", DNUM);
            printf ("Debug(Core %d): <CompletionTag> Example #2 failed.\n", DNUM);
            return -1;
        }

        /*
         * terminate program 
         */
        if (my_exit () < 0)
        {
            printf ("Debug(Core %d): my_exit failed.\n", DNUM);
            printf ("Debug(Core %d): <CompletionTag> Example #2 failed.\n", DNUM);
            return -1;
        }

        printf ("Debug(Core %d): <CompletionTag> Example #2 passed.\n", DNUM);
        printf ("\n==========\n");

    }
    /*
     * on dispatcher core 
     */
    else
    {
        /*
         * synchronisation barrier 
         */
        my_waitAtCoreBarrier ();

        /*
         * request first event from scheduler 
         */
        ti_em_preschedule ();

        /*
         * call event dispatcher 
         */
        while (1)
            ti_em_dispatch_once ();
    }

    return 0;
}
