/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// This file implements additional functions, types, variables and constants to be used by the application
// The implementations are application specific.  Each application will likely add/remove implementations.

#include <stdio.h>
#include "my_init.h"
#include "my_em_init.h"
#include "my_util.h"

extern cregister volatile uint32_t DNUM;

int
my_init (
    void
)
{
    /*
     * initialize memory per core 
     */
    if (my_initMemLocal () < 0)
        return -1;

    /*
     * synchronisation barrier 
     */
    my_waitAtCoreBarrier ();

    /*
     * on master core 
     */
    if (DNUM == MY_EM_INIT_CORE_IDX)
    {
        /*
         * initialize memory translation for QMSS 
         */
        if (my_initMemGlobal () < 0)
            return -1;

        /*
         * OpenEM global initialization 
         */
        if (my_em_initGlobal () < 0)
            return -1;

        /*
         * application timer global initialization 
         */
        if (my_initClockGlobal () < 0)
            return -1;
    }

    my_waitAtCoreBarrier ();

    /*
     * OpenEM local initialization 
     */
    if (my_em_initLocal () < 0)
        return -1;

    /*
     * application timer local initialization 
     */
    if (my_initClockLocal () < 0)
        return -1;

    my_waitAtCoreBarrier ();

    /*
     * on master core 
     */
    if (DNUM == MY_EM_INIT_CORE_IDX)
    {
        /*
         * OpenEM objects creation 
         */
        if (my_em_initQueues () < 0)
            return -1;
    }

    return 0;
}

int
my_exit (
    void
)
{
    if (my_em_exitGlobal () < 0)
        return -1;
    return 0;
}
