/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


// This file implements additional functions, types, variables and constants to be used for initializing the EM
// The implementations are TI specific and are likely different from one application to another

#include "my_em_init.h"
#include "ti_em_init.h"
#include <ti/csl/csl_cache.h>
#include <stdlib.h>
#include <stdio.h>

/* array of application event buffers */
#pragma DATA_ALIGN   (my_em_svProcEventBufMem, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svProcEventBufMem, ".my_em_svProcEventBufMem");
uint8_t my_em_svProcEventBufMem[MY_EM_EVENT_NUM * MY_EM_EVENT_BUF_SIZE];


/************************************************************/
/*** BUFFERS POOL                                            */
#pragma DATA_SECTION (chain0Buffer, ".BufferRegion")
#pragma DATA_ALIGN (chain0Buffer, CACHE_L2_LINESIZE)
uint8_t chain0Buffer[MY_EM_PROCESS_CHAIN0_EVENT_NUM * MY_EM_PROCESS_CHAIN0_EVENT_BUF_SIZE];


/* array of preload event buffers */
#pragma DATA_ALIGN   (my_em_gvPlEventBufMem, CACHE_L2_LINESIZE) //
#pragma DATA_SECTION (my_em_gvPlEventBufMem, ".my_em_gvLocalEventBufMem");
uint8_t my_em_gvPlEventBufMem[MY_EM_PL_EVENT_NUM * MY_EM_PL_EVENT_BUF_SIZE];

#pragma DATA_ALIGN   (my_em_svProcEoHdl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svProcEoHdl, ".my_svL1Mem")
em_eo_t my_em_svProcEoHdl;
#pragma DATA_ALIGN   (my_em_svSinkEoHdl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svSinkEoHdl, ".my_svL1Mem")
em_eo_t my_em_svSinkEoHdl;
#pragma DATA_ALIGN   (my_em_svExitEoHdl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svExitEoHdl, ".my_svL1Mem")
em_eo_t my_em_svExitEoHdl;

#pragma DATA_ALIGN   (my_em_svProcQueueHdlTbl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svProcQueueHdlTbl, ".my_svL1Mem")
em_queue_t my_em_svProcQueueHdlTbl[MY_EM_PROC_QUEUE_NUM];
#pragma DATA_ALIGN   (my_em_svSinkQueueHdl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svSinkQueueHdl, ".my_svL1Mem")
em_queue_t my_em_svSinkQueueHdl;
#pragma DATA_ALIGN   (my_em_svExitQueueHdl, CACHE_L2_LINESIZE)
#pragma DATA_SECTION (my_em_svExitQueueHdl, ".my_svL1Mem")
em_queue_t my_em_svExitQueueHdl;

#pragma DATA_SECTION (my_em_svExitStatus, ".tiEmSvPublicEventDscMem")
#pragma DATA_ALIGN (my_em_svExitStatus, CACHE_L2_LINESIZE)
uint32_t my_em_svExitStatus;

void my_processJob (
    void *eoCtxtPtr,
    em_event_t eventHdl,
    em_event_type_t eventType,
    em_queue_t queueHdl,
    void *queueCtxtPtr
);
void my_sinkJob (
    void *eoCtxtPtr,
    em_event_t eventHdl,
    em_event_type_t eventType,
    em_queue_t queueHdl,
    void *queueCtxtPtr
);
void
my_em_exitLocal (
    void *eoCtxtPtr,
    em_event_t eventHdl,
    em_event_type_t eventType,
    em_queue_t queueHdl,
    void *queueCtxtPtr
)
{
    my_em_svExitStatus++;
    em_atomic_processing_end ();
    em_free (eventHdl);
    abort ();
}

uint32_t
my_em_makeAddressGlobal (
    uint32_t coreIdx,
    uint32_t address
)
{
    if (address < 0x01000000)
    {
        address = (0x10 + coreIdx) << 24 | address;
    }

    return address;
}

void
my_em_initPoolConfig (
    ti_em_pool_config2_t * poolConfig2Tbl
)
{
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].pool_config.free_queue_idx = MY_EM_FREE_QUEUE_BASE_IDX + MY_EM_EXIT_POOL_IDX;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].pool_config.buf_mode = TI_EM_BUF_MODE_TIGHT;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].pool_config.dsc_wsize = MY_EM_PUBLIC_EVENT_DSC_SIZE;

    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].event_num = MY_EM_EXIT_EVENT_NUM;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].free_policy = 0;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].buffer_ptr = NULL;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].buffer_size = 0;
    poolConfig2Tbl[MY_EM_EXIT_POOL_IDX].ps_word_num = 0;

    poolConfig2Tbl[MY_EM_POOL_IDX].pool_config.free_queue_idx = MY_EM_FREE_QUEUE_BASE_IDX + MY_EM_POOL_IDX;
    poolConfig2Tbl[MY_EM_POOL_IDX].pool_config.buf_mode = MY_EM_BUF_MODE;
    poolConfig2Tbl[MY_EM_POOL_IDX].pool_config.dsc_wsize = MY_EM_PUBLIC_EVENT_DSC_SIZE;

    poolConfig2Tbl[MY_EM_POOL_IDX].event_num = MY_EM_EVENT_NUM;
    poolConfig2Tbl[MY_EM_POOL_IDX].free_policy = 0;
    poolConfig2Tbl[MY_EM_POOL_IDX].buffer_ptr = my_em_svProcEventBufMem;
    poolConfig2Tbl[MY_EM_POOL_IDX].buffer_size = MY_EM_EVENT_BUF_SIZE;
    poolConfig2Tbl[MY_EM_POOL_IDX].ps_word_num = 2;

    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].pool_config.free_queue_idx =
        MY_EM_FREE_QUEUE_BASE_IDX + MY_EM_PROCESS_CHAIN0_POOL_IDX;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].pool_config.buf_mode = TI_EM_BUF_MODE_TIGHT;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].pool_config.dsc_wsize = MY_EM_PUBLIC_EVENT_DSC_SIZE;

    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].event_num = MY_EM_PROCESS_CHAIN0_EVENT_NUM;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].free_policy = 1;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].buffer_ptr = chain0Buffer;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].buffer_size = MY_EM_PROCESS_CHAIN0_EVENT_BUF_SIZE;
    poolConfig2Tbl[MY_EM_PROCESS_CHAIN0_POOL_IDX].ps_word_num = 2;

}

void
my_em_initPlPoolConfig (
    ti_em_pl_pool_config2_t * poolConfig2Tbl
)
{
    int i;

    for (i = 0; i < MY_EM_CORE_NUM; i++)
    {
        poolConfig2Tbl[i].core_idx = i;
        poolConfig2Tbl[i].event_num = MY_EM_PL_EVENT_NUM;
        poolConfig2Tbl[i].buffer_ptr = (uint8_t *) my_em_makeAddressGlobal (i, (uint32_t) my_em_gvPlEventBufMem);
        poolConfig2Tbl[i].buffer_size = MY_EM_PL_EVENT_BUF_SIZE;
        poolConfig2Tbl[i].ps_word_num = 0;
        poolConfig2Tbl[i].free_queue_idx = MY_EM_PL_FREE_QUEUE_BASE_IDX + i;
    }
}

void
my_em_initHwConfig (
    ti_em_hw_config2_t * hwConfig2Ptr
)
{
    hwConfig2Ptr->ap_private_free_queue_idx = MY_EM_AP_PRIVATE_FREE_QUEUE_IDX;
    hwConfig2Ptr->cd_private_free_queue_idx = MY_EM_CD_PRIVATE_FREE_QUEUE_IDX;
    hwConfig2Ptr->hw_queue_base_idx = MY_EM_HW_QUEUE_BASE_IDX;
    hwConfig2Ptr->hw_sem_idx = MY_EM_HW_SEM_IDX;
    hwConfig2Ptr->dma_queue_base_idx = MY_EM_DMA_QUEUE_BASE_IDX;
    hwConfig2Ptr->preload_size_a = MY_EM_PRELOAD_SIZE_A;
    hwConfig2Ptr->preload_size_b = MY_EM_PRELOAD_SIZE_B;
    hwConfig2Ptr->preload_size_c = MY_EM_PRELOAD_SIZE_C;
    hwConfig2Ptr->my_device_idx = MY_DEVICE_IDX;
    hwConfig2Ptr->my_process_idx = MY_PROCESS_IDX;
}

void
my_em_initHwChainConfig (
    ti_em_chain_config_t * hwChainConfig2Ptr
)
{
    hwChainConfig2Ptr->pool_idx_size_a = MY_EM_PROCESS_CHAIN0_POOL_IDX;
    hwChainConfig2Ptr->pool_idx_size_b = MY_EM_PROCESS_CHAIN0_POOL_IDX;
    hwChainConfig2Ptr->pool_idx_size_c = MY_EM_PROCESS_CHAIN0_POOL_IDX;
    hwChainConfig2Ptr->pool_idx_size_d = MY_EM_PROCESS_CHAIN0_POOL_IDX;
    hwChainConfig2Ptr->pool_idx_overflow = MY_EM_PROCESS_CHAIN0_POOL_IDX;

    hwChainConfig2Ptr->dma_tx_queue_base_idx = MY_EM_CHAINING_DMA_QUEUE_BASE_IDX;

    return;
}

void
my_em_initHwChainRioConfig (
    ti_em_chain_rio_config_t * hwChainRioConfig2Ptr
)
{
    hwChainRioConfig2Ptr->dma_idx = 0xFF;
    hwChainRioConfig2Ptr->dma_tx_queue_idx = MY_EM_SRIO_QUEUE_BASE_IDX;
    hwChainRioConfig2Ptr->service_class = 0;
    hwChainRioConfig2Ptr->my_node_idx[0] = (uint8_t) (MY_RIO_NODE_IDX & 0xFF);
    hwChainRioConfig2Ptr->my_node_idx[1] = (uint8_t) ((MY_RIO_NODE_IDX >> 8) & 0xFF);
    return;
}

int
my_em_initGlobal (
    void
)
{
    ti_em_pool_config2_t lvPoolConfig2Tbl[MY_EM_POOL_NUM];
    ti_em_pl_pool_config2_t lvPlPoolConfig2Tbl[MY_EM_CORE_NUM];
    ti_em_hw_config2_t lvHwConfig2;
    ti_em_chain_config_t lvHwChainConfig2;
    ti_em_chain_rio_config_t lvHwRioChainConfig2;

    /*
     * set pools initialization parameters 
     */
    my_em_initPoolConfig (lvPoolConfig2Tbl);

    /*
     * set preload pools initialization parameters 
     */
    my_em_initPlPoolConfig (lvPlPoolConfig2Tbl);

    /*
     * set OpenEM initialization parameters 
     */
    lvHwConfig2.chain_config_ptr = &lvHwChainConfig2;
    lvHwConfig2.chain_config_ptr->xge_config_ptr = NULL;
    lvHwConfig2.chain_config_ptr->rio_config_ptr = &lvHwRioChainConfig2;

    my_em_initHwConfig (&lvHwConfig2);
    my_em_initHwChainConfig (&lvHwChainConfig2);
    my_em_initHwChainRioConfig (&lvHwRioChainConfig2);

    /*
     * call OpenEM global initialization 
     */
    if (ti_em_init_global2
        (0, NULL, 0, NULL, MY_EM_POOL_NUM, lvPoolConfig2Tbl, MY_EM_CORE_NUM, lvPlPoolConfig2Tbl, lvHwConfig2) != EM_OK)
        return -1;

    return 0;
}

int
my_em_initLocal (
    void
)
{
    /*
     * call OpenEM local initialization 
     */
    if (ti_em_init_local2 () != EM_OK)
        return -1;
    return 0;
}

em_status_t
my_em_eoStartDefault (
    void *eo_ctxt_ptr,
    em_eo_t eo_hdl
)
{
    return EM_OK;
}

em_status_t
my_em_eoStopDefault (
    void *eo_ctxt_ptr,
    em_eo_t eo_hdl
)
{
    return EM_OK;
}

int
my_em_initQueues (
    void
)
{
    uint32_t i;
    em_queue_group_t exit_queue_group;

    /*
     * "Proc EO" creation 
     */
    my_em_svProcEoHdl =
        em_eo_create ("Proc EO", my_em_eoStartDefault, NULL, my_em_eoStopDefault, NULL, my_processJob, NULL);
    if (my_em_svProcEoHdl == EM_EO_UNDEF)
        return -1;

    /*
     * loop over all Proc Queue 
     */
    for (i = 0; i < MY_EM_PROC_QUEUE_NUM; i++)
    {
        /*
         * "Proc Queue" create 
         */
        my_em_svProcQueueHdlTbl[i] =
            em_queue_create ("Proc Queue", MY_EM_PROC_QUEUE_TYPE, EM_QUEUE_PRIO_HIGH, EM_QUEUE_GROUP_DEFAULT);
        if (my_em_svProcQueueHdlTbl[i] == EM_QUEUE_UNDEF)
            return -1;

        /*
         * add "Proc Queue" to "Proc EO" 
         */
        if (em_eo_add_queue (my_em_svProcEoHdl, my_em_svProcQueueHdlTbl[i]) != EM_OK)
            return -1;
    }

    /*
     * "Proc EO" start 
     */
    if (em_eo_start (my_em_svProcEoHdl, NULL, 0, NULL) != EM_OK)
        return -1;


    /*
     * "Sink EO" creation 
     */
    my_em_svSinkEoHdl =
        em_eo_create ("Sink EO", my_em_eoStartDefault, NULL, my_em_eoStopDefault, NULL, my_sinkJob, NULL);
    if (my_em_svSinkEoHdl == EM_EO_UNDEF)
        return -1;

    /*
     * "Sink Queue" create 
     */
    my_em_svSinkQueueHdl =
        em_queue_create ("Sink Queue", EM_QUEUE_TYPE_PARALLEL, EM_QUEUE_PRIO_LOW, EM_QUEUE_GROUP_DEFAULT);
    if (my_em_svSinkQueueHdl == EM_QUEUE_UNDEF)
        return -1;

    /*
     * add "Sink Queue" to "Sink EO" 
     */
    if (em_eo_add_queue (my_em_svSinkEoHdl, my_em_svSinkQueueHdl) != EM_OK)
        return -1;

    /*
     * "Sink EO" start 
     */
    if (em_eo_start (my_em_svSinkEoHdl, NULL, 0, NULL) != EM_OK)
        return -1;


    /*
     * "Exit EO" creation 
     */
    my_em_svExitEoHdl =
        em_eo_create ("Exit EO", my_em_eoStartDefault, NULL, my_em_eoStopDefault, NULL, my_em_exitLocal, NULL);
    if (my_em_svExitEoHdl == EM_EO_UNDEF)
        return -1;

    {
        em_core_mask_t lvMask;
        em_core_mask_zero (&lvMask);

        for (i = 1; i < MY_EM_CORE_NUM; i++)
            em_core_mask_set (i, &lvMask);
        exit_queue_group = em_queue_group_create ("Exit_Queue_Group", (const em_core_mask_t *) &lvMask, 0, NULL);
    }

    /*
     * "Exit Queue" create 
     */
    my_em_svExitQueueHdl = em_queue_create ("Exit Queue", EM_QUEUE_TYPE_ATOMIC, EM_QUEUE_PRIO_LOWEST, exit_queue_group);
    if (my_em_svExitQueueHdl == EM_QUEUE_UNDEF)
        return -1;

    /*
     * add "Exit Queue" to "Exit EO" 
     */
    if (em_eo_add_queue (my_em_svExitEoHdl, my_em_svExitQueueHdl) != EM_OK)
        return -1;

    /*
     * "Exit EO" start 
     */
    if (em_eo_start (my_em_svExitEoHdl, NULL, 0, NULL) != EM_OK)
        return -1;

    return 0;
}

int
my_em_exitGlobal (
    void
)
{
    int i;
    my_em_svExitStatus = 0;
    volatile uint32_t *my_em_svExitStatusPtr = (volatile uint32_t *) &my_em_svExitStatus;

    for (i = 0; i < (MY_EM_CORE_NUM - 1); i++)
    {
        em_event_t lvEventHdl;

        lvEventHdl = em_alloc (0, TI_EM_EVENT_TYPE_PRELOAD_OFF, MY_EM_EXIT_POOL_IDX);
        if (lvEventHdl == NULL)
            exit (-1);

        em_send (lvEventHdl, my_em_svExitQueueHdl);
    }

    while (*my_em_svExitStatusPtr < (MY_EM_CORE_NUM - 1))
    {
        for (i = 0; i < 10000; i++);
    }

    if (ti_em_exit_global2 () != EM_OK)
        return -1;

    return 0;
}
