/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 * 
 * From the original event_machine_hw_config.h.template , openem/event_machine_hw_functions.h was
 * created with:
 *   - A modified returned value for em_get_type_minor() function 
 *   - Doxygen tags are added for API documentation generation.
 *----------------------------------------------------------------------------------------------
 *
 */


/**
 * @file: event_machine_hw_functions.h
 *
 * @brief
 *    Open Event Machine HW functions declarations
 *
 */

#ifndef EVENT_MACHINE_HW_FUNCTIONS_H_
#define EVENT_MACHINE_HW_FUNCTIONS_H_

/**
 * @ingroup DOC_api_generic
 *
 * Get major event type.
 * 
 * Event type includes major and minor part. This function
 * returns the major part. It can be compared against 
 * enumeration em_event_type_major_e.
 * 
 * @param type          Event type
 *
 * @return Major event type
 */
em_event_type_t em_get_type_major(em_event_type_t type);

/**
 * @ingroup DOC_api_generic
 *
 * Get minor event type.
 * 
 * Event type includes major and minor part. This function
 * returns the minor part. It can be compared against 
 * the enumeration specified by the major part.
 * 
 * EM_EVENT_TYPE_SW_DEFAULT is reserved for (SW) events that are
 * generic and directly accessible buffers of memory. 
 * 
 * @param type          Event type
 *
 * @return Minor event type
 */
em_event_type_t em_get_type_minor(em_event_type_t type);

#endif /* EVENT_MACHINE_HW_FUNCTIONS_H_ */

