/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 * 
 * From the original event_machine_hw_config.h.template , openem/event_machine_hw_macros.h was
 * created with:
 *   - Static HW macro definitions with the TI optimized implementation values
 *   - Doxygen tags are added for API documentation generation.
 *----------------------------------------------------------------------------------------------
 *
 */


/**
 * @file: event_machine_hw_macros.h
 *
 * @brief
 *    Open Event Machine HW TI specific macros
 *
 */

#ifndef EVENT_MACHINE_HW_MACROS_H
#define EVENT_MACHINE_HW_MACROS_H

#include <ti/runtime/openem/event_machine_hw_ti_macros.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 *
 * HW specific constants
 ***************************************
 */


/** 
 * @ingroup DOC_api_generic
 *
 * Invalid identifier (64-bit)
 *
 */
#define EM_UNDEF_U64       (0x0000000000000000u)

/** 
 * @ingroup DOC_api_generic
 *
 * Invalid identifier (32-bit)
 */
#define EM_UNDEF_U32       (0x00000000u)

/** 
 * @ingroup DOC_api_generic
 *
 * Invalid identifier (16-bit)
 */
#define EM_UNDEF_U16       (0x0000u)

/** 
 * @ingroup DOC_api_generic
 *
 * Invalid identifier (8-bit)
 */
#define EM_UNDEF_U8        (0x00u)

/**
 * @ingroup DOC_api_generic
 *
 * Undefined event
 */
#define EM_EVENT_UNDEF     (0x00000000u)

/**
 * @ingroup DOC_api_generic
 * 
 * Default queue group for EM
 */
#define EM_QUEUE_GROUP_DEFAULT    (1u)

/**
 * @ingroup DOC_api_generic
 * 
 * Maximum number of queue groups supported by the event machine.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 *
 */
#define EM_MAX_QUEUE_GROUPS       (16u)

/**
 * @ingroup DOC_api_generic
 * 
 * Define default memory pool
 */
#define EM_POOL_DEFAULT           (0u)

/**
 * @ingroup DOC_api_generic
 * 
 * Fatal error mask
 */
#define EM_ERROR_FATAL_MASK       (0x80000000u)

/**
 * @ingroup DOC_api_generic
 * 
 * Test if error is fatal
 */
#define EM_ERROR_IS_FATAL(error)  (EM_ERROR_FATAL_MASK & (error))

/**
 * @ingroup DOC_api_generic
 * 
 * Set a fatal error code
 */
#define EM_ERROR_SET_FATAL(error) (EM_ERROR_FATAL_MASK | (error))

/**
 * @ingroup DOC_api_generic
 * 
 * EM internal error scope mask
 */
#define EM_ESCOPE_INTERNAL_TYPE       (0x00u)

/**
 * @ingroup DOC_api_generic
 * 
 * EM internal error scope type
 */
#define EM_ESCOPE_INTERNAL_MASK             (EM_ESCOPE_BIT | (EM_ESCOPE_INTERNAL_TYPE << 24))

/**
 * @ingroup DOC_api_generic
 * 
 * Test if the error scope identifies an EM Internal function
 */
#define EM_ESCOPE_INTERNAL(escope) (((escope) & EM_ESCOPE_MASK) == EM_ESCOPE_INTERNAL_MASK)

/*
 *
 * EM scaling
 ***************************************
 */



/**
 * @ingroup DOC_api_generic
 * 
 * Static queues: minimal value
 *
 */
#define EM_QUEUE_STATIC_MIN         (1)
/**
 * @ingroup DOC_api_generic
 * 
 * Static queues: maximal value
 *
 */
#define EM_QUEUE_STATIC_MAX         ((TI_EM_QUEUE_SET_NUM * TI_EM_STATIC_QUEUE_NUM_IN_SET) - 1)
/**
 * @ingroup DOC_api_generic
 * 
 * Static queues: number of status queues
 *
 */
#define EM_QUEUE_STATIC_NUM         (EM_QUEUE_STATIC_MAX - EM_QUEUE_STATIC_MIN + 1)


#ifdef __cplusplus
}
#endif


#endif

