/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 * @file: event_machine_hw_ti_macros.h
 *
 * @brief
 *    Open Event Machine HW TI specific constant definitions
 *
 */

#ifndef EVENT_MACHINE_HW_TI_MACROS_H_
#define EVENT_MACHINE_HW_TI_MACROS_H_

/**
 * @ingroup DOC_api_specific
 *
 * Number of 32 bits words in an ti_em_iterator_t array.
 */
#define TI_EM_ITERATOR_WSIZE (8u)

/**
 * @ingroup DOC_api_specific
 *
 * Number of PF events per core.
 */
#define TI_EM_PF_LEN (1u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of cores on a device on which the Event Machine can be executed.
 */
#define TI_EM_CORE_NUM (8u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of Navigator PDSP instances on which this process scheduler can be executed.
 * Only relevant on a Keystone II target.
 */
#define TI_EM_SCHEDULER_THREAD_NUM (4u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of Open-EM processes on a device on which the Event Machine can be executed.
 */
#define TI_EM_PROCESS_NUM (2u)

/**
 * @ingroup DOC_api_specific
 *
 * Chaining disabled on Open-EM process.
 */
#define TI_EM_CHAIN_DISABLED (0u)

/**
 * @ingroup DOC_api_specific
 *
 * Chaining enabled on Open-EM process.
 */
#define TI_EM_CHAIN_ENABLED (1u)

/** 
 * @ingroup DOC_api_specific
 *
 * Maximum number of TX queues for chaining and poststoring. Must be a power of 2: 1, 2, 4 or 8.
 */
#ifdef TI_EM_LINUX
#define TI_EM_CHAIN_TX_QUEUE_NUM (1u)
#else /* TI_EM_LINUX */
#define TI_EM_CHAIN_TX_QUEUE_NUM (4u)
#endif /* TI_EM_LINUX */

/**
 * @ingroup DOC_api_specific
 *
 * Preload disabled.
 */
#define TI_EM_PRELOAD_DISABLED (0u)

/**
 * @ingroup DOC_api_specific
 *
 * Preload enabled.
 */
#define TI_EM_PRELOAD_ENABLED (1u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of devices on which the Open-EM can be executed.
 */
#define TI_EM_DEVICE_NUM (256)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of queue sets. Must be a power of 2.
 */
#define TI_EM_QUEUE_SET_NUM (256u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of queues in a set. TI_EM_QUEUE_SET_NUM * TI_EM_QUEUE_NUM_IN_SET <= TI_EM_QUEUE_NUM_MAX.
 */
#define TI_EM_QUEUE_NUM_IN_SET (8u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of queues.
 */
#define TI_EM_QUEUE_NUM_MAX (16384u)

/**
 * @ingroup DOC_api_specific
 *
 * Number of static queues in a set. TI_EM_QUEUE_SET_NUM*TI_EM_STATIC_QUEUE_NUM_IN_SET <= TI_EM_STATIC_QUEUE_NUM_MAX
 */
#define TI_EM_STATIC_QUEUE_NUM_IN_SET (2u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of static queues. TI_EM_STATIC_QUEUE_NUM_MAX <= TI_EM_QUEUE_NUM_MAX.
 */
#define TI_EM_STATIC_QUEUE_NUM_MAX (256u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of execution objects.
 */
#define TI_EM_EO_NUM_MAX (TI_EM_QUEUE_NUM_MAX)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of event groups.
 */
#define TI_EM_EVENT_GROUP_NUM_MAX (16384u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of queue groups.
 */
#define TI_EM_QUEUE_GROUP_NUM_MAX (TI_EM_PRIO_NUM * TI_EM_CORE_NUM)

/**
 * @ingroup DOC_api_specific
 *
 * Alignment of QM queue base index
 */
#define TI_EM_HW_QUEUE_STEP (32u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of XGE streams.
 */
#define TI_EM_STREAM_NUM (256u)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) of the XGE header.
 */
#define TI_EM_XGE_ENET_HEADER_SIZE (18u)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) of the XGE header.
 */
#define TI_EM_XGE_CHAIN_HEADER_SIZE (8u)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) of the XGE header (18+8=26).
 */
#define TI_EM_XGE_HEADER_SIZE (TI_EM_XGE_ENET_HEADER_SIZE + TI_EM_XGE_CHAIN_HEADER_SIZE)

/**
 * @ingroup DOC_api_specific
 *
 * Minimal size (in bytes) of the XGE frame.
 */
#define TI_EM_XGE_FRAME_SIZE_MIN (64u)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) of the CRC added by XGE.
 */
#define TI_EM_XGE_CRC_SIZE (4)

/**
 * @ingroup DOC_api_specific
 *
 * Minimum size (in bytes) of the fragment size (64 - 26 - 4 = 34 Bytes).
 */
#define TI_EM_XGE_PAYLOAD_SIZE_MIN (TI_EM_XGE_FRAME_SIZE_MIN - TI_EM_XGE_HEADER_SIZE - TI_EM_XGE_CRC_SIZE )

/**
 * @ingroup DOC_api_specific
 *
 * Minimal size (in bytes) to be allocated for the XGE RX fragment buffer (34 + 4 = 38).
 */
#define TI_EM_XGE_RX_FRAGMENT_SIZE_MIN (TI_EM_XGE_PAYLOAD_SIZE_MIN + TI_EM_XGE_CRC_SIZE)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) to be allocated for the XGE TX header buffer (26 + 34 = 60).
 */
#define TI_EM_XGE_TX_HEADER_SIZE (TI_EM_XGE_HEADER_SIZE + TI_EM_XGE_PAYLOAD_SIZE_MIN)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) to be allocated for the XGE RX header buffer (26).
 */
#define TI_EM_XGE_RX_HEADER_SIZE (TI_EM_XGE_HEADER_SIZE)

/**
 * @ingroup DOC_api_specific
 *
 * Size (in bytes) to be allocated for the XGE TX fragment buffer
 */
#define TI_EM_XGE_TX_FRAGMENT_SIZE 0

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of contiguous XGE TX queues
 */
#define TI_EM_XGE_TX_QUEUE_NUM (1u)

/**
 * @ingroup DOC_api_specific
 *
 * Number of VLAN priorities that need to be reserved for chaining over XGE
 */
#define TI_EM_XGE_VLAN_PRIO_NUM (8u)

/**
 * @ingroup DOC_api_specific
 *
 * Number of contiguous RX miss queues that need to be reserved for chaining over XGE
 */
#define TI_EM_XGE_RX_MISS_QUEUE_NUM (TI_EM_XGE_VLAN_PRIO_NUM)

/**
 * @ingroup DOC_api_specific
 *
 * Number of contiguous TX divert queues that need to be reserved for chaining over XGE
 */
#define TI_EM_XGE_TX_DIVERT_QUEUE_NUM (32u)

/**
 * @ingroup DOC_api_specific
 *
 * Total number of PKTDMA queues used by the EM
 */
#define TI_EM_DMA_QUEUE_NUM (TI_EM_CORE_NUM + 2)

/**
 * @ingroup DOC_api_specific
 *
 * Highest Priority
 */
#define TI_EM_PRIO_NUM (EM_QUEUE_PRIO_HIGHEST + 1)

/**
 * @ingroup DOC_api_specific
 *
 * Total number of queues used by the EM
 */
#define TI_EM_HW_QUEUE_NUM (11                        \
                + TI_EM_PRIO_NUM                \
                + (3 * TI_EM_CORE_NUM)            \
                + (3 * TI_EM_CORE_NUM * TI_EM_PRIO_NUM)    \
                + (TI_EM_CORE_NUM * TI_EM_PRIO_NUM * TI_EM_PF_LEN) \
                + TI_EM_XGE_VLAN_PRIO_NUM)

/**
 * @ingroup DOC_api_specific
 *
 * Size for all the private token used by the EM
 */
#define TI_EM_PRIVATE_EVENT_DSC_SIZE (16u)

/**
 * @ingroup DOC_api_specific
 *
 * Multi-core Navigator PDSP memory size to allocated for
 * private events and private firmware data.
 */
#define TI_EM_PDSP_GLOBAL_DATA_SIZE (8192u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of AP private tokens used by the EM
 */
#define TI_EM_AP_PRIVATE_EVENT_NUM (256u)

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of CD private tokens used by the EM
 */
#define TI_EM_CD_PRIVATE_EVENT_NUM (64u)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the em_alloc API.
 *
 */
#define TI_EM_TSCOPE_ALLOC (1)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the em_send API.
 *
 */
#define TI_EM_TSCOPE_SEND (2)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the ti_em_dispatch_once API.
 *
 */
#define TI_EM_TSCOPE_DISPATCH (3)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the ti_em_preschedule API.
 *
 */
#define TI_EM_TSCOPE_PRESCHEDULE (4)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the ti_em_claim_local API.
 *
 */
#define TI_EM_TSCOPE_CLAIM_LOCAL (5)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the em_atomic_processing_end API.
 *
 */
#define TI_EM_TSCOPE_ATOMIC_PROCESSING_END (6)

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope for the em_free API.
 *
 */
#define TI_EM_TSCOPE_FREE (7)

/**
 * @ingroup DOC_api_specific
 * 
 * Major event types (portable) : Data Preloading Off
 * @note Application should always ignore the actual values.
 */
#define TI_EM_EVENT_TYPE_PRELOAD_OFF (0u)

/**
 * @ingroup DOC_api_specific
 * 
 * Major event types (portable) : Data Preloading up to size A
 * @note Application should always ignore the actual values.
 */
#define TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_A (1u<<6)

/**
 * @ingroup DOC_api_specific
 * 
 * Major event types (portable) : Data Preloading up to size B
 * @note Application should always ignore the actual values.
 */
#define TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_B (2u<<6)

/**
 * @ingroup DOC_api_specific
 * 
 * Major event types (portable) : Data Preloading up to size C
 * @note Application should always ignore the actual values.
 */
#define TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_C (3u<<6)

/**
 * @ingroup DOC_api_specific
 *
 * Event type preload mask.
 *
 */
#define TI_EM_EVENT_TYPE_PRELOAD_MSK (0xC0)

/**
 * @ingroup DOC_api_specific
 *
 * Buffer mode - Tight buffer
 */
#define TI_EM_BUF_MODE_TIGHT (0)

/**
 * @ingroup DOC_api_specific
 *
 * Buffer mode - Loose buffer
 */
#define TI_EM_BUF_MODE_LOOSE (1)

/**
 * @ingroup DOC_api_specific
 *
 * Cache coherency mode - Off
 */
#define TI_EM_COH_MODE_OFF (0x0)

/**
 * @ingroup DOC_api_specific
 *
 * Cache coherency mode - On
 */
#define TI_EM_COH_MODE_ON (0x1)

/**
 * @ingroup DOC_api_specific
 *
 * Cache coherency mode - Reserved0
 */
#define TI_EM_COH_MODE_RESERVED0 (0x2)

/**
 * @ingroup DOC_api_specific
 *
 * Cache coherency mode - Reserved1
 */
#define TI_EM_COH_MODE_RESERVED1 (0x3)

/**
 * @ingroup DOC_api_specific
 *
 * Queue associated with a scheduling (SD) queue.
 */
#define TI_EM_QUEUE_MODE_SD (0u)

/**
 * @ingroup DOC_api_specific
 *
 * Queue associated with an hardware (HW) queue.
 */
#define TI_EM_QUEUE_MODE_HW (1u)

/**
 * @ingroup DOC_api_specific
 *
 * Push policy to tail
 */
#define TI_EM_PUSH_POLICY_TAIL (0x0)

/**
 * @ingroup DOC_api_specific
 *
 * Push policy to head
 */
#define TI_EM_PUSH_POLICY_HEAD (0x1)

/**
 * @ingroup DOC_api_specific
 *
 * ID for the various chaining mechanisms (i.e. chaining modules)
 */
#define TI_EM_CHAINING_PKTDMA    (0)
#define TI_EM_CHAINING_XGE       (1)
#define TI_EM_CHAINING_RIO       (2)
#define TI_EM_CHAINING_POSTSTORE (3)

/**
 * @ingroup DOC_api_specific
 *
 * Definitions of the various PDSP communication memory slots
 */
#define TI_EM_SLOT_SPCB        (0)
#define TI_EM_SLOT_RPCB        (1)
#define TI_EM_SLOT_SPSB        (2)
#define TI_EM_SLOT_RPSB        (3)
#define TI_EM_SLOT_RDCB        (4)
#define TI_EM_SLOT_ESB         (5 + DNUM)

/*
 * @ingroup DOC_api_specific
 * 
 * These are the different memory mappings for I/O, descriptors or PDSP memory
 */
#define TI_EM_MEM_NONE         (0)
#define TI_EM_MEM_QMSS_REGS    (1)
#define TI_EM_MEM_PDSP_DRAM    (2)
#define TI_EM_MEM_PUBLIC_DESC  (3)
#define TI_EM_MEM_LOCAL_DESC   (4)
#define TI_EM_MEM_PRIVATE_DESC (5)
#define TI_EM_MEM_PDSP_COMM    (6)
#define TI_EM_MEM_BUFFER       (7)
#define TI_EM_MEM_NUM          (TI_EM_BUFFER_POOL_ID_NUM + TI_EM_MEM_BUFFER) /* total number of available mappings */

/**
 * @ingroup DOC_api_specific
 *
 * Maximum number of pool Id that can be set in the event descriptor pool index field (8 bits)
 */
#define TI_EM_BUFFER_POOL_ID_NUM (256u)

/**
 * @ingroup DOC_api_specific
 *
 * Enable/disable PDSP interrupt generation
 */
#define TI_EM_INTERRUPT_DISABLE (0)
#define TI_EM_INTERRUPT_ENABLE  (1)

#endif /*EVENT_MACHINE_HW_TI_MACROS_H_*/
