/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 * @file: event_machine_hw_ti_types.h
 *
 * @brief
 *    Open Event Machine HW TI specific type definitions
 *
 */

#ifndef EVENT_MACHINE_HW_TI_TYPES_H_
#define EVENT_MACHINE_HW_TI_TYPES_H_

/**
 * @ingroup DOC_api_specific
 *
 * typedef associated with ti_em_config_t_
 */
typedef struct ti_em_config_t_ ti_em_config_t;
typedef struct ti_em_chain_config_t_ ti_em_chain_config_t;
typedef struct ti_em_chain_xge_config_t_ ti_em_chain_xge_config_t;
typedef struct ti_em_chain_rio_config_t_ ti_em_chain_rio_config_t;
typedef struct ti_em_preload_config_t_ ti_em_preload_config_t;
typedef struct ti_em_poststore_config_t_ ti_em_poststore_config_t;
typedef struct ti_em_device_xge_route_t_ ti_em_device_xge_route_t;
typedef struct ti_em_device_rio_route_t_ ti_em_device_rio_route_t;
typedef struct ti_em_buffer_config_t_ ti_em_buffer_config_t;
typedef struct ti_em_pool_config_t_ ti_em_pool_config_t;
typedef struct ti_em_pair_t_ ti_em_pair_t;
typedef struct ti_em_iterator_t_ ti_em_iterator_t; 
typedef struct ti_em_process_route_t_ ti_em_process_route_t; 

#include <inttypes.h>
#include <stdlib.h>
#include <ti/runtime/openem/event_machine_hw_types.h>
#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti/runtime/openem/event_machine_sw_ti_config.h>
#include <ti/runtime/openem/src/event_machine_hwpform.h>

/**
 * @ingroup DOC_api_specific
 *
 * Rx miss type.
 * Identifies the rx miss type associated with the chaining over xge.
 *
 */
typedef uint32_t ti_em_xge_rx_miss_type_t;

/**
 * @ingroup DOC_api_specific
 *
 * Counter type.
 * Identifies the counter type.
 *
 */
typedef uint32_t ti_em_counter_type_t;

/**
 * @ingroup DOC_api_specific
 *
 * Counter value.
 * Identifies the counter value.
 *
 */
typedef uint32_t ti_em_counter_value_t;

/**
 * @ingroup DOC_api_specific
 *
 * Packet.
 * Identifies the packet of the event.
 *
 */
typedef uint32_t ti_em_packet_t;

/**
 * @ingroup DOC_api_specific
 *
 * Event queue id.
 * Identifies the event queue.
 *
 */
typedef uint16_t ti_em_queue_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * DMA id.
 * Identifies the DMA.
 *
 */
typedef uint32_t ti_em_dma_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * flow id.
 * Identifies the flow.
 *
 */
typedef uint32_t ti_em_flow_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Hardware semaphore id.
 * Identifies the hardware semaphore.
 *
 */
typedef uint32_t ti_em_sem_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Openem process id.
 * Identifies the openem process.
 *
 */
typedef uint8_t ti_em_process_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Openem process type.
 * Identifies the openem process type.
 *
 */
typedef uint32_t ti_em_process_type_t;

/**
 * @ingroup DOC_api_specific
 *
 * Pdsp id.
 * Identifies the Pdsp.
 *
 */
typedef uint8_t ti_em_pdsp_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Device id.
 * Identifies the device.
 *
 */
typedef uint8_t ti_em_device_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Destination id.
 * Identifies the destination for a route. It may be a ti_em_process_id_t or
 * a ti_em_device_id_t.
 */
typedef uint32_t ti_em_destination_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Stream id.
 * Identifies the stream.
 *
 */
typedef uint32_t ti_em_stream_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Trace scope.
 * Identifies the scope for interpreting trace codes and variable arguments
 *
 */
typedef uint32_t ti_em_tscope_t;

/**
 * @ingroup DOC_api_specific
 *
 * Cache Coherency mode.
 * Identifies the data buffer cache coherency mode.
 *
 */
typedef uint32_t ti_em_coh_mode_t;

/**
 * @ingroup DOC_api_specific
 *
 *
 * Identifies the event buffer mode.
 *
 */
typedef uint32_t ti_em_buf_mode_t;

/**
 * @ingroup DOC_api_specific
 *
 *
 * Identifies the push policy.
 *
 */
typedef uint32_t ti_em_push_policy_t;

/**
 * @ingroup DOC_api_specific
 *
 * Identifies the Interrupt index.
 *
 */
typedef uint8_t ti_em_interrupt_id_t;

/**
 * @ingroup DOC_api_specific
 *
 * Identifies the Queue Mode.
 *
 */
typedef uint8_t ti_em_queue_mode_t;

/**
 * @ingroup DOC_api_specific
 *
 *  The structure describes the XGE chain header miss types checked by the XGE router.
 *
 */
typedef enum ti_em_xge_rx_miss_type_e
{
    /**
     * @brief   XGE chain header ETHER_TYPE check.
     */
    ti_em_xge_rx_miss_type_ETHER_TYPE = 0,
    /**
     * @brief   XGE chain header SERVICE_TYPE check.
     */
    ti_em_xge_rx_miss_type_SERVICE_TYPE = 1    

} ti_em_xge_rx_miss_type_e;

/**
 * @ingroup DOC_api_specific
 *
 *  The structure describes the counters addressed by the ti_em_counter_get() api.
 *
 */
typedef enum ti_em_counter_type_e
{
     /**
     * @brief   XGE router counter counting the received fragments.
     */
    ti_em_counter_type_XGE_RECEIVE = 0,
     /**
     * @brief   XGE router counter counting the received fragments with wrong ether type.
     */
    ti_em_counter_type_XGE_WRONG_ETHER_TYPE = 1,    
     /**
     * @brief   XGE router counter counting the received fragments with wrong MAC address.
     */
    ti_em_counter_type_XGE_WRONG_MAC = 2,
     /**
     * @brief   XGE router counter counting the received fragments with wrong process index.
     */
    ti_em_counter_type_XGE_WRONG_PROCESS = 3,
     /**
     * @brief   XGE router counter counting the received fragments with wrong sequence index.
     */
    ti_em_counter_type_XGE_WRONG_SEQUENCE = 4,
     /**
     * @brief   XGE router counter counting the received fragments with wrong service type.
     */
    ti_em_counter_type_XGE_WRONG_SERVICE_TYPE = 5    
} ti_em_counter_type_e;

/**
 * @ingroup DOC_api_specific
 *
 * Free funtion handler.
 * 
 * @param buffer_ptr        buffer pointer
 * @param buffer_size       buffer size
 *
 */
typedef void (*ti_em_free_func_t)(void* buffer_ptr, size_t buffer_size);

/**
 * @ingroup DOC_api_specific
 *
 * Trace function handler.
 * 
 * @param tscope        Error scope. Identifies the scope for interpreting the error code and variable arguments
 * @param args          Variable number and type of arguments
 *
 * @return The function may not return depending on implementation.
 *
 * @see em_register_trace_handler(), em_unregister_trace_handler()
 */
typedef em_status_t (*ti_em_trace_handler_t)(ti_em_tscope_t tscope, ...);

/**
 * @ingroup DOC_api_specific
 *
 *
 * Identifies the iterator type.
 *
 * @details This describes the array of iterator fields.
 *
 */
struct ti_em_iterator_t_ 
{
    /**
      * array of iterator fields
      */
    uint32_t body[TI_EM_ITERATOR_WSIZE];
}; 

/**
 * @ingroup DOC_api_specific
 *
 * Data buffer pool configuration.
 *
 * @details This describes the data buffer pool configurable parameters which are required.
 */
struct ti_em_pool_config_t_
{
    /**
      * Number of bytes used for storing a buffer
      */
    size_t buf_size;

    /**
      * Number of words used for storing a descriptor
      */
    size_t dsc_wsize;
    
    /**
      * Event machine pool buffer mode
      */
    ti_em_buf_mode_t buf_mode;

    /**
      * Event machine pool free queues
      */
    uint16_t free_queue_idx;
};


/**
 * @ingroup DOC_api_specific
 *
 * Buffer parameters.
 *
 * @details This describes the buffer configurable parameters which are required.
 */
struct ti_em_buffer_config_t_
{
    /**
      * Buffer pointer
      */
    void* buffer_ptr;

    /**
      * Buffer size
      */
    size_t buffer_size;

    /**
      * Original buffer pointer
      */
    void* orig_buffer_ptr;

    /**
      * Original buffer size
      */
    size_t orig_buffer_size;

   /**
      * Original buffer pool id 
      */
    uint32_t orig_buffer_pool;

    /**
      * Buffer coherency mode
      */
    ti_em_coh_mode_t coh_mode;

    /**
      * Buffer free function
      */
    ti_em_free_func_t free_func;

    /**
      * Event machine pool free push policy
      */
    ti_em_push_policy_t free_push_policy;
};

/**
 * @ingroup DOC_api_specific
 *
 * Event Machine hardware configuration.
 *
 * @details This describes the EM configurable parameters which are required.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_config_t_, 64)
#endif
struct ti_em_config_t_
{
    /**
      * Pointer to the chaining configuration.
      */
    ti_em_chain_config_t* chain_config_ptr;

    /**
      * Pointer to the preload configuration.
      */
    ti_em_preload_config_t* preload_config_ptr;

    /**
      * Index for Packet DMA flows configuration. It has to map to a Multicore Navigator infrastructure Packet DMA.
      */
    ti_em_dma_id_t  dma_idx;

    /**
      * Index of the first DMA queue that will be used by the TI EM.
      */
    uint16_t dma_queue_base_idx;

    /**
      * Index of the 1st hardware queue that will be used by the TI EM.
      */
    uint16_t hw_queue_base_idx;

    /**
      * Hardware Semaphore used to protect internal data structures.
      */
    uint32_t hw_sem_idx;

    /**
     * @brief   Identifies the device on which this OpenEM instance is running.
     */
    ti_em_device_id_t my_device_idx;

    /**
     * @brief   Identifies the process in which this OpenEM instance is running.
     */
    ti_em_process_id_t my_process_idx;

    /**
      * Indexes of the pdsp threads running the FW scheduler.
      */
    ti_em_pdsp_id_t  pdsp_idx_tbl[TI_EM_SCHEDULER_THREAD_NUM];

    /**
      * pool configuration descriptors.
      */
    ti_em_pool_config_t pool_config_tbl[TI_EM_POOL_NUM];

    /**
     * Number of pools
     */
    uint32_t pool_num;

    /**
      * Hardware queue containing the private events used for Atomic Processing.
      * This queue shall contain at least 256 private events.
      */
    ti_em_queue_id_t ap_region_queue_idx;

    /**
      * Hardware queue containing the private events used for Command Processing.
      * This queue shall contain at least 32 private events.
      * This queue may be the same as ap_region_queue_idx.
      */
    ti_em_queue_id_t cd_region_queue_idx;

    /**
      * Number of pdsp instances running the FW scheduler.
      */
    uint8_t pdsp_num;

    /**
      * Identifies the interrupt base index on which the scheduler will notify dispatching.
      */
    ti_em_interrupt_id_t hw_interrupt_base_idx;

#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * Event machine chaining configuration.
 *
 * @details This describes the EM configurable parameters which are required for the chaining.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_chain_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_chain_config_t_, 64)
#endif
struct ti_em_chain_config_t_
{
    /**
      * Identifies the pool used for allocating very small events (event size <= size A).
      * Size A corresponds to the buffer size of the pool.
      * The received event uses one buffer from the pool.
      */
    em_pool_id_t pool_idx_size_a;

    /**
      * Identifies the pool used for allocating small events (size A <= event size <= size B).
      * Size B corresponds to the buffer size of the pool.
      * The received event uses one buffer from the pool.
      */
    em_pool_id_t pool_idx_size_b;

    /**
      * Identifies the pool used for allocating big events (size B <= event size <= size C).
      * Size C corresponds to the buffer size of the pool.
      * The received event uses one buffer from the pool.
      */
    em_pool_id_t pool_idx_size_c;

    /**
      * Identifies the pool used for allocating very big events (size C <= event size <= size D).
      * Size D corresponds to the buffer size of the pool.
      * The received event uses one buffer from the pool.
      */
    em_pool_id_t pool_idx_size_d;

    /**
      * Identifies the pool used for allocating even bigger events (size D <= event size).
      * The received event uses one buffer (first) from the pool 'pool_idx_size_d'.
      * The received event uses one or more buffers (second and next) from the pool 'pool_idx_overflow'.
      */
    em_pool_id_t pool_idx_overflow;

    /**
      * Pointer to the SRIO Configuration.
      */
    ti_em_chain_rio_config_t* rio_config_ptr;

    /**
      * Pointer to the 10 Gigabit Configuration.
      */
    ti_em_chain_xge_config_t* xge_config_ptr;

    /**
      * Pointer to the post-storing configuration.
      */
    ti_em_poststore_config_t* poststore_config_ptr;

    /**
      * Index for Packet DMA flows configuration. It has to map to a Multicore Navigator infrastructure Packet DMA.
      */
    ti_em_dma_id_t dma_idx;

    /**
      * Identifies the first relative DMA TX queue index used for the chaining flows.
      */
    ti_em_queue_id_t dma_tx_queue_base_idx;

#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * Event machine chaining configuration for xge.
 *
 * @details This describes the EM configurable parameters which are required for the chaining over xge.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_chain_xge_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_chain_xge_config_t_, 64)
#endif
struct ti_em_chain_xge_config_t_
{
    /**
      * Index for Packet DMA flows configuration. It has to map to a Multicore Navigator infrastructure Packet DMA.
      */
    ti_em_dma_id_t dma_idx;

    /**
      * Specifies the Ethernet type.
      */
    uint8_t ether_type[2];

    /**
      * Specifies the MAC address of the device on which this OpenEM instance is running.
      */
    uint8_t my_mac_address[6];

    /**
      * Identifies the free queue for RX fragments.
      */
    ti_em_queue_id_t rx_fragment_free_queue_idx;

    /**
      * Identifies the free queue for RX headers.
      */
    ti_em_queue_id_t rx_header_free_queue_idx;

    /**
     * Identifies the first of TI_EM_XGE_VLAN_PRIO_NUM XGE RX miss queues
     */
     ti_em_queue_id_t rx_miss_queue_base_idx;
    
    
    /**
      * Identifies the free queue for TX fragments (without buffers).
      */
    ti_em_queue_id_t tx_fragment_free_queue_idx;

    /**
      * Identifies the free queue for TX headers
      */
    ti_em_queue_id_t tx_header_free_queue_idx;

    /**
      * Identifies the base index for the OpenEM instance to allocate the XGE TX divert queue.
      */
    ti_em_queue_id_t tx_divert_queue_base_idx;

    /**
      * Identifies a PDSP running the chaining firmware.
      */
    ti_em_pdsp_id_t pdsp_idx;

    /**
      * Specifies the VLAN priority Mask.
      */
    uint8_t vlan_prio_mask;

    /**
     * Specifies the service type.
     */
    uint8_t service_type;
#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * Event machine chaining configuration for SRIO.
 *
 * @details This describes the EM configurable parameters which are required for the chaining over SRIO.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_chain_rio_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_chain_rio_config_t_, 64)
#endif
struct ti_em_chain_rio_config_t_
{
    /**
      * Index for Packet DMA flows configuration. It has to map to a Multicore Navigator infrastructure Packet DMA.
      */
    ti_em_dma_id_t dma_idx;

    /**
      * Index for SRIO tx queue.
      */
    ti_em_queue_id_t dma_tx_queue_idx;
    
    /**
      * Specifies the SRIO node index
      */
    uint8_t my_node_idx[2];
    
    /**
      * Specifies the Service Class
      */
    uint8_t service_class;
#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * Event machine preload configuration.
 *
 * @details This describes the EM configurable parameters which are required for the preload.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_preload_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_preload_config_t_, 64)
#endif
struct ti_em_preload_config_t_
{
    /**
      * Identifies the preload free queues.
      */
    ti_em_queue_id_t local_free_queue_idx_tbl[TI_EM_CORE_NUM];

    /**
      * Specifies the maximum number of bytes to preload if event type matches TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_A.
      * Must be more than 0 and less than or equal to 8Kbytes.
      */
    size_t preload_size_a;

    /**
      * Specifies the maximum number of bytes to preload if event type matches TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_B.
      * Must be more than 0 and less than or equal to 8Kbytes.
      */
    size_t preload_size_b;

    /**
      * Specifies the maximum number of bytes to preload if event type matches TI_EM_EVENT_TYPE_PRELOAD_ON_SIZE_B.
      * Must be more than 0 and less than or equal to 1Mbytes.
      */
    size_t preload_size_c;
#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * Event machine post-storing configuration.
 *
 * @details This describes the EM configurable parameters which are required for the post-storing.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (ti_em_poststore_config_t_, 64);
#pragma STRUCT_ALIGN (ti_em_poststore_config_t_, 64)
#endif
struct ti_em_poststore_config_t_
{
    /**
      * Identifies the post-storing free queues. Array entry allocations per core are (core index * 2) + 0 and (core index * 2) + 1.
      */
    ti_em_queue_id_t local_free_queue_idx_tbl[2 * TI_EM_CORE_NUM];

    /**
      * Identifies the post-storing heap pointers. Array entry allocations per core are (core index * 2) + 0 and (core index * 2) + 1.
      */
    void * local_heap_ptr_tbl[2 * TI_EM_CORE_NUM];

    /**
      * Specifies the maximum number of bytes to post-store.
      */
    size_t poststore_size_max;

#ifdef TI_EM_GCC
} __attribute__ ((aligned(CACHE_L1D_LINESIZE)));
#else /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @ingroup DOC_api_specific
 *
 * ...
 *
 * @details This describes the parameters required to configure a route over the 10 Gigabit Ethernet (XGE).
 */
struct ti_em_device_xge_route_t_
{
    /**
      * Specifies the size of an event fragment.
      */
    size_t fragmentSize;

    /**
      * Specifies the MAC address.
      */
    uint8_t mac_address[6];

    //acTicket302/**
    //acTicket302  * Specifies the service type.
    //acTicket302  */
    //acTicket302uint8_t service_type;

    /**
      * Specifies the VLAN tag.
      * Must match the VLAN priority .
      */
    uint8_t vlan_tag[4];
};

/**
 * @ingroup DOC_api_specific
 *
 * ...
 *
 * @details This describes the parameters required to configure a route over the Serial Rapid IO.
 */
struct ti_em_device_rio_route_t_
{
    /**
      * Specifies the node index.
      */
    uint8_t node_idx[2];

};

/**
 * @ingroup DOC_api_specific
 *
 * Event handle pair.
 *
 * @details This describes the event handle pair when combining/splitting events.
 *
 */
struct ti_em_pair_t_
{
    /**
      * Event machine head event in the pair.
      */
    em_event_t head_event_hdl;
    /**
      * Event machine tail event in the pair.
      */
    em_event_t tail_event_hdl;
};

/**
 * @ingroup DOC_api_specific
 *
 * Event handle pair.
 *
 * @details This describes the parameters required to configure a route to another process.
 *
 */
struct ti_em_process_route_t_
{
    /**
      * Index for Packet DMA flows configuration. It has to map to infrastructure Packet DMA.
      */
    ti_em_dma_id_t dma_idx;
    /**
      * Identifies the relative DMA TX queue index used for the chaining flow.
      */
    ti_em_queue_id_t dma_tx_queue_idx;
};

#endif /*EVENT_MACHINE_HW_TI_TYPES_H_*/


