/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 * 
 * From the original event_machine_hw_config.h.template , openem/event_machine_hw_types.h was
 * created with:
 *   - HW types definitions 
 *   - Doxygen tags are added for API documentation generation.
 *----------------------------------------------------------------------------------------------
 *
 */


/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 *
 * Elements of the Nokia Siemens Networks template files have been concatenated in this file and 
 * configured to match TI implementation.
 *----------------------------------------------------------------------------------------------
 */

 /**
 * @file: event_machine_hw_types.h
 *
 * @brief
 *    Open Event Machine HW specific types
 *
 */

#ifndef EVENT_MACHINE_HW_TYPES_H
#define EVENT_MACHINE_HW_TYPES_H



#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

/*
 *
 * HW specific types
 ***************************************
 */

/**
 * @ingroup DOC_api_generic
 *
 * The structure describes the queue types
 *
 */
typedef enum em_queue_type_e
{

    /**
     * @brief Application receives events one by one, non-concurrently to guarantee exclusive processing and ordering
     */
    EM_QUEUE_TYPE_UNDEF            = 0,

    /**
     * @brief Application receives events one by one, non-concurrently to guarantee exclusive processing and ordering
     */
    EM_QUEUE_TYPE_ATOMIC           = 1,  

    /**
     * @brief Application may receive events fully concurrently, egress event ordering (when processed in parallel) not guaranteed
     */
    EM_QUEUE_TYPE_PARALLEL         = 2,  

    /**
     * @brief Application may receive events concurrently, but system takes care of egress order (between two queues)
     */
    EM_QUEUE_TYPE_PARALLEL_ORDERED = 3   

} em_queue_type_e;

/**
 * @ingroup DOC_api_generic
 *
 * The structure describes the portable queue priorities
 *
 */
typedef enum em_queue_prio_e
{
    /**
     * @brief Undefined
     */
    
    EM_QUEUE_PRIO_UNDEF    = 0xFF, 
    /**
     * @brief lowest priority
     */
    EM_QUEUE_PRIO_LOWEST   =  0,    
    
    /**
     * @brief low priority
     */
     EM_QUEUE_PRIO_LOW     =  12,

     /**
     * @brief normal priority
     */
    EM_QUEUE_PRIO_NORMAL   =  13,
    
    /**
     * @brief high priority
     */
    EM_QUEUE_PRIO_HIGH     =  14,
    
    /**
     * @brief highest priority
     */
    EM_QUEUE_PRIO_HIGHEST  =  15

} em_queue_prio_e;

/**
 * @ingroup DOC_api_generic
 *
 * The structure lists the error codes
 *
 */
typedef enum em_status_e
{

    /**
     * @brief Illegal context for this function call
     */
    EM_ERR_BAD_CONTEXT      = 1, 

    /**
     * @brief Illegal (eo, queue, ...) state for this function call
     */
    EM_ERR_BAD_STATE        = 2,   

    /**
     * @brief ID not from a valid range
     */
    EM_ERR_BAD_ID           = 3,   

    /**
     * @brief Resource allocation failed
     */
    EM_ERR_ALLOC_FAILED     = 4, 

    /**
     * @brief Resource already reserved by someone else
     */
    EM_ERR_NOT_FREE         = 5,   

    /**
     * @brief Resource not found
     */
    EM_ERR_NOT_FOUND        = 6,   

    /**
     * @brief Value over the limit
     */
    EM_ERR_TOO_LARGE        = 7, 

    /**
     * @brief Failure in a library function
     */
    EM_ERR_LIB_FAILED       = 8,

    /**
     * @brief Implementation missing (placeholder)
     */
    EM_ERR_NOT_IMPLEMENTED  = 9,

    /**
     * @brief Pointer from bad memory area (e.g. NULL)
     */
    EM_ERR_BAD_POINTER      = 10,


    /**
     * @brief Other error. This is the last error code (for bounds checking).
     */
    EM_ERR        

} em_status_e;

/**
 * @ingroup DOC_api_generic
 *
 * The structure describes the major event types
 *
 */
typedef enum em_event_type_major_e
{

    /**
     * @brief
     */
    EM_EVENT_TYPE_UNDEF  = 0,

    /**
     * @brief event from SW (EO)
     */
    EM_EVENT_TYPE_SW     = 1 << 24,

    /**
     * @brief event from packet HW
     */
    EM_EVENT_TYPE_PACKET = 2 << 24, 

    /**
     * @brief event from timer HW
     */
    EM_EVENT_TYPE_TIMER  = 3 << 24, 

    /**
     * @brief event from crypto HW
     */
    EM_EVENT_TYPE_CRYPTO = 4 << 24 

} em_event_type_major_e;


/**
 * @ingroup DOC_api_generic
 *
 * The structure describes the Minor event types for the major EM_EVENT_TYPE_SW type
 *
 */
typedef enum em_event_type_sw_minor_e
{

    /**
     * @brief
     */
    EM_EVENT_TYPE_SW_DEFAULT  = 1

} em_event_type_sw_minor_e;

/**
 * @ingroup DOC_api_generic
 * 
 * Event type
 *
 * In Event Machine application processing is driven by events. An event 
 * describes a piece of work. Structure of an event is implementation and
 * event type specific. It may be a directly accessible buffer of memory, 
 * a descriptor containing a list of buffer pointers, a descriptor of 
 * a packet buffer, etc.
 *
 * Applications use event type to interpret the event structure.
 *
 * Since em_event_t may not carry a direct pointer value to the event
 * structure, em_event_pointer() must be used to translate an event to
 * an event structure pointer (for maintaining portability).
 *
 * @see em_event_pointer()

 */
typedef uint32_t em_event_t;


#ifdef __cplusplus
}
#endif


#endif

