/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 * 
 * Changes were done on the original NSN header files in order to avoid 
 * circular dependencies when called in the TI implementation files.
 *
 * From the original event_machine_types.h, openem/ti/event_machine_macros.h was
 * created with:
 *   - Constants definitions
 *   - Doxygen tags are added for API documentation generation.
 *----------------------------------------------------------------------------------------------
 *
 */


/**
 * @file: event_machine_macros.h
 *
 * @brief
 *    Open Event Machine public constant definitions
 */

#ifndef EVENT_MACHINE_MACROS_H_
#define EVENT_MACHINE_MACROS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <ti/runtime/openem/event_machine_hw_macros.h>

#ifdef EM_64_BIT

/*
 * Printf formats
 */
/**
 * @ingroup DOC_api_generic
 *
 * em_eo_t printf format 
 */
#define PRI_EO      PRIu64       

/**
 * @ingroup DOC_api_generic
 *
 * em_queue_t printf format 
 */
#define PRI_QUEUE   PRIu64

/**
 * @ingroup DOC_api_generic
 *
 * em_queue_group_t printf format 
 */
#define PRI_QGRP    PRIu64        


/**
 * @ingroup DOC_api_generic
 *
 * Invalid EO id 
 */
#define EM_EO_UNDEF             EM_UNDEF_U64    

/**
 * @ingroup DOC_api_generic
 *
 * Invalid queue 
 */
#define EM_QUEUE_UNDEF          EM_UNDEF_U64    

/**
 * @ingroup DOC_api_generic
 *
 * Invalid queue group 
 */
#define EM_QUEUE_GROUP_UNDEF    EM_UNDEF_U64    


#elif defined(EM_32_BIT)

/*
 * Printf formats
 */
/**
 * @ingroup DOC_api_generic
 *
 * em_eo_t printf format          
 */
#define PRI_EO       PRIu32                     

/**
 * @ingroup DOC_api_generic
 *
 * em_queue_t printf format       
 */
#define PRI_QUEUE    PRIu32                     

/**
 * @ingroup DOC_api_generic
 *
 * em_queue_group_t printf format 
 */
#define PRI_QGRP     PRIu32                     


/**
 * @ingroup DOC_api_generic
 *
 * Invalid EO id 
 */
#define EM_EO_UNDEF      EM_UNDEF_U32           

/**
 * @ingroup DOC_api_generic
 *
 * Invalid queue 
 */
#define EM_QUEUE_UNDEF   EM_UNDEF_U32           

/**
 * @ingroup DOC_api_generic
 *
 * Invalid queue group id 
 */
#define EM_QUEUE_GROUP_UNDEF     EM_UNDEF_U32   


#else

  #error Missing architecture definition. Define EM_64_BIT or EM_32_BIT!

#endif

/**
 * @ingroup DOC_api_generic
 *
 * Operation successful 
 */
#define EM_OK   0      

/**
 * @ingroup DOC_api_generic
 *
 * Operation not successful 
 */
#define EM_ERROR 0xffffffff

/**
 * @ingroup DOC_api_generic
 *
 * All EM internal error scopes should have bit 31 set
 * NOTE: High bit is RESERVED for EM internal escopes and should not be
 * used by the application.
 */
#define EM_ESCOPE_BIT                  (0x80000000u)

/**
 * @ingroup DOC_api_generic
 *
 * Test if the error scope identifies an EM function (API or other internal)
 */
#define EM_ESCOPE(escope)              (EM_ESCOPE_BIT & (escope))

/**
 * @ingroup DOC_api_generic
 *
 * Mask selects the high byte of the 32-bit escope
 */
#define EM_ESCOPE_MASK                 (0xFF000000)

/**
 * @ingroup DOC_api_generic
 *
 * EM API functions error scope type
 */
#define EM_ESCOPE_API_TYPE             (0xFFu)

/**
 * @ingroup DOC_api_generic
 *
 * EM API functions error scope mask
 */
#define EM_ESCOPE_API_MASK             (EM_ESCOPE_BIT | (EM_ESCOPE_API_TYPE << 24))

/**
 * @ingroup DOC_api_generic
 *
 * Test if the error scope identifies an API function
 */
#define EM_ESCOPE_API(escope)          (((escope) & EM_ESCOPE_MASK) == EM_ESCOPE_API_MASK)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_create API.
 *
 */
#define EM_ESCOPE_QUEUE_CREATE                    (EM_ESCOPE_API_MASK | 0x0001)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_create_static API.
 *
 */
#define EM_ESCOPE_QUEUE_CREATE_STATIC             (EM_ESCOPE_API_MASK | 0x0002)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_delete API.
 *
 */
#define EM_ESCOPE_QUEUE_DELETE                    (EM_ESCOPE_API_MASK | 0x0003)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_enable API.
 *
 */
#define EM_ESCOPE_QUEUE_ENABLE                    (EM_ESCOPE_API_MASK | 0x0004)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_enable_all API.
 *
 */
#define EM_ESCOPE_QUEUE_ENABLE_ALL                (EM_ESCOPE_API_MASK | 0x0005)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_disable API.
 *
 */
#define EM_ESCOPE_QUEUE_DISABLE                   (EM_ESCOPE_API_MASK | 0x0006)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_disable_all API.
 *
 */
#define EM_ESCOPE_QUEUE_DISABLE_ALL               (EM_ESCOPE_API_MASK | 0x0007)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_set_context API.
 *
 */
#define EM_ESCOPE_QUEUE_SET_CONTEXT               (EM_ESCOPE_API_MASK | 0x0008)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_get_context API.
 *
 */
#define EM_ESCOPE_QUEUE_GET_CONTEXT               (EM_ESCOPE_API_MASK | 0x0009)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_get_name API.
 *
 */
#define EM_ESCOPE_QUEUE_GET_NAME                  (EM_ESCOPE_API_MASK | 0x000A)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_get_priority API.
 *
 */
#define EM_ESCOPE_QUEUE_GET_PRIORITY              (EM_ESCOPE_API_MASK | 0x000B)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_get_type API.
 *
 */
#define EM_ESCOPE_QUEUE_GET_TYPE                  (EM_ESCOPE_API_MASK | 0x000C)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_create API.
 *
 */
#define EM_ESCOPE_QUEUE_GET_GROUP                 (EM_ESCOPE_API_MASK | 0x000D)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_create API.
 *
 */
#define EM_ESCOPE_QUEUE_GROUP_CREATE              (EM_ESCOPE_API_MASK | 0x0101)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_delete API.
 *
 */
#define EM_ESCOPE_QUEUE_GROUP_DELETE              (EM_ESCOPE_API_MASK | 0x0102)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_modify API.
 *
 */
#define EM_ESCOPE_QUEUE_GROUP_MODIFY              (EM_ESCOPE_API_MASK | 0x0103)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_find API.
 *
 */
#define EM_ESCOPE_QUEUE_GROUP_FIND                (EM_ESCOPE_API_MASK | 0x0104)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_queue_group_mask API.
 *
 */
#define EM_ESCOPE_QUEUE_GROUP_MASK                (EM_ESCOPE_API_MASK | 0x0105)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_create API.
 *
 */
#define EM_ESCOPE_EO_CREATE                       (EM_ESCOPE_API_MASK | 0x0201)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_delete API.
 *
 */
#define EM_ESCOPE_EO_DELETE                       (EM_ESCOPE_API_MASK | 0x0202)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_get_name API.
 *
 */
#define EM_ESCOPE_EO_GET_NAME                     (EM_ESCOPE_API_MASK | 0x0203)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_add_queue API.
 *
 */
#define EM_ESCOPE_EO_ADD_QUEUE                    (EM_ESCOPE_API_MASK | 0x0204)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_remove_queue API.
 *
 */
#define EM_ESCOPE_EO_REMOVE_QUEUE                 (EM_ESCOPE_API_MASK | 0x0205)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_register_error_handler API.
 *
 */
#define EM_ESCOPE_EO_REGISTER_ERROR_HANDLER       (EM_ESCOPE_API_MASK | 0x0206)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_unregister_error_handler API.
 *
 */
#define EM_ESCOPE_EO_UNREGISTER_ERROR_HANDLER     (EM_ESCOPE_API_MASK | 0x0207)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_start API.
 *
 */
#define EM_ESCOPE_EO_START                        (EM_ESCOPE_API_MASK | 0x0208)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_eo_stop API.
 *
 */
#define EM_ESCOPE_EO_STOP                         (EM_ESCOPE_API_MASK | 0x0209)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_core_id API.
 *
 */
#define EM_ESCOPE_CORE_ID                         (EM_ESCOPE_API_MASK | 0x0301)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_core_count API.
 *
 */
#define EM_ESCOPE_CORE_COUNT                      (EM_ESCOPE_API_MASK | 0x0302)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_alloc API.
 *
 */
#define EM_ESCOPE_ALLOC                           (EM_ESCOPE_API_MASK | 0x0401)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_free API.
 *
 */
#define EM_ESCOPE_FREE                            (EM_ESCOPE_API_MASK | 0x0402)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_send API.
 *
 */
#define EM_ESCOPE_SEND                            (EM_ESCOPE_API_MASK | 0x0403)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_atomic_processing_end API.
 *
 */
#define EM_ESCOPE_ATOMIC_PROCESSING_END           (EM_ESCOPE_API_MASK | 0x0404)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_register_error_handler API.
 *
 */
#define EM_ESCOPE_REGISTER_ERROR_HANDLER          (EM_ESCOPE_API_MASK | 0x0501)
/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_unregister_error_handler API.
 *
 */
#define EM_ESCOPE_UNREGISTER_ERROR_HANDLER        (EM_ESCOPE_API_MASK | 0x0502)

/**
 * @ingroup DOC_api_generic
 *
 * Error scope for the em_error API.
 *
 */
#define EM_ESCOPE_ERROR                           (EM_ESCOPE_API_MASK | 0x0503)

/**
 * @ingroup DOC_api_generic
 *
 * Size of the core mask in bits
 */
#define EM_CORE_MASK_SIZE  64

#ifdef __cplusplus
}
#endif

#endif /* EVENT_MACHINE_MACROS_H_ */

