/*
 *   Copyright (c) 2012, Nokia Siemens Networks
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Nokia Siemens Networks nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *
 * Contributors
 *
 * The initial developer of the original code is Nokia Siemens Networks.
 *  The following contributors have made changes to 
 *  the original code as described below.
 * 
 *----------------------------------------------------------------------------------------------
 * Copyright (c) 2012, Texas Instruments Incorporated. All Rights Reserved. - http://www.ti.com/
 * 
 * From the original event_machine_hw_config.h.template , openem/event_machine_sw_config.h was
 * created with:
 *   - SW macro definitions with the TI optimized implementation values
 *   - Doxygen tags are added for API documentation generation.
 *----------------------------------------------------------------------------------------------
 *
 */


/**
 * @file: event_machine_sw_config.h
 *
 * @brief
 *    Open Event Machine SW specific configuration
 *
 */

#ifndef EVENT_MACHINE_SW_CONFIG_H
#define EVENT_MACHINE_SW_CONFIG_H

#include <ti/runtime/openem/event_machine_hw_ti_macros.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 *
 * EM scaling
 ***************************************
 */



/**
 * @ingroup DOC_api_generic
 * 
 * Maximum number of queues supported by the event machine.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 *
 */
#define EM_MAX_QUEUES               (TI_EM_QUEUE_SET_NUM * TI_EM_QUEUE_NUM_IN_SET)

/**
 * @ingroup DOC_api_generic
 * 
 * Max queue name string length.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 *
 */
#define EM_QUEUE_NAME_LEN           (32)


/**
 * @ingroup DOC_api_generic
 * 
 * Maximum number of execution objects supported by the event machine. EM_MAX_EOS <= TI_EM_EO_NUM_MAX.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 *
 */
#define EM_MAX_EOS                  (256)


/**
 * @ingroup DOC_api_generic
 * 
 * Max EO name string lenght.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 */
#define EM_EO_NAME_LEN              (32)


/**
 * @ingroup DOC_api_generic
 * 
 * Maximum number of event groups supported by the event machine. EM_MAX_EVENT_GROUPS <= TI_EM_EVENT_GROUP_NUM_MAX.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 *
 */
#define EM_MAX_EVENT_GROUPS         (1024)


/**
 * @ingroup DOC_api_generic
 * 
 * Maximum number of event notifications supported by one event group.
 *
 */
#define EM_EVENT_GROUP_MAX_NOTIF    (8)


/**
 * @ingroup DOC_api_generic
 * 
 * Max queue group name string lenght.
 *
 * @note This value can be modified by an application. But the event machine library needs to be recompiled.
 */
#define EM_QUEUE_GROUP_NAME_LEN    (8u)






#ifdef __cplusplus
}
#endif


#endif
