LIBOPENEM_MODULES_DEVICE_K1= \
	../modules/chaining/src/event_machine_hw_ti_chain_pktdma.o \
	../modules/chaining/src/event_machine_hw_ti_chain_rio.o

LIBOPENEM_MODULES_DEVICE_K2K= \
	../modules/chaining/src/event_machine_hw_ti_chain_pktdma.o \
	../modules/chaining/src/event_machine_hw_ti_chain_xge.o \
	../modules/chaining/src/event_machine_hw_ti_chain_rio.o

LIBOPENEM_OBJS= \
	../src/event_machine.o	\
	../src/event_machine_chaining.o \
	../src/event_machine_dispatcher.o \
	../src/event_machine_error.o \
	../src/event_machine_event.o \
	../src/event_machine_event_group.o \
	../src/event_machine_execution_object.o \
	../src/event_machine_hw_ti_functions.o \
	../src/event_machine_init.o \
	../src/event_machine_exit.o \
	../src/event_machine_interrupt.o \
	../src/event_machine_packet.o \
	../src/event_machine_private_variables.o \
	../src/event_machine_queue.o \
	../src/event_machine_queue_group.o \
	../src/event_machine_scattered.o \
	../src/event_machine_send.o \
	$(LIBOPENEM_MODULES_$(MY_PLATFORM))

LIBOSAL_OBJS= \
	osal/src/ti_em_osal_queue.o \
	osal/src/ti_em_osal_cppi.o \
	osal/src/ti_em_osal_firmware.o \
	osal/src/ti_em_osal_uaccess.o \
	osal/src/ti_em_osal_core.o \
	osal/src/ti_em_osal_shm.o \
	osal/src/ti_em_osal_cma.o \
	osal/src/ti_em_osal_xge.o \
	osal/src/ti_em_osal_rio.o \
	osal/src/ti_em_osal_event.o \

LIBRH_OBJS= \
	rh/src/ti_em_rh_init.o \
	rh/src/ti_em_rh_cores.o \
	rh/src/ti_em_rh_join.o \
	rh/src/ti_em_rh_dispatch.o

TOOLS_OBJS= tools/src/get_pending_desc.o

EXAMPLE0_OBJS= \
	examples/Example_0/src/main.o \
	examples/Example_0/src/my_em_init.o \
	examples/Example_0/src/my_receive.o

EXAMPLE1_OBJS= \
	examples/Example_1/src/main.o \
	examples/Example_1/src/my_em_init.o \
	examples/Example_1/src/my_receive.o

EXAMPLE2_OBJS= \
	examples/Example_2/src/main.o \
	examples/Example_2/src/my_em_init.o \
	examples/Example_2/src/my_receive.o

EXAMPLE3_OBJS= \
	examples/Example_3/src/main.o \
	examples/Example_3/src/my_em_init.o \
	examples/Example_3/src/my_receive.o

all: static dynamic tools examples install

target_dir:
	mkdir -p ../lib/$(TARGET_DIR)

static: target_dir ../lib/$(TARGET_DIR)/libopenem.a  ../lib/$(TARGET_DIR)/libopenem_rh.a ../lib/$(TARGET_DIR)/libopenem_osal.a

dynamic: target_dir ../lib/$(TARGET_DIR)/libopenem.so

../lib/$(TARGET_DIR)/libopenem.a: $(LIBOPENEM_OBJS)
	echo "  AR    $@"
	$(AR) r $@ $(LIBOPENEM_OBJS)
	echo "  RL    $@"
	$(RANLIB) $@

../lib/$(TARGET_DIR)/libopenem.so: $(LIBOPENEM_OBJS)
	echo "  LD    $@"
	$(CC) -shared -Wl,-soname,libopenem.so $(LDFLAGS) -o $@ $(LIBOPENEM_OBJS)

../lib/$(TARGET_DIR)/libopenem_rh.a: $(LIBRH_OBJS)
	echo "  AR    $@"
	$(AR) r $@ $(LIBRH_OBJS)
	echo "  RL    $@"
	$(RANLIB) $@

../lib/$(TARGET_DIR)/libopenem_osal.a: $(LIBOSAL_OBJS)
	echo "  AR    $@"
	$(AR) r $@ $(LIBOSAL_OBJS)
	echo "  RL    $@"
	$(RANLIB) $@

.c.o:
	echo "  CC    $<"
	$(CC) $(CFLAGS) $(OFLAGS) -c $< -o $@

install: static dynamic tools examples
	echo "  INSTALLING IN $(ROOT_DIR)"
	mkdir -p $(ROOT_DIR)/usr/lib/ $(ROOT_DIR)/opt/openem/examples $(ROOT_DIR)/opt/openem/tools
	cp ../lib/$(TARGET_DIR)/libopenem.so $(ROOT_DIR)/usr/lib/
	cp tools/$(TARGET_DIR)/get_pending_desc $(ROOT_DIR)/opt/openem/tools/
	cp examples/$(TARGET_DIR)/Example_* $(ROOT_DIR)/opt/openem/examples/
	cp set_interrupt_affinity.sh $(ROOT_DIR)/opt/openem/
	chmod 755 $(ROOT_DIR)/opt/openem/set_interrupt_affinity.sh

clean:
	rm -f $(LIBOPENEM_OBJS) $(LIBRH_OBJS) $(LIBOSAL_OBJS) ../lib/$(TARGET_DIR)/*.a ../lib/$(TARGET_DIR)/*.so
	rm -f  $(TOOLS_OBJS) tools/$(TARGET_DIR)/get_pending_desc
	rm -f  $(EXAMPLE0_OBJS) $(EXAMPLE1_OBJS) $(EXAMPLE2_OBJS) $(EXAMPLE3_OBJS) examples/$(TARGET_DIR)/Example_*

tools_target_dir:
	mkdir -p tools/$(TARGET_DIR)

tools: static dynamic tools_target_dir \
	tools/$(TARGET_DIR)/get_pending_desc

tools/$(TARGET_DIR)/get_pending_desc: $(TOOLS_OBJS)
	echo "  LD    $@"
	$(CC) $(LDFLAGS) -o $@ $(TOOLS_OBJS) -L../lib/$(TARGET_DIR) \
	-Wl,--whole-archive -lopenem_osal -Wl,--no-whole-archive

example_target_dir:
	mkdir -p examples/$(TARGET_DIR)

examples: static dynamic example_target_dir \
	examples/$(TARGET_DIR)/Example_0 \
	examples/$(TARGET_DIR)/Example_1 \
	examples/$(TARGET_DIR)/Example_2 \
	examples/$(TARGET_DIR)/Example_3

examples/$(TARGET_DIR)/Example_0: $(EXAMPLE0_OBJS)
	echo "  LD    $@"
	$(CC) $(LDFLAGS) -o $@ $(EXAMPLE0_OBJS) -L../lib/$(TARGET_DIR) \
	-Wl,--whole-archive -lopenem_osal -lopenem_rh -Wl,--no-whole-archive \
	-lopenem

examples/$(TARGET_DIR)/Example_1: $(EXAMPLE1_OBJS)
	echo "  LD    $@"
	$(CC) $(LDFLAGS) -o $@ $(EXAMPLE1_OBJS) -L../lib/$(TARGET_DIR) \
	-Wl,--whole-archive -lopenem_osal -lopenem_rh -Wl,--no-whole-archive \
	-lopenem

examples/$(TARGET_DIR)/Example_2: $(EXAMPLE2_OBJS)
	echo "  LD    $@"
	$(CC) $(LDFLAGS) -o $@ $(EXAMPLE2_OBJS) -L../lib/$(TARGET_DIR) \
	-Wl,--whole-archive -lopenem_osal -lopenem_rh -Wl,--no-whole-archive \
	-lopenem

examples/$(TARGET_DIR)/Example_3: $(EXAMPLE3_OBJS)
	echo "  LD    $@"
	$(CC) $(LDFLAGS) -o $@ $(EXAMPLE3_OBJS) -L../lib/$(TARGET_DIR) \
	-Wl,--whole-archive -lopenem_osal -lopenem_rh -Wl,--no-whole-archive \
	-lopenem

mod:
	make -C modules/ all MY_PLATFORM=$(MY_PLATFORM)

mod_install:
	make -C modules/ install

mod_clean:
	make -C modules/ clean

ifneq ($(TEST_DIR),)
include $(TEST_DIR)/Test.mak
endif
