/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef _MY_RECEIVE_H_
#define _MY_RECEIVE_H_

/* importing some of the Open EM defined symbols */
#include "my_event_machine.h"

#include <stdint.h>
#include <stdlib.h>

typedef struct my_job_desc_t_
{
	uint32_t job_idx;
	uint32_t flow_idx;
	uint32_t data_size;
	uint32_t success_flag;
	uint32_t core_idx;
	uint64_t start_time;
	uint64_t stop_time;
} my_job_desc_t;

extern volatile my_job_desc_t *my_sv_job_desc_tbl;
extern volatile uint32_t      *my_sv_running_job_num;
extern volatile uint32_t      *my_sv_received_events;

int my_init_jobs(void);
int my_source_jobs(unsigned long delay);
void my_process_job(
	void*           ctx,
	em_event_t      event,
	em_event_type_t event_type,
	em_queue_t      queue,
	void*           queue_ctx);
void my_sink_job(
	void*           ctx,
	em_event_t      event,
	em_event_type_t event_type,
	em_queue_t      queue,
	void*           queue_ctx);
void my_stats_jobs(int status_display);

#define MY_FLOW_NUM         (32)
#define MY_JOB_NUM_PER_FLOW (32)
#define MY_JOB_NUM          (MY_FLOW_NUM * MY_JOB_NUM_PER_FLOW)

#endif /* _MY_RECEIVE_H_ */
