/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_hw_functions.h
 *
 *   @brief
 *      Machine platform functions
 *
 */

#ifndef TI_EM_HW_FUNCTIONS_H_
#define TI_EM_HW_FUNCTIONS_H_

/* 
 * This is for compatibility purpose, DNUM should be removed from OpenEM in the near
 * future.
 */
#define DNUM         (ti_em_osal_get_core_idx())

/* Temporary intrinsic definition, must be written in proper assembly per arch */
#define _itoll(h, l) (((uint64_t)(h) << 32) | (uint64_t)(l))
#define _loll(ll)    ((uint32_t)((ll) & 0xFFFFFFFFULL))
#define _hill(ll)    ((uint32_t)(((ll) >> 32) & 0xFFFFFFFFULL))
#define _mem8(i)     (*(uint64_t*)(i))

#ifdef TI_EM_C66X
#define TI_EM_HW_HAS_FAST_TSC
#define TI_EM_HW_INIT_TSC()  0
#define TI_EM_HW_GET_TSC()   ti_em_get_hw_c66x_tsc()

static inline uint64_t ti_em_get_hw_c66x_tsc(void) {
	unsigned int l, h;
	__asm__ __volatile__  (" dint\n"
			       " mvc .s2 TSCL,%0\n"
			       " mvc .s2 TSCH,%1\n"
			       " rint\n"
			       : "=b"(l), "=b"(h));
	return ((uint64_t) h << 32) | l;
}

static inline unsigned long _ffz(int val)
{
	__asm__ __volatile__ (" bitr  .M1  %0,%0\n"
			      " nop\n"
			      " lmbd  .L1  0,%0,%0\n"
			      : "+a"(val));
	return val;
}

#define	TI_EM_ARCH_MEM_BARRIER() __asm__ __volatile__ (" .word 0x10008000\n" \
						       " .word 0x10008000\n")
#endif /* TI_EM_C66X */

#ifdef TI_EM_ARM_A15
#include <ti_em_osal_cma.h>
#define TI_EM_HW_HAS_FAST_TSC
#define TI_EM_HW_INIT_TSC()  ti_em_osal_cma_enable_user_pmu()
#define TI_EM_HW_GET_TSC()   ti_em_get_hw_arm_pmccntr()

static inline uint64_t ti_em_get_hw_arm_pmccntr(void) {
	unsigned int val;

	/* read PMCCNTR */
        asm volatile("mrc p15, 0, %0, c9, c13, 0" : "=r" (val));
        return (uint64_t) val;
}

static inline int _ffz(int val)
{
        int bit;
	val = ~val;
        __asm__ __volatile__ ("clz %0, %1" : "=r" (bit) : "r" (val & -val) : "cc");
        return (31 - bit);
}

#define	TI_EM_ARCH_MEM_BARRIER() __asm__ __volatile__ ("dsb" : : : "memory")

#endif /* TI_EM_ARM_A15 */
#endif /* TI_EM_HW_FUNCTIONS_H_ */
