/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_hw_mach.h
 *
 *   @brief
 *      Machine platform definitions
 *
 */

#ifndef TI_EM_HW_MACH_H_
#define TI_EM_HW_MACH_H_

/*
 * Architecture definitions
 */
#ifdef TI_EM_ARM_A15
#define TI_EM_COHERENT_IO                            /* I/O are coherent */     
#undef  TI_EM_USE_HUGETLB                            /* use hugeTLB feature (not yet available) */
#define CACHE_L1D_LINESIZE                           (64)   /* size of L1D cache line for alignment purpose */
#define CACHE_L2_LINESIZE                            (64)   /* size of L2 cache line for alignment purpose */
#define QMSS_GENERAL_PURPOSE_USER_QUEUE_BASE         (4096) /* first queue available and not used by Linux */
#endif /* TI_EM_ARM_A15 */

#ifdef TI_EM_C66X
#define CACHE_L1D_LINESIZE                           (64)   /* size of L1D cache line for alignment purpose */
#define CACHE_L2_LINESIZE                            (128)  /* size of L2 cache line for alignment purpose */
#define QMSS_GENERAL_PURPOSE_USER_QUEUE_BASE         (4096) /* first queue available and not used by Linux */
#endif /* TI_EM_C66X */

/* Linux MMU page size */
#define PAGE_SIZE                                    (4096)

/* This is the physical memory range that will be mapped as I/O in the virtual user address space*/
#define TI_EM_IO_CFG_BASE                            (0x02000000)
#define TI_EM_IO_CFG_SIZE                            (0x01000000)

/*
 * KeyStone 2 platform definitions
 */
#define CSL_QMSS_CFG_QM_1_CFG_REGS                   (0x02A02000)
#define CSL_QMSS_CFG_QM_1_DESCRIPTOR_REGS            (0x02A02000 + 0x1000)
#define CSL_QMSS_CFG_QM_2_CFG_REGS                   (0x02A04000)
#define CSL_QMSS_CFG_QM_2_DESCRIPTOR_REGS            (0x02A04000 + 0x1000)
#define CSL_QMSS_CFG_QM_1_STATUS_RAM                 (0x02A06000)
#define CSL_QMSS_CFG_QM_2_STATUS_RAM                 (0x02A06400)
#define CSL_QMSS_CFG_PKTDMA_1                        (0x02A08000)
#define CSL_QMSS_CFG_PKTDMA_1_GLOBAL_CFG_REGS        (0x02A08000)
#define CSL_QMSS_CFG_PKTDMA_1_TX_CFG_REGS            (0x02A08000 + 0x400)
#define CSL_QMSS_CFG_PKTDMA_1_RX_CFG_REGS            (0x02A08000 + 0x800)
#define CSL_QMSS_CFG_PKTDMA_1_TX_SCHEDULER_CFG_REGS  (0x02A08000 + 0xC00)
#define CSL_QMSS_CFG_PKTDMA_1_RX_FLOW_CFG_REGS       (0x02A08000 + 0x1000)
#define CSL_QMSS_CFG_PKTDMA_2                        (0x02A0A000)
#define CSL_QMSS_CFG_PKTDMA_2_GLOBAL_CFG_REGS        (0x02A0A000)
#define CSL_QMSS_CFG_PKTDMA_2_TX_CFG_REGS            (0x02A0A000 + 0x400)
#define CSL_QMSS_CFG_PKTDMA_2_RX_CFG_REGS            (0x02A0A000 + 0x800)
#define CSL_QMSS_CFG_PKTDMA_2_TX_SCHEDULER_CFG_REGS  (0x02A0A000 + 0xC00)
#define CSL_QMSS_CFG_PKTDMA_2_RX_FLOW_CFG_REGS       (0x02A0A000 + 0x1000)
#define CSL_QMSS_CFG_INTD_1_REGS                     (0x02A0C000)
#define CSL_QMSS_CFG_INTD_2_REGS                     (0x02A0D000)
#define CSL_QMSS_CFG_PDSP1_REGS                      (0x02A0F000)
#define CSL_QMSS_CFG_PDSP2_REGS                      (0x02A0F000 + 0x100)
#define CSL_QMSS_CFG_PDSP3_REGS                      (0x02A0F000 + 0x200)
#define CSL_QMSS_CFG_PDSP4_REGS                      (0x02A0F000 + 0x300)
#define CSL_QMSS_CFG_PDSP5_REGS                      (0x02A0F000 + 0x400)
#define CSL_QMSS_CFG_PDSP6_REGS                      (0x02A0F000 + 0x500)
#define CSL_QMSS_CFG_PDSP7_REGS                      (0x02A0F000 + 0x600)
#define CSL_QMSS_CFG_PDSP8_REGS                      (0x02A0F000 + 0x700)
#define CSL_QMSS_CFG_PDSP1_IRAM                      (0x02A10000)
#define CSL_QMSS_CFG_PDSP2_IRAM                      (0x02A10000 + 0x1000)
#define CSL_QMSS_CFG_PDSP3_IRAM                      (0x02A10000 + 0x2000)
#define CSL_QMSS_CFG_PDSP4_IRAM                      (0x02A10000 + 0x3000)
#define CSL_QMSS_CFG_PDSP5_IRAM                      (0x02A10000 + 0x4000)
#define CSL_QMSS_CFG_PDSP6_IRAM                      (0x02A10000 + 0x5000)
#define CSL_QMSS_CFG_PDSP7_IRAM                      (0x02A10000 + 0x6000)
#define CSL_QMSS_CFG_PDSP8_IRAM                      (0x02A10000 + 0x7000)
#define CSL_QMSS_CFG_PDSP1_SRAM                      (0x02A20000)
#define CSL_QMSS_CFG_PDSP2_SRAM                      (0x02A20000 + 0x4000)
#define CSL_QMSS_CFG_PDSP3_SRAM                      (0x02A20000 + 0x8000)
#define CSL_QMSS_CFG_PDSP4_SRAM                      (0x02A20000 + 0xC000)
#define CSL_QMSS_CFG_PDSP5_SRAM                      (0x02A20000 + 0x10000)
#define CSL_QMSS_CFG_PDSP6_SRAM                      (0x02A20000 + 0x14000)
#define CSL_QMSS_CFG_PDSP7_SRAM                      (0x02A20000 + 0x18000)
#define CSL_QMSS_CFG_PDSP8_SRAM                      (0x02A20000 + 0x1C000)
#define CSL_QMSS_CFG_QM_1_QUEUE_PEEK_REGS            (0x02A40000)
#define CSL_QMSS_CFG_QM_2_QUEUE_PEEK_REGS            (0x02A60000)
#define CSL_QMSS_CFG_QM_1_QUEUE_MANAGEMENT_REGS      (0x02A80000)
#define CSL_QMSS_CFG_QM_2_QUEUE_MANAGEMENT_REGS      (0x02AA0000)
#define CSL_QMSS_CFG_QM_1_QUEUE_PROXY_REGS           (0x02AC0000)
#define CSL_QMSS_CFG_QM_2_QUEUE_PROXY_REGS           (0x02AE0000)
#define CSL_QMSS_LINKING_RAM                         (0x02B00000)

#define CSL_SRIO_CFG_PKTDMA_GLOBAL_CFG_REGS          (0x02900000 + 0x1000)
#define CSL_SRIO_CFG_PKTDMA_TX_CFG_REGS              (0x02900000 + 0x1400)
#define CSL_SRIO_CFG_PKTDMA_RX_CFG_REGS              (0x02900000 + 0x1800)
#define CSL_SRIO_CFG_PKTDMA_TX_SCHEDULER_CFG_REGS    (0x02900000 + 0x1C00)
#define CSL_SRIO_CFG_PKTDMA_RX_FLOW_CFG_REGS         (0x02900000 + 0x2000)

#define CSL_NETCP_CFG_PKTDMA_GLOBAL_CFG_REGS         (0x02000000 + 0x4000)
#define CSL_NETCP_CFG_PKTDMA_TX_CFG_REGS             (0x02000000 + 0x4400)
#define CSL_NETCP_CFG_PKTDMA_RX_CFG_REGS             (0x02000000 + 0x4800)
#define CSL_NETCP_CFG_PKTDMA_TX_SCHEDULER_CFG_REGS   (0x02000000 + 0x4C00)
#define CSL_NETCP_CFG_PKTDMA_RX_FLOW_CFG_REGS        (0x02000000 + 0x5000)

#define CSL_XGE_CFG_PKTDMA_GLOBAL_CFG_REGS           (0x02F00000 + 0xA1000)
#define CSL_XGE_CFG_PKTDMA_TX_CFG_REGS               (0x02F00000 + 0xA1400)
#define CSL_XGE_CFG_PKTDMA_RX_CFG_REGS               (0x02F00000 + 0xA1800)
#define CSL_XGE_CFG_PKTDMA_TX_SCHEDULER_CFG_REGS     (0x02F00000 + 0xA1C00)
#define CSL_XGE_CFG_PKTDMA_RX_FLOW_CFG_REGS          (0x02F00000 + 0xA2000)

/*
 * Our QMSS defines and aliases
 */
#define QMSS_OFFSET(i)                               ((CSL_QMSS_CFG_QM_2_CFG_REGS - CSL_QMSS_CFG_QM_1_CFG_REGS) * (i))

#define TI_EM_QMSS_CFG_STARVATION_REGS(i)            (CSL_QMSS_CFG_QM_1_CFG_REGS + QMSS_OFFSET(i))
#define TI_EM_QMSS_CFG_DESCRIPTOR_REGS(i)            (CSL_QMSS_CFG_QM_1_DESCRIPTOR_REGS + QMSS_OFFSET(i))
#define TI_EM_QMSS_CFG_QUEUE_MGT_REGS                CSL_QMSS_CFG_QM_1_QUEUE_MANAGEMENT_REGS
#define TI_EM_QMSS_CFG_DEQUEUE_REGS                  CSL_QMSS_CFG_QM_1_QUEUE_PROXY_REGS
#define TI_EM_QMSS_CFG_QUEUE_PEEK_REGS               CSL_QMSS_CFG_QM_1_QUEUE_PEEK_REGS

/*
 * PDSP firmwares
 */
#define TI_EM_FIRMWARE_SCHED_PDSP1                   em_scheduler_router_fw_c663x
#define TI_EM_FIRMWARE_SCHED_PDSP2                   em_scheduler_router_fw_c663x
#define TI_EM_FIRMWARE_ROUTER_PDSP                   em_router_fw_c663x

/*
 * PDSP DRAM
 *
 * This is located at offset 0x20000 and 0x24000 of the QMSS data space
 *
 */
#define TI_EM_PDSP_DRAM_SIZE                         (0x00004000)
#define TI_EM_PDSP_DRAM(i)                           (0x23a20000 + (i) * TI_EM_PDSP_DRAM_SIZE)
#define TI_EM_PDSPSH_DRAM                            (0x23a1c000)
#define TI_EM_PDSP_DRAM_OFFSET                       (0x0)        /* no offset, CPU and QMSS/PDSP mapping is same */

#endif /*TI_EM_HW_MACH_H_*/
