/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_hw_qmss.h
 *
 *   @brief   
 *      This is the Queue Manager module include file.
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2009, 2013 Texas Instruments, Inc.
 *  @n   Use of this software is controlled by the terms and conditions found 
 *  @n   in the license agreement under which this software has been supplied.
 *  ============================================================================ 
 *  \par
 */

#ifndef TI_EM_HW_QMSS_H_
#define TI_EM_HW_QMSS_H_

/** 
 * @brief Queue start number and maximum number of each type of queue supported.
 */
#define QMSS_LOW_PRIORITY_QUEUE_BASE                0
#define QMSS_MAX_LOW_PRIORITY_QUEUE               512

#define QMSS_AIF_QUEUE_BASE                       512
#define QMSS_MAX_AIF_QUEUE                        128
#define QMSS_PASS_QUEUE_BASE                      640
#define QMSS_MAX_PASS_QUEUE                         9

/* Broadcast to CPINTC 0/1/2 */
#define QMSS_INTC_SET1_QUEUE_BASE                 652
#define QMSS_MAX_INTC_SET1_QUEUE                    6
/* For backward compatibility */
#define QMSS_INTC_QUEUE_BASE     QMSS_INTC_SET1_QUEUE_BASE
#define QMSS_MAX_INTC_QUEUE      QMSS_MAX_INTC_SET1_QUEUE

/* Broadcast to CPINTC 0/1/2, GIC, EDMA 4 and Hyperlink */
#define QMSS_INTC_SET2_QUEUE_BASE                 658
#define QMSS_MAX_INTC_SET2_QUEUE                    8
/* Broadcast to CPINTC 0/1/2 */
#define QMSS_INTC_SET3_QUEUE_BASE                 666
#define QMSS_MAX_INTC_SET3_QUEUE                    6

#define QMSS_SRIO_QUEUE_BASE                      672
#define QMSS_MAX_SRIO_QUEUE                        16
#define QMSS_FFTC_A_QUEUE_BASE                    688
#define QMSS_MAX_FFTC_A_QUEUE                       4
#define QMSS_FFTC_B_QUEUE_BASE                    692
#define QMSS_MAX_FFTC_B_QUEUE                       4

#define QMSS_HIGH_PRIORITY_QUEUE_BASE             704
#define QMSS_MAX_HIGH_PRIORITY_QUEUE               32
#define QMSS_STARVATION_COUNTER_QUEUE_BASE        736
#define QMSS_MAX_STARVATION_COUNTER_QUEUE          64

#define QMSS_QM1_INFRASTRUCTURE_DMA_QUEUE_BASE    800
#define QMSS_MAX_QM1_INFRASTRUCTURE_DMA_QUEUE      32
/* For backward compatibility */
#define QMSS_INFRASTRUCTURE_QUEUE_BASE   QMSS_QM1_INFRASTRUCTURE_DMA_QUEUE_BASE
#define QMSS_MAX_INFRASTRUCTURE_QUEUE    QMSS_MAX_QM1_INFRASTRUCTURE_DMA_QUEUE

#define QMSS_TRAFFIC_SHAPING_QUEUE_BASE           832
#define QMSS_MAX_TRAFFIC_SHAPING_QUEUE             32

#define QMSS_BCP_QUEUE_BASE                       864
#define QMSS_MAX_BCP_QUEUE                          8
#define QMSS_FFTC_C_QUEUE_BASE                    872
#define QMSS_MAX_FFTC_C_QUEUE                       4
#define QMSS_FFTC_D_QUEUE_BASE                    876
#define QMSS_MAX_FFTC_D_QUEUE                       4
#define QMSS_FFTC_E_QUEUE_BASE                    880
#define QMSS_MAX_FFTC_E_QUEUE                       4
#define QMSS_FFTC_F_QUEUE_BASE                    884
#define QMSS_MAX_FFTC_F_QUEUE                       4

#define QMSS_GENERAL_PURPOSE_QUEUE_BASE           896
#define QMSS_MAX_GENERAL_PURPOSE_QUEUE           7296

#define QMSS_QM2_LOW_PRIORITY_QUEUE_BASE         8192
#define QMSS_MAX_QM2_LOW_PRIORITY_QUEUE           512

#define QMSS_GIC400_QUEUE_BASE                   8704
#define QMSS_MAX_GIC400_QUEUE                      32
#define QMSS_EDMA4_QUEUE_BASE                    8736
#define QMSS_MAX_EDMA4_QUEUE                        8
#define QMSS_BCAST_HYPERLINK_0_1_QUEUE_BASE      8744
#define QMSS_MAX_BCAST_HYPERLINK_0_1_QUEUE          8
#define QMSS_XGE_QUEUE_BASE                      8752
#define QMSS_MAX_XGE_QUEUE                          8
#define QMSS_DXB_QUEUE_BASE                      8836
#define QMSS_DXB_XGE_QUEUE                          8

#define QMSS_HYPERLINK_0_QUEUE_BASE              8760
#define QMSS_MAX_HYPERLINK_0_QUEUE                 16

/* Broadcast to CPINTC 0/1/2 */                  
#define QMSS_INTC_SET4_QUEUE_BASE                8844
#define QMSS_MAX_INTC_SET4_QUEUE                   20
                                                 
#define QMSS_HYPERLINK_1_QUEUE_BASE              8864
#define QMSS_MAX_HYPERLINK_1_QUEUE                 16

#define QMSS_QM2_INFRASTRUCTURE_DMA_QUEUE_BASE   8992
#define QMSS_MAX_QM2_INFRASTRUCTURE_DMA_QUEUE      32

#define QMSS_QM2_GENERAL_PURPOSE_QUEUE_BASE      9024
#define QMSS_MAX_QM2_GENERAL_PURPOSE_QUEUE       7360

/** 
 * @brief Queue Type. Specifies different queue classifications
 */
typedef enum
{
    /** Low priority queue */
    Qmss_QueueType_LOW_PRIORITY_QUEUE = 0,
    /** AIF queue */
    Qmss_QueueType_AIF_QUEUE,
    /** PASS queue */
    Qmss_QueueType_PASS_QUEUE,
    /** INTC pending queue */
    Qmss_QueueType_INTC_QUEUE,
    /** SRIO queue */
    Qmss_QueueType_SRIO_QUEUE,
    /** FFTC queue A */
    Qmss_QueueType_FFTC_A_QUEUE,
    /** FFTC queue B */
    Qmss_QueueType_FFTC_B_QUEUE,
    /** FFTC queue C */
    Qmss_QueueType_FFTC_C_QUEUE,
    /** FFTC queue D */
    Qmss_QueueType_FFTC_D_QUEUE,
    /** FFTC queue E */
    Qmss_QueueType_FFTC_E_QUEUE,
    /** FFTC queue F */
    Qmss_QueueType_FFTC_F_QUEUE,
    /** BCP queue */
    Qmss_QueueType_BCP_QUEUE,
    /** High priority queue */
    Qmss_QueueType_HIGH_PRIORITY_QUEUE,
    /** starvation counter queue */
    Qmss_QueueType_STARVATION_COUNTER_QUEUE,
    /** Infrastructure queue first QM */
    Qmss_QueueType_INFRASTRUCTURE_QUEUE,
    /** Infrastructure queue second QM */
    Qmss_QueueType_QM2_INFRASTRUCTURE_QUEUE,
    /** Traffic shaping queue */
    Qmss_QueueType_TRAFFIC_SHAPING_QUEUE,
    /** GIC400 queue */
    Qmss_QueueType_GIC400_QUEUE,
    /** EDMA 4 queue */
    Qmss_QueueType_EDMA_4_QUEUE,
    /** Broadcast to Hyperlink 0 and 1 queue */
    Qmss_QueueType_HLINK_BROADCAST_QUEUE,
    /** Hyperlink 0 queue */
    Qmss_QueueType_HLINK_0_QUEUE,
    /** Hyperlink 1 queue */
    Qmss_QueueType_HLINK_1_QUEUE,
    /** XGE (10 gigabit ethernet) queue */
    Qmss_QueueType_XGE_QUEUE,
    /** DXB queue */
    Qmss_QueueType_DXB_QUEUE,
    /** General purpose queue -- MUST be last */
    Qmss_QueueType_GENERAL_PURPOSE_QUEUE
}Qmss_QueueType;

/**
@}
*/
/** QMSS maximum number of memory regions */
#define QMSS_MAX_MEM_REGIONS                        64
/** QMSS maximum number of PDSPS */
#define QMSS_MAX_PDSP                               8
/** QMSS maximum number of INTDs */
#define QMSS_MAX_INTD                               2
/** QMSS maximum number of queue groups (QMGRs that run independently) */
#define QMSS_MAX_QMGR_GROUPS                        2
/** QMSS maximum number of queue types (per group) */
#define QMSS_MAX_QUEUE_TYPES                        30
/** QMSS maximum number of queues */
#define QMSS_MAX_QUEUES                             16384

/** 
 * @brief QMSS RM Handle
 */
typedef void *  Qmss_RmHnd;

/** 
 * @brief QMSS start configuration structure
 */
typedef struct
{
    /** Provide a handle to the Resource Manager instance */
    Qmss_RmHnd rmHandle;
} Qmss_StartCfg;

/** 
 * @brief Queue definition
 */
typedef struct
{
    /** Queue manager number */
    int32_t qMgr;        
    /** Queue number within Queue Manager */
    int32_t qNum;
}Qmss_Queue;

/** 
 * @brief Queue definition
 */
typedef struct
{
    /** Queue type */
    Qmss_QueueType queueType;
    /** Queue manager number */
    int32_t        startIndex;        
    /** Queue number within Queue Manager */
    int32_t        maxNum;
}Qmss_QueueNumRange;

/** 
 * @brief QMSS Global register definition for registers that replicate
 * per QM group
 */
typedef struct 
{
    /** QM Global Config registers */
    CSL_Qm_configRegs                       *qmConfigReg;
    /** QM Descriptor Config registers */
    CSL_Qm_descriptor_region_configRegs     *qmDescReg;
    /** QM queue Management registers, accessed via CFG port */
    CSL_Qm_queue_managementRegs             *qmQueMgmtReg;
    /** QM queue Management Proxy registers, accessed via CFG port */
    CSL_Qm_queue_managementRegs             *qmQueMgmtProxyReg;
    /** QM queue status registers */
    CSL_Qm_queue_status_configRegs          *qmQueStatReg;
    /** QM Status RAM */
    CSL_Qm_Queue_Status                     *qmStatusRAM;
    /** QM queue Management registers, accessed via DMA port */
    CSL_Qm_queue_managementRegs             *qmQueMgmtDataReg;
    /** QM queue Management Proxy registers, accessed via DMA port */
    CSL_Qm_queue_managementRegs             *qmQueMgmtProxyDataReg;
} Qmss_GlobalConfigGroupRegs;

/** 
 * @brief QMSS Global register definition for registers that do not replicate
 * per QM group
 */
typedef struct 
{
    /** QM INTD registers */
    CSL_Qm_intdRegs                         *qmQueIntdReg[QMSS_MAX_INTD];
    /** QM PDSP command register */
    volatile uint32_t                       *qmPdspCmdReg[QMSS_MAX_PDSP];
    /** QM PDSP control register */
    CSL_PdspRegs                            *qmPdspCtrlReg[QMSS_MAX_PDSP];
    /** QM PDSP IRAM register */
    volatile uint32_t                       *qmPdspIRamReg[QMSS_MAX_PDSP];
    /** QM Linking RAM register */
    volatile uint32_t                       *qmLinkingRAMReg;
    /** QM peripheral base address used to calculate internal addresses */
    void                                    *qmBaseAddr;
} Qmss_GlobalConfigRegs;

/** 
 * @brief QMSS Global configuration structure definition
 */
typedef struct
{
    /** Maximum number of queue manager groups */
    uint32_t                                maxQueMgrGroups;
    /** Maximum number of queue Managers */
    uint32_t                                maxQueMgr;
    /** Maximum number of queues */
    uint32_t                                maxQue;
    /** Maximum number of memory regions */
    uint32_t                                maxMemReg;
    /** Maximum number of PDSPs */
    uint32_t                                maxPDSP;
    /** Requires ordered memory regions? */
    int                                     orderedMemReg;
    /** Queue start index and maximum number of queues of each queue type for each queue group */
    Qmss_QueueNumRange                      maxQueueNum[QMSS_MAX_QMGR_GROUPS][QMSS_MAX_QUEUE_TYPES];
    /* Register definitions for each QM group */
    Qmss_GlobalConfigGroupRegs              groupRegs[QMSS_MAX_QMGR_GROUPS];
    /* Register definitions for whole SS */
    Qmss_GlobalConfigRegs                   regs;
    /** QM stores the Resource Manager handle for internal use */
    Qmss_RmHnd                              qmRmHandle;
} Qmss_GlobalConfigParams;

/**
@}
*/
/** Internal Linking RAM default size */
#define QMSS_LINKING_RAM_REGION_0_DEFAULT_SIZE      0x7FFF

/** Used as input parameter when queue number is 
 * not known and not specified */
#define QMSS_PARAM_NOT_SPECIFIED                    -1

/** Used to indicate that QMSS HW Initialization is complete */
#define QMSS_HW_INIT_COMPLETE                       0xABCD

/** QMSS Low level Driver return and Error Codes */
/** QMSS successful return code */
#define QMSS_SOK                                    0
/** QMSS Error Base */       
#define QMSS_LLD_EBASE                              (-128)
/** QMSS LLD invalid parameter */
#define QMSS_INVALID_PARAM                          QMSS_LLD_EBASE-1
/** QMSS LLD not initialized */
#define QMSS_NOT_INITIALIZED                        QMSS_LLD_EBASE-2
/** QMSS LLD queue open error */
#define QMSS_QUEUE_OPEN_ERROR                       QMSS_LLD_EBASE-3
/** QMSS memory region not initialized */
#define QMSS_MEMREGION_NOT_INITIALIZED              QMSS_LLD_EBASE-4
/** QMSS memory region already initialized */
#define QMSS_MEMREGION_ALREADY_INITIALIZED          QMSS_LLD_EBASE-5
/** QMSS memory region invalid parameter */
#define QMSS_MEMREGION_INVALID_PARAM                QMSS_LLD_EBASE-6
/** QMSS maximum number of allowed descriptor are already configured */
#define QMSS_MAX_DESCRIPTORS_CONFIGURED             QMSS_LLD_EBASE-7
/** QMSS Specified memory region index is invalid or no memory regions are available */
#define QMSS_MEMREGION_INVALID_INDEX                QMSS_LLD_EBASE-8
/** QMSS memory region overlap */
#define QMSS_MEMREGION_OVERLAP                      QMSS_LLD_EBASE-9
/** QMSS memory region not in acscending order */
#define QMSS_MEMREGION_ORDERING                     QMSS_LLD_EBASE-10
/** QMSS PDSP firmware download failure */
#define QMSS_FIRMWARE_DOWNLOAD_FAILED               QMSS_LLD_EBASE-11
/** QMSS resource initialization permission denied */
#define QMSS_RESOURCE_INIT_DENIED                   QMSS_LLD_EBASE-12
/** QMSS resource usage permission denied */
#define QMSS_RESOURCE_USE_DENIED                    QMSS_LLD_EBASE-13
/** QMSS memory region initialization permission denied */
#define QMSS_RESOURCE_MEM_REGION_INIT_DENIED        QMSS_LLD_EBASE-14
/** QMSS memory region usage permission denied */
#define QMSS_RESOURCE_MEM_REGION_USE_DENIED         QMSS_LLD_EBASE-15
/** QMSS general linking RAM initialization permission denied */
#define QMSS_RESOURCE_LINKING_RAM_INIT_DENIED       QMSS_LLD_EBASE-16
/** QMSS firmware revision difference */
#define QMSS_FIRMWARE_REVISION_DIFFERENCE           QMSS_LLD_EBASE-17
/** QMSS firmware invalid garbage return queue */
#define QMSS_INVALID_SRIO_GARBAGE_QUEUE             QMSS_LLD_EBASE-18
/** QMSS Bad Queue Number Configuration Table */
#define QMSS_INVALID_QUEUE_NUMBER_TAB               QMSS_LLD_EBASE-19

/** QMSS maximum number of memory regions */
#define QMSS_MAX_MEM_REGIONS                        64
/** QMSS maximum number of PDSPS */
#define QMSS_MAX_PDSP                               8
/** QMSS maximum number of INTDs */
#define QMSS_MAX_INTD                               2
/** QMSS maximum number of queue groups (QMGRs that run independently) */
#define QMSS_MAX_QMGR_GROUPS                        2
/** QMSS maximum number of queue types (per group) */
#define QMSS_MAX_QUEUE_TYPES                        30
/** QMSS maximum number of queues */
#define QMSS_MAX_QUEUES                             16384

/** Macro to get the descriptor pointer if the popped descriptor contains the descriptor size. 
 * If Qmss_queuePushDescSize() API is used to push a descriptor onto a queue, the descriptor when 
 * popped will have the descriptor size information in the lower 4 bits. This macro is provided to 
 * clear out the size information */
#define QMSS_DESC_PTR(desc)                         ((uint32_t)desc & 0xFFFFFFF0)

/** Macro to get the descriptor size if the popped descriptor contains the descriptor size. 
 * If Qmss_queuePushDescSize() API is used to push a descriptor onto a queue, the descriptor when 
 * popped will have the descriptor size information in the lower 4 bits. This macro is provided to 
 * obtain the size information. Minimum size is 16 bytes. Maximum size is 256 bytes */
#define QMSS_DESC_SIZE(desc)                        ((((uint32_t)desc & 0x0000000F) + 1) << 4) 

/** Macro to extract the queue manager group from the queue handle */
#define QMSS_QUEUE_GROUP(hndl)                      ((uint32_t)(hndl) >> 13)
#define QMSS_QUEUE_NUMBER(hndl)                     ((uint32_t)(hndl) & 0x1FFF)

/** 
 * @brief location where the packet is queued
 */
typedef enum
{
    /** Queue packet to the tail of the queue. Default behavior. */
    Qmss_Location_TAIL = 0,
    /** Queue packet to the head of the queue. */
    Qmss_Location_HEAD 
}Qmss_Location;

/** 
 * @brief Descriptor resource management
 */
typedef enum
{
    /** LLD doesnot manage the descriptors. The caller should manage them. */
    Qmss_ManageDesc_UNMANAGED_DESCRIPTOR = 0,
    /** LLD manages the descriptors. The descriptors are reclaimed using 
     * the QMSS_initDescriptor() or CPPI_initDescriptor() APIs
     * */
    Qmss_ManageDesc_MANAGE_DESCRIPTOR 
}Qmss_ManageDesc;

/** 
 * @brief Queue Manager's memory regions
 */
typedef enum
{
    /** Memory region not specified. LLD allocates the next available memory region */
    Qmss_MemRegion_MEMORY_REGION_NOT_SPECIFIED = -1,
    /** Configure memory region0. */
    Qmss_MemRegion_MEMORY_REGION0 = 0,
    /** Configure memory region 1. */
    Qmss_MemRegion_MEMORY_REGION1,
    /** Configure memory region 2. */
    Qmss_MemRegion_MEMORY_REGION2,
    /** Configure memory region 3. */
    Qmss_MemRegion_MEMORY_REGION3,
    /** Configure memory region 4. */
    Qmss_MemRegion_MEMORY_REGION4,
    /** Configure memory region 5. */
    Qmss_MemRegion_MEMORY_REGION5,
    /** Configure memory region 6. */
    Qmss_MemRegion_MEMORY_REGION6,
    /** Configure memory region 7. */
    Qmss_MemRegion_MEMORY_REGION7,
    /** Configure memory region 8. */
    Qmss_MemRegion_MEMORY_REGION8,
    /** Configure memory region 9. */
    Qmss_MemRegion_MEMORY_REGION9,
    /** Configure memory region 10. */
    Qmss_MemRegion_MEMORY_REGION10,
    /** Configure memory region 11. */
    Qmss_MemRegion_MEMORY_REGION11,
    /** Configure memory region 12. */
    Qmss_MemRegion_MEMORY_REGION12,
    /** Configure memory region 13. */
    Qmss_MemRegion_MEMORY_REGION13,
    /** Configure memory region 14. */
    Qmss_MemRegion_MEMORY_REGION14,
    /** Configure memory region 15. */
    Qmss_MemRegion_MEMORY_REGION15,
    /** Configure memory region 16. */
    Qmss_MemRegion_MEMORY_REGION16,
    /** Configure memory region 17. */
    Qmss_MemRegion_MEMORY_REGION17,
    /** Configure memory region 18. */
    Qmss_MemRegion_MEMORY_REGION18,
    /** Configure memory region 19. */
    Qmss_MemRegion_MEMORY_REGION19,
    /** Configure memory region 20. */
    Qmss_MemRegion_MEMORY_REGION20,
    /** Configure memory region 21. */
    Qmss_MemRegion_MEMORY_REGION21,
    /** Configure memory region 22. */
    Qmss_MemRegion_MEMORY_REGION22,
    /** Configure memory region 23. */
    Qmss_MemRegion_MEMORY_REGION23,
    /** Configure memory region 24. */
    Qmss_MemRegion_MEMORY_REGION24,
    /** Configure memory region 25. */
    Qmss_MemRegion_MEMORY_REGION25,
    /** Configure memory region 26. */
    Qmss_MemRegion_MEMORY_REGION26,
    /** Configure memory region 27. */
    Qmss_MemRegion_MEMORY_REGION27,
    /** Configure memory region 28. */
    Qmss_MemRegion_MEMORY_REGION28,
    /** Configure memory region 29. */
    Qmss_MemRegion_MEMORY_REGION29,
    /** Configure memory region 30. */
    Qmss_MemRegion_MEMORY_REGION30,
    /** Configure memory region 31. */
    Qmss_MemRegion_MEMORY_REGION31,
    /** Configure memory region 32. */
    Qmss_MemRegion_MEMORY_REGION32,
    /** Configure memory region 33. */
    Qmss_MemRegion_MEMORY_REGION33,
    /** Configure memory region 34. */
    Qmss_MemRegion_MEMORY_REGION34,
    /** Configure memory region 35. */
    Qmss_MemRegion_MEMORY_REGION35,
    /** Configure memory region 36. */
    Qmss_MemRegion_MEMORY_REGION36,
    /** Configure memory region 37. */
    Qmss_MemRegion_MEMORY_REGION37,
    /** Configure memory region 38. */
    Qmss_MemRegion_MEMORY_REGION38,
    /** Configure memory region 39. */
    Qmss_MemRegion_MEMORY_REGION39,
    /** Configure memory region 40. */
    Qmss_MemRegion_MEMORY_REGION40,
    /** Configure memory region 41. */
    Qmss_MemRegion_MEMORY_REGION41,
    /** Configure memory region 42. */
    Qmss_MemRegion_MEMORY_REGION42,
    /** Configure memory region 43. */
    Qmss_MemRegion_MEMORY_REGION43,
    /** Configure memory region 44. */
    Qmss_MemRegion_MEMORY_REGION44,
    /** Configure memory region 45. */
    Qmss_MemRegion_MEMORY_REGION45,
    /** Configure memory region 46. */
    Qmss_MemRegion_MEMORY_REGION46,
    /** Configure memory region 47. */
    Qmss_MemRegion_MEMORY_REGION47,
    /** Configure memory region 48. */
    Qmss_MemRegion_MEMORY_REGION48,
    /** Configure memory region 49. */
    Qmss_MemRegion_MEMORY_REGION49,
    /** Configure memory region 50. */
    Qmss_MemRegion_MEMORY_REGION50,
    /** Configure memory region 51. */
    Qmss_MemRegion_MEMORY_REGION51,
    /** Configure memory region 52. */
    Qmss_MemRegion_MEMORY_REGION52,
    /** Configure memory region 53. */
    Qmss_MemRegion_MEMORY_REGION53,
    /** Configure memory region 54. */
    Qmss_MemRegion_MEMORY_REGION54,
    /** Configure memory region 55. */
    Qmss_MemRegion_MEMORY_REGION55,
    /** Configure memory region 56. */
    Qmss_MemRegion_MEMORY_REGION56,
    /** Configure memory region 57. */
    Qmss_MemRegion_MEMORY_REGION57,
    /** Configure memory region 58. */
    Qmss_MemRegion_MEMORY_REGION58,
    /** Configure memory region 59. */
    Qmss_MemRegion_MEMORY_REGION59,
    /** Configure memory region 60. */
    Qmss_MemRegion_MEMORY_REGION60,
    /** Configure memory region 61. */
    Qmss_MemRegion_MEMORY_REGION61,
    /** Configure memory region 62. */
    Qmss_MemRegion_MEMORY_REGION62,
    /** Configure memory region 63. */
    Qmss_MemRegion_MEMORY_REGION63
}Qmss_MemRegion;

/** 
 * @brief PDSP ID
 */
typedef enum
{
    /** PDSP 1 */
    Qmss_PdspId_PDSP1 = 0,
    /** PDSP 2 */
    Qmss_PdspId_PDSP2,
    /** PDSP 3 */
    Qmss_PdspId_PDSP3,
    /** PDSP 4 */
    Qmss_PdspId_PDSP4,
    /** PDSP 5 */
    Qmss_PdspId_PDSP5,
    /** PDSP 6 */
    Qmss_PdspId_PDSP6,
    /** PDSP 7 */
    Qmss_PdspId_PDSP7,
    /** PDSP 8 */
    Qmss_PdspId_PDSP8
}Qmss_PdspId;

/** 
 * @brief Queue handle
 */
typedef int32_t   Qmss_QueueHnd;

/** 
 * @brief QMSS return result
 */
typedef int32_t   Qmss_Result;

/** 
 * @brief Handle used in the "Fast Push" set of APIs
 */
typedef uint32_t* Qmss_QueuePushHnd;

/**
 * @brief Map PDK HAL to Linux OSAL
 */
#define ti_em_hw_queue_open      ti_em_osal_hw_queue_open
#define ti_em_hw_queue_close     ti_em_osal_hw_queue_close

/* Retrieve QMSS manager number from queue (8192 queues per QMSS) */
#define ti_em_qmss_get(queue)    ((queue) >> 13)

/* Retrieve absolute base queue for a given infrastructure DMA */
#define ti_em_qmss_get_dma_queue_base(dma_idx) ((dma_idx == Cppi_CpDma_QMSS_CPDMA) ? \
						QMSS_QM1_INFRASTRUCTURE_DMA_QUEUE_BASE : \
						QMSS_QM2_INFRASTRUCTURE_DMA_QUEUE_BASE)
/*
 * XGE
 */
#define TI_EM_XGE_FLOW_NUM       (16) /* total number of flow used for XGE (8 VLAN priorities per interface) */

#ifdef TI_EM_XGE_LOOPBACK
/* Use infra-PktDMA to simulate XGE */
#define TI_EM_XGE_FLOW_IDX_START (14) /* starting at flow 14 */
#define TI_EM_XGE_TX_QUEUE_BASE  (QMSS_INFRASTRUCTURE_QUEUE_BASE)
#define TI_EM_XGE_TMP_RX_QUEUE   (QMSS_GENERAL_PURPOSE_QUEUE_BASE + QMSS_MAX_GENERAL_PURPOSE_QUEUE -  TI_EM_XGE_FLOW_NUM)
#else /* TI_EM_XGE_LOOPBACK */
/* XGE transmit queue base */
#define TI_EM_XGE_FLOW_IDX_START (0) /* starting at first flow */
#define TI_EM_XGE_TX_QUEUE_BASE  (QMSS_XGE_QUEUE_BASE)
#endif /* TI_EM_XGE_LOOPBACK */

#endif /*TI_EM_HW_QMSS_H_*/
