/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_osal_event.h
 *
 *   @brief
 *      This is the OS abstraction layer for waiting events
 *
 */
#ifndef TI_EM_OSAL_EVENT_H_
#define TI_EM_OSAL_EVENT_H_

#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti_em_hw_mach.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>

#define TI_EM_OSAL_EVENT_DEVICE_BASENAME   "/dev/openem"

struct ti_em_intd_irq_config {
	int channel_base;
	int intd;
};

#define EM_IOC_MAGIC		           'O'
#define EM_IOC_WAITEVENT                   _IO(EM_IOC_MAGIC, 2)
#define EM_IOC_WAITEVENTINIT               _IOWR(EM_IOC_MAGIC, 3, struct ti_em_intd_irq_config)
#define EM_IOC_WAITEVENTFREE               _IO(EM_IOC_MAGIC, 4)

extern int  ti_em_osal_event_init(int pdsp, int channel_base);
extern int  ti_em_osal_event_release(void);
extern int  ti_em_osal_event_wait(void);

#endif /*TI_EM_OSAL_EVENT_H_*/
