/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_osal_queue.h
 *
 *   @brief
 *      This is the OS abstraction layer for the QMSS management.
 *
 */

#ifndef TI_EM_OSAL_QUEUE_H_
#define TI_EM_OSAL_QUEUE_H_

#include <ti/runtime/openem/event_machine_types.h>

#include <ti_em_hw_csl.h>
#include <ti_em_hw_qmss.h>
#include <ti_em_hw_cppi.h>

#include <ti_em_osal_uaccess.h>

typedef struct ti_em_osal_hw_region_config_t_
{
	uint32_t    region_idx;
	phys_addr_t desc_base;
	uint32_t    desc_vbase;
	uint32_t    desc_offset;
	uint32_t    desc_size;
	uint32_t    desc_num;
	uint32_t    desc_flag;
	int32_t     start_idx;
} ti_em_osal_hw_region_config_t;

extern em_status_t  ti_em_osal_hw_queue_open(int queue);
extern em_status_t  ti_em_osal_hw_queue_close(int queue);
extern em_status_t  ti_em_osal_hw_queue_empty(int queue);
extern em_status_t  ti_em_osal_hw_queue_insert_region(ti_em_osal_hw_region_config_t *config, int qmss);
extern unsigned int ti_em_osal_hw_queue_get_memory_region_addr(short memory_idx, int qmss);
extern void         ti_em_osal_hw_queue_push(int queue, void *desc, short mapping);
extern void         ti_em_osal_hw_queue_push_to_head(int queue, void *desc, short mapping);
extern void*        ti_em_osal_hw_queue_pop(int queue, short mapping);
extern void         ti_em_osal_hw_queue_set_threshold(int queue, short hilo, char threshold);
extern void         ti_em_osal_hw_queue_divert(int src_queue, int dst_queue, char location);
extern unsigned int ti_em_osal_hw_queue_peek(int queue);

static inline void* ti_em_osal_queue_desc_ptov(unsigned int paddr, short mapping) {
	return (paddr == 0) ?
		NULL : (void*) ti_em_osal_mem_phys_to_virt(paddr, mapping);
}

static inline unsigned int ti_em_osal_queue_desc_vtop(void *vaddr, short mapping) {
	return (vaddr == NULL) ?
		0 : ti_em_osal_mem_virt_to_phys((unsigned int) vaddr, mapping);
}

#endif /*TI_EM_OSAL_QUEUE_H_*/
