/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_osal_shm.h
 *
 *   @brief
 *      This is the OS abstraction layer for the shared memory management
 *
 */

#ifndef TI_EM_OSAL_SHM_H_
#define TI_EM_OSAL_SHM_H_

extern int   ti_em_osal_shm_init(int id, char *shm_name, unsigned int size, short create);
extern void  ti_em_osal_shm_release(int id);
extern void* ti_em_osal_shm_get(int id, unsigned int size);
extern void* ti_em_osal_shm_alloc(int id, unsigned int size);
extern void  ti_em_osal_shm_free(void *ptr);


#define TI_EM_OSAL_SHM_MAPPING_NUM             8 /* number of shared memory mappings */
#define TI_EM_OSAL_SHM_OSAL_ID                 1 /* shared mem Id 1 is used by OSAL */
#define TI_EM_OSAL_SHM_OSAL_NAME               "openem_osal"
#define TI_EM_OSAL_SHM_OSAL_MAPPING_DB_ID      2 /* shared mem Id 2 is used for OSAL mapping db */
#define TI_EM_OSAL_SHM_OSAL_MAPPING_DB_NAME    "openem_osal_mapping_db"
#define TI_EM_OSAL_SHM_DEFAULT_ID              3 /* first available mapping*/

#define TI_EM_OSAL_SHM_GLOBAL_ALLOC(p) p  = (typeof(p)) ti_em_osal_shm_alloc(TI_EM_OSAL_SHM_DEFAULT_ID, \
									     sizeof(*p))

#endif /* TI_EM_OSAL_SHM_H_ */
