/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_osal_uaccess.h
 *
 *   @brief
 *      This is the OS abstraction layer for the I/O accesses from user mode
 *
 */

#ifndef TI_EM_OSAL_UACCESS_H_
#define TI_EM_OSAL_UACCESS_H_

#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti_em_hw_mach.h>

#define TI_EM_OSAL_MEM_UACCESS_DEVICE    "/dev/openem0"

#define TI_EM_OSAL_MEM_CACHED            (0x0)
#define TI_EM_OSAL_MEM_UNCACHED          (0x1)

#define TI_EM_OSAL_DEFAULT_MAPPING_FLAGS (TI_EM_OSAL_MEM_UNCACHED)

typedef unsigned int phys_addr_t;

struct ti_em_osal_mem_mapping {
	int          fd;
	phys_addr_t  paddr;
	unsigned int vaddr;
	unsigned int size;
	off_t        offset;
	unsigned int flags;
};

extern struct ti_em_osal_mem_mapping __ti_em_osal_mem_mappings[TI_EM_MEM_NUM];

/*
 * Transform a physical address to a virtual one for a given mapping
 *
 * There is no check here since we must do this fast.
 */
static inline unsigned int ti_em_osal_mem_phys_to_virt(phys_addr_t paddr, short mapping)
{
	return __ti_em_osal_mem_mappings[mapping].vaddr
	    + (paddr - __ti_em_osal_mem_mappings[mapping].paddr);
}

/*
 * Transform a physical address to a virtual one for a given mapping
 *
 * There is no check here since we must do this fast.
 */
static inline phys_addr_t ti_em_osal_mem_virt_to_phys(unsigned int vaddr, short mapping)
{
	return (phys_addr_t) __ti_em_osal_mem_mappings[mapping].paddr
	    + (vaddr -  __ti_em_osal_mem_mappings[mapping].vaddr);
}

static inline unsigned int ti_em_osal_io_phys_to_virt(phys_addr_t paddr)
{
	return (unsigned int) ti_em_osal_mem_phys_to_virt(paddr, TI_EM_MEM_QMSS_REGS);
}

static inline phys_addr_t ti_em_osal_io_virt_to_phys(unsigned int vaddr)
{
	return (phys_addr_t) ti_em_osal_mem_virt_to_phys(vaddr, TI_EM_MEM_QMSS_REGS);
}

static inline phys_addr_t ti_em_osal_get_mem_base(short mapping)
{
	return __ti_em_osal_mem_mappings[mapping].paddr;
}

extern phys_addr_t __ti_em_osal_pdsp_comm_mem_base;

static inline phys_addr_t ti_em_osal_get_pdsp_comm_mem_base(uint32_t slot)
{
	return __ti_em_osal_pdsp_comm_mem_base + (slot * PAGE_SIZE);
}

static inline void ti_em_osal_set_pdsp_comm_mem_base(phys_addr_t paddr)
{
	__ti_em_osal_pdsp_comm_mem_base = paddr;
}

extern int ti_em_osal_create_mapping(short mapping, struct ti_em_osal_mem_mapping *mapping_info);
extern void ti_em_osal_release_mapping(short mapping);

#ifndef TI_EM_THREAD_MODEL
extern int ti_em_osal_retrieve_mapping(short mapping, struct ti_em_osal_mem_mapping *mapping_info);
#endif

#endif /*TI_EM_OSAL_UACCESS_H_*/
