/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_osal_xge.c
 *
 *   @brief
 *      This is the OS abstraction layer for the 10Gig Ethernet management.
 *
 */

#ifndef TI_EM_OSAL_XGE_H_
#define TI_EM_OSAL_XGE_H_

#include <ti/runtime/openem/event_machine_types.h>

#include <ti_em_hw_csl.h>
#include <ti_em_hw_qmss.h>
#include <ti_em_hw_cppi.h>

#include <ti_em_osal_uaccess.h>
#include <ti_em_osal_queue.h>

extern em_status_t ti_em_osal_xge_rx_flow_open(int dma_idx, int flow_idx, int dst_queue_idx, int free_queue_idx0, int free_queue_idx1, int error_handling);
extern em_status_t ti_em_osal_xge_tx_channel_open(int dma_idx, int channel_idx);
extern em_status_t ti_em_osal_xge_rx_channel_open(int dma_idx, int channel_idx);
extern ti_em_queue_id_t ti_em_osal_xge_tx_queue_open(ti_em_queue_id_t queue_base_idx, int vlan_priority);
extern em_status_t ti_em_osal_xge_rx_flow_close(int dma_idx, int flow_idx);
extern em_status_t ti_em_osal_xge_rx_channel_close(int dma_idx, int channel_idx);
extern em_status_t ti_em_osal_xge_tx_channel_close(int dma_idx, int channel_idx);
extern ti_em_queue_id_t ti_em_osal_xge_tx_queue_base_idx_get (void);
extern ti_em_queue_id_t ti_em_osal_xge_rx_queue_idx_get(int flow_idx);
#ifdef TI_EM_XGE_LOOPBACK
extern em_status_t ti_em_osal_xge_rx_insert_crc(int flow_idx,
						ti_em_queue_id_t rx_fragment_free_queue,
						ti_em_queue_id_t xge_dst_queue,
						int count);
#else /* TI_EM_XGE_LOOPBACK */
static inline em_status_t ti_em_osal_xge_rx_insert_crc(int flow_idx,
						       ti_em_queue_id_t rx_fragment_free_queue,
						       ti_em_queue_id_t xge_dst_queue,
						       int count) { return EM_OK; };
#endif /* TI_EM_XGE_LOOPBACK */
#endif /*TI_EM_OSAL_XGE_H_*/
