/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  ti_em_rh.h
 *
 *   @brief
 *      This is the OpenEM runtime helper library for Linux
 *
 */
#ifndef _TI_EM_RH_H_
#define _TI_EM_RH_H_

#include <ti_em_osal.h>
#include <ti_em_osal_queue.h>

#include <ti/runtime/openem/event_machine_hw_ti_types.h>

#define TI_EM_RH_PUBLIC                (0)
#define TI_EM_RH_PRIVATE               (1)

#define TI_EM_RH_UNMANAGED_DESCRIPTOR  (0)
#define TI_EM_RH_MANAGE_DESCRIPTOR     (1)

#define TI_EM_RH_POLL_MODE             (0)
#define TI_EM_RH_INTERRUPT_MODE        (1)

#define TI_EM_RH_PRIVATE_EVENT_NUM     (TI_EM_PDSP_GLOBAL_DATA_SIZE / TI_EM_PRIVATE_EVENT_DSC_SIZE)

/* 
 * PDSP firmware download information structure
 */
typedef struct ti_em_rh_pdsp_config_t_
{
	short     pdsp_id;
	void     *firmware;
	uint32_t  size; 
} ti_em_rh_pdsp_config_t;

/*
 * Physical memory description structure
 */
typedef struct ti_em_rh_mem_config_t_
{
	phys_addr_t paddr;
	uint32_t    vaddr;
	uint32_t    size;
	uint32_t    offset;
	uint32_t    mapping_id;
	uint32_t    flags;
} ti_em_rh_mem_config_t;

/*
 * Pool of event configuration structure
 */
typedef struct ti_em_rh_pool_config_t_
{
	uint32_t              event_num;
	uint32_t              buffer_size;
	uint32_t              offset;
	uint32_t              ps_word_num;
	uint32_t              free_policy;
	ti_em_pool_config_t   pool;
	ti_em_rh_mem_config_t mem;
} ti_em_rh_pool_config_t;

/*
 * Main OpenEM hw configuration structure
 */
typedef struct ti_em_rh_hw_config_t_
{
	ti_em_device_id_t              device_id;
	ti_em_process_id_t             process_id;
	ti_em_chain_config_t*          chain_config_ptr;
	uint32_t                       private_free_queue_idx;
	uint32_t                       region_num; 
	ti_em_osal_hw_region_config_t *region_configs;
	uint32_t                       hw_queue_base_idx;
	uint32_t                       dma_queue_base_idx;
	int32_t                        dma_idx;
	uint32_t                       preload_size_a;
	uint32_t                       preload_size_b;
	uint32_t                       preload_size_c;
	ti_em_rh_mem_config_t          pdsp_comm_mem_config;
	uint32_t                       pdsp_num;
	ti_em_rh_pdsp_config_t         pdsp_sched[TI_EM_SCHEDULER_THREAD_NUM];
	ti_em_rh_pdsp_config_t         pdsp_router;
	uint32_t                       dispatch_mode;
	uint32_t                       hw_interrupt_base_idx;
} ti_em_rh_hw_config_t;

extern em_status_t ti_em_rh_init_global(
	uint32_t                       pool_num,
	ti_em_rh_pool_config_t        *pool_configs,
	uint32_t                       core_num,
	ti_em_rh_hw_config_t          *hw_config);
extern em_status_t ti_em_rh_release_global(void);
extern em_status_t ti_em_rh_init_local(void);
extern em_status_t ti_em_rh_release_local(void);
extern em_status_t ti_em_rh_join(int core_id);
extern em_status_t ti_em_rh_create_cores(
	int core_num,
	int(*dispatcher_fn)(void*),
	int(*init_fn)(void),
	int hw_core_start);
extern em_status_t ti_em_rh_wait_cores(int core_num);
extern em_status_t ti_em_rh_kill_cores(int core_num);
extern em_status_t ti_em_rh_dispatch(int mode, uint32_t sleep_quota, uint32_t cancel_quota);

#endif /*_TI_EM_RH_H_*/
