/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*
 * Copyright (c) 2013, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <ti_em_osal.h>
#include <ti_em_osal_core.h>
#include <ti_em_osal_queue.h>
#include <ti_em_osal_shm.h>

#include <ti_em_rh.h>

#include <getopt.h>

int main(int argc, char **argv)
{
	struct ti_em_osal_mem_mapping mapping_info;
	int c;
	int lop;
	int queue = -1;
	int res;
	int dump = 0;

	struct option long_options[] = {
		{ "queue", 1, 0, 'q'},
		{ "dump", 1, 0, 'd'},
		{ 0, 0, 0, 0 }
	};
	
	memset(&mapping_info, 0, sizeof(mapping_info));

	/* Map the I/O configuration registers */
	mapping_info.paddr  = TI_EM_IO_CFG_BASE;
	mapping_info.vaddr  = 0;
	mapping_info.offset = 0;
	mapping_info.size   = TI_EM_IO_CFG_SIZE;
	mapping_info.flags  = TI_EM_OSAL_MEM_UNCACHED;
	res = ti_em_osal_create_mapping(TI_EM_MEM_QMSS_REGS, &mapping_info);
	if (res == -1) {
		printf("cannot map QMSS registers\n");
		return EM_ERR;
	}

	/* runtime options parse */
	while ((c = getopt_long(argc, argv, "q:d:", long_options, &lop)) != EOF) {
		switch(c) {
		case 'q':
			queue = strtoul(optarg, (char **) NULL, 10);
			break;
		case 'd':
			queue = strtoul(optarg, (char **) NULL, 10);
			dump = 1;
			break;
		default:
			break;
		}
	}

	printf("\n");

	if (dump) {
		void *desc = NULL;

		while ((desc = ti_em_osal_hw_queue_pop(queue, 0)) != NULL)
			printf("queue %05d: desc = 0x%x\n", queue, (unsigned) desc);
	       
		return 0;
	}

	if (queue == -1) {
	    for (queue = 0; queue < QMSS_MAX_QUEUES; queue++) {
		unsigned int desc_num  = ti_em_osal_hw_queue_peek(queue);
		if (desc_num != 0)
		    printf("queue %05d: number of pending desc = %d\n", queue, desc_num);
	    }
	} else {
		unsigned int desc_num  = ti_em_osal_hw_queue_peek(queue);
		printf("queue %05d: number of pending desc = %d\n", queue, desc_num);
	}

	return 0;
}
