/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*
 * em_router.c
 *
 *  Created on: Mar 25, 2013
 *      Author: x0019308
 */

#include <ti/runtime/openem/event_machine_types.h>
#include <ti/runtime/openem/src/event_machine_private_types.h>
#include <ti/runtime/openem/src/event_machine_private_macros.h>
#include <ti/runtime/openem/src/event_machine_private_variables.h>
#include <ti/runtime/openem/src/event_machine_private_functions.h>
#include <ti/runtime/openem/event_machine_hw_ti_macros.h>

#ifndef TI_EM_LINUX
#include <ti/drv/cppi/cppi_drv.h>
#include <ti/drv/cppi/cppi_desc.h>
#include <ti/drv/qmss/qmss_drv.h>
#endif /* TI_EM_LINUX */

#define CPPI_W1_TAG_INFO_OFFSET          4  /*  1  words offset = 1x4 Bytes */
#define CPPI_W3_BUFF_LEN_OFFSET         12  /*  3  words offset = 3x4 Bytes */
#define CPPI_W4_BUFF_PTR_OFFSET         16  /*  4  words offset = 4x4 Bytes */
#define CPPI_W5_NEXT_BUFF_PTR_OFFSET    20  /*  5  words offset = 5x4 Bytes */
#define CPPI_W8_TIMESTAMP_OFFSET        32  /*  8  words offset = 8x4 Bytes */
#define CPPI_W9_SW_INFO_0_OFFSET        36  /*  9  words offset = 9x4 Bytes */
#define CPPI_W12_PS_DATA_OFFSET         48  /*  12 words offset = 12x4 Bytes */



#ifndef TI_EM_LINUX
#pragma DATA_SECTION (Emti_fwRpsb, ".tiEmFw");
#endif
Emti_Rpsb_s Emti_fwRpsb[TI_EM_PROCESS_NUM];
#ifndef TI_EM_LINUX
#pragma DATA_SECTION (Emti_fwRpcb, ".tiEmFw");
#endif
Emti_Rpcb_s Emti_fwRpcb[TI_EM_PROCESS_NUM];

/**
 * @internal
 *
 * RIO Chain Header structure
 *
 */
struct Emti_RioChainHdr_s_
{
    uint16_t dstDeviceIdx;
    uint16_t srcDeviceIdx;
    uint8_t serviceClass;
    uint8_t transportInfo;
    uint16_t dstInfo;
};

typedef struct Emti_RioChainHdr_s_ Emti_RioChainHdr_s;

em_status_t
event_machine_sw_RouteToQueue (
    uint32_t lvEventDscPtr,
    uint16_t lvProcessIdx,
    uint16_t lvQueueIdx,
    uint8_t lvEventType
)
{
    uint16_t lvHwQueueIdx;
    uint8_t lvQueueSetIdx;
    uint8_t lvQueueIdxInSet;
    uint16_t lvRouteInfo;
    uint16_t lvQueueTag;
    uint8_t lvQueueMode;
    uint8_t lvQueueType;
    uint32_t lvW8;

    lvQueueSetIdx = (lvQueueIdx & Emti_QUEUE_HANDLE_QUEUE_SET_IDX_MASK) >> Emti_QUEUE_HANDLE_QUEUE_SET_IDX_SHIFT;
    lvQueueIdxInSet =
        (lvQueueIdx & Emti_QUEUE_HANDLE_QUEUE_IDX_IN_SET_MASK) >> Emti_QUEUE_HANDLE_QUEUE_IDX_IN_SET_SHIFT;

    if (lvQueueIdx < (Emti_fwRpcb[lvProcessIdx].staticQueueNum + 1))
    {
        lvRouteInfo = Emti_fwRpsb[lvProcessIdx].staticQueueRouteTbl[lvQueueIdx].routeInfo;
        lvQueueIdxInSet = 0;
    }
    /*
     * If not static queue  
     */
    else
    {
        lvRouteInfo = Emti_fwRpsb[lvProcessIdx].queueRouteTbl[lvQueueSetIdx].routeInfo;
    }

    /*
     * if queue not enabled    
     */
    if (((lvRouteInfo & Emti_QUEUE_ROUTE_INFO_ROUTE_ENABLED_MASK) >> Emti_QUEUE_ROUTE_INFO_ROUTE_ENABLED_SHIFT) == 0)
    {
        /*
         * Put 0 into packetSize    
         */
        *(uint32_t *) ((uint8_t *) lvEventDscPtr) = 0;
        lvEventDscPtr = (((uint32_t) lvEventDscPtr) & 0xFFFFFFF0) | Emti_DSC_HINT_RS;
        Emti_mfence ();
        Emti_pushEvent (Emti_fwRpcb[lvProcessIdx].rsTxQueueIdx, (uint32_t *) lvEventDscPtr, TI_EM_MEM_PUBLIC_DESC);
        //MN Emti_fwRdsb.errorCnt++;
        return (em_status_t) EM_ERROR;
    }
    lvQueueMode = (lvRouteInfo & Emti_QUEUE_ROUTE_INFO_QUEUE_MODE_MASK) >> Emti_QUEUE_ROUTE_INFO_QUEUE_MODE_SHIFT;
    if (lvQueueMode == TI_EM_QUEUE_MODE_SD)
    {
        /*
         * Set Word8 of Event desc    
         */
        /*
         * -> queueIdx                
         *
         * -> queueType               
         *
         * -> eventType               
         */
        lvQueueType = (lvRouteInfo & Emti_QUEUE_ROUTE_INFO_QUEUE_TYPE_MASK) >> Emti_QUEUE_ROUTE_INFO_QUEUE_TYPE_SHIFT;
        lvW8 = lvQueueIdx << Emti_EVENT_DSC_W8_QUEUE_IDX_SHIFT;
        lvW8 += (lvQueueType << Emti_EVENT_DSC_W8_QUEUE_TYPE_SHIFT);
        lvW8 += (lvEventType << Emti_EVENT_DSC_W8_EVENT_TYPE_SHIFT);
        *(uint32_t *) ((uint8_t *) lvEventDscPtr + CPPI_W8_TIMESTAMP_OFFSET) = lvW8;
        /*
         * Get SD queue    
         */
        lvQueueTag =
            (lvRouteInfo & Emti_QUEUE_ROUTE_INFO_SD_QUEUE_IDX_MASK) >> Emti_QUEUE_ROUTE_INFO_SD_QUEUE_IDX_SHIFT;
        lvHwQueueIdx = Emti_fwRpcb[lvProcessIdx].sdQueueBaseIdx + lvQueueTag;
    }
    else if (lvQueueMode == TI_EM_QUEUE_MODE_HW)
    {
        lvQueueTag =
            (lvRouteInfo & Emti_QUEUE_ROUTE_INFO_HW_QUEUE_IDX_MASK) >> Emti_QUEUE_ROUTE_INFO_HW_QUEUE_IDX_SHIFT;
        if (lvQueueIdx < (Emti_fwRpcb[lvProcessIdx].staticQueueNum + 1))
            lvHwQueueIdx = lvQueueTag + lvQueueIdxInSet;
        else
            lvHwQueueIdx = lvQueueTag + lvQueueIdxInSet - TI_EM_STATIC_QUEUE_NUM_IN_SET;
    }
    
    /* check HwQueueIdx == PA Tx queue Idx then fill ps words and send to PA*/
    if (lvHwQueueIdx == Emti_fwRpcb[lvProcessIdx].paTxQueueIdx)
    {
        /* get RIO chain header pointer */
        Emti_PaHdrRioFormat_s *lvPaHdrPtr;
        uint32_t * lvUint32PaHdrPtr;
        Cppi_HostDesc * lvCppiHostDscPtr;
        
        lvCppiHostDscPtr = (Cppi_HostDesc *)(lvEventDscPtr);
        lvPaHdrPtr = (Emti_PaHdrRioFormat_s *) (&(lvCppiHostDscPtr->psData));
        lvUint32PaHdrPtr = (uint32_t*)(&(lvCppiHostDscPtr->psData));
        
        /* clear RIO chain header */
        *lvUint32PaHdrPtr++ = 0;
        *lvUint32PaHdrPtr = 0;
    
        /* fill RIO chain header */
        Emti_setField (lvPaHdrPtr->dstInfo, RIO_CHAIN_HDR_DST_INFO_PROCESS_IDX, lvProcessIdx);
        Emti_setField (lvPaHdrPtr->dstInfo, RIO_CHAIN_HDR_DST_INFO_QUEUE_IDX, lvQueueIdx);

        /*
         * set PS data Length (8 Bytes) 
         */
        lvCppiHostDscPtr->packetInfo &= 0xF0FFFFFF;
        lvCppiHostDscPtr->packetInfo |= 0x02000000;
        
        /*
         * set Packet Type to 30 (Type9)    
         */
        lvCppiHostDscPtr->descInfo &= ~Emti_EVENT_SIZE_EVENT_TAG_MASK;
        lvCppiHostDscPtr->descInfo |= (Emti_EVENT_TAG_RIO << Emti_EVENT_SIZE_EVENT_TAG_SHIFT);
        
        /*
         * Set descriptor size to Emti_DSC_HINT_MAX  
         */
        lvEventDscPtr = (Cppi_HostDesc *) (((uint32_t) lvEventDscPtr & 0xFFFFFFF0) + Emti_DSC_HINT_MAX);    
    }
    
    Emti_mfence ();
    /*** PUSH EVENT INTO QUEUE ***/
    Emti_pushEvent (lvHwQueueIdx, (uint32_t *) lvEventDscPtr, TI_EM_MEM_PUBLIC_DESC);
    return (em_status_t) EM_OK;
}



em_status_t
event_machine_sw_DmaReceive (
    uint16_t lvProcessIdx
)
{
    uint32_t lvEventDscPtr;

    /*
     * Event received in DMA Rx queue    
     */
    /*
     * Pop Event from rx queue           
     */
    lvEventDscPtr = (uint32_t) Emti_popEvent (Emti_fwRpcb[lvProcessIdx].rxQueueIdx, TI_EM_MEM_PUBLIC_DESC) & 0xFFFFFFF0;
    if (lvEventDscPtr != 0)
    {
        uint32_t lvWord;
        uint16_t lvQueueIdx;
        uint8_t lvEventType;
        lvWord = (*(uint32_t *) ((uint8_t *) lvEventDscPtr + CPPI_W8_TIMESTAMP_OFFSET));
        lvQueueIdx = (uint16_t) (lvWord & Emti_EVENT_DSC_W8_QUEUE_IDX_MASK);
        lvEventType = (uint8_t) ((lvWord & Emti_EVENT_DSC_W8_EVENT_TYPE_MASK) >> Emti_EVENT_DSC_W8_EVENT_TYPE_SHIFT);
        event_machine_sw_RouteToQueue (lvEventDscPtr, lvProcessIdx, lvQueueIdx, lvEventType);
    }                           /* EndOf (if(lvEventDscPtr != 0)) */
    return (em_status_t) EM_OK;
}

em_status_t
event_machine_sw_RioReceive (
    uint16_t lvProcessIdx
)
{
    /*
     * Event received in SRIO Rx queue    
     */


    uint32_t lvEventDscPtr;

    /*
     * If the RIO is not connected
     * exit!
     */
    if (Emti_fwRpcb[lvProcessIdx].chainRioDsc.rxQueueIdx == 0)
        return (em_status_t) EM_OK;

    /*
     * Pop Event from rx queue            
     */
    lvEventDscPtr =
        (uint32_t) Emti_popEvent (Emti_fwRpcb[lvProcessIdx].chainRioDsc.rxQueueIdx, TI_EM_MEM_PUBLIC_DESC) & 0xFFFFFFF0;

    if (lvEventDscPtr != 0)
    {
        Emti_RioChainHdr_s *lvRioChainHdrPtr;

        uint16_t lvQueueIdx;
        uint8_t lvErrorCode;
        lvRioChainHdrPtr = (Emti_RioChainHdr_s *) ((uint8_t *) lvEventDscPtr + CPPI_W12_PS_DATA_OFFSET);

        /*
         * Get queueIdx 
         */
        lvQueueIdx = (uint16_t) lvRioChainHdrPtr->dstInfo & Emti_RIO_CHAIN_HDR_DST_INFO_QUEUE_IDX_MASK;
        /*
         * Error Code   
         */
        lvErrorCode = (uint8_t) lvRioChainHdrPtr->transportInfo & Emti_RIO_CHAIN_HDR_TPT_INFO_ERROR_CODE_MASK;

        /**** DEBUG ***/
        lvErrorCode = 0;
        /**** DEBUG ***/
        if (lvErrorCode > 0)
        {
            /*
             * Set PacketSize to 0  
             */
            *(uint32_t *) lvEventDscPtr = *(uint32_t *) lvEventDscPtr & 0xFFC00000;
            /*
             * Set descriptor size to Emti_DSC_HINT_RS  
             */
            lvEventDscPtr = ((uint32_t) lvEventDscPtr & 0xFFFFFFF0) + Emti_DSC_HINT_RS;
            Emti_mfence ();
            Emti_pushEvent (Emti_fwRpcb[lvProcessIdx].rsTxQueueIdx, (uint32_t *) lvEventDscPtr, TI_EM_MEM_PUBLIC_DESC);
            Emti_fwRpsb[lvProcessIdx].errorCnt++;
            return (em_status_t) EM_ERROR;
        }                       /* EndOf (if (lvErrorCode > 0)) */
        event_machine_sw_RouteToQueue (lvEventDscPtr, lvProcessIdx, lvQueueIdx, 0);
    }                           /* EndOf (if(lvEventDscPtr != 0)) */
    return (em_status_t) EM_OK;
}



void
StartRouterPart (
    uint16_t processIdx
)
{
    uint32_t lvProcessCdPtr;
    uint16_t lvRouteIdx;
    uint16_t lvCtType;
    Emti_Rpsb_s *lvShadowRpsbPtr;
    uint16_t lvRpQueueIdx;
    //uint16_t lvProcessIdx;

    /*
     * For Each open process, check process command queue    
     */
    //for (lvProcessIdx = 0; lvProcessIdx < TI_EM_PROCESS_NUM; lvProcessIdx++)
    //{
    /*
     * If Process is enabled    
     */
    //if (Emti_fwRdsb.processMsk & (1 << processIdx))
    {
        /*
         * Check the process command queue    
         */
        if ((lvProcessCdPtr =
             (uint32_t) Emti_popEvent (Emti_fwRpcb[processIdx].cdQueueIdx, TI_EM_MEM_PRIVATE_DESC) & 0xFFFFFFF0) != 0)
        {
            lvCtType = *(uint32_t *) lvProcessCdPtr & 0xFFFF;
            lvRouteIdx = (*(uint32_t *) lvProcessCdPtr >> 16) & 0xFFFF;
#ifdef TI_EM_LINUX
            /*
             * AJ: This is a hack since osal functions must be called through Emti_*() functions 
             */
            lvShadowRpsbPtr =
                (Emti_Rpsb_s *) ti_em_osal_mem_phys_to_virt (*((uint32_t *) lvProcessCdPtr + 1), TI_EM_MEM_PDSP_COMM);;
#else
            lvShadowRpsbPtr = (Emti_Rpsb_s *)Emti_publicDesc_ptov((uint32_t)(* ((uint32_t *) lvProcessCdPtr + 1)));
#endif
            lvRpQueueIdx = *((uint32_t *) lvProcessCdPtr + 3) & 0xFFFF;

            switch (lvCtType)
            {
                case Emti_CtType_ROUTER_UPDATE_ROUTE:
                    /*
                     * Update queueRouteTable    
                     */
                    memcpy (&Emti_fwRpsb[processIdx].queueRouteTbl[lvRouteIdx],
                            &lvShadowRpsbPtr->queueRouteTbl[lvRouteIdx], sizeof (Emti_QueueRoute_s));
                    /*
                     * Returns desc with 'NO ERROR'    
                     */
                    *((uint32_t *) lvProcessCdPtr + 2) = (uint32_t) Emti_CdQueueErrorCode_NO_ERROR;
                    Emti_mfence ();
                    Emti_pushEvent (lvRpQueueIdx, (uint32_t *) lvProcessCdPtr, TI_EM_MEM_PRIVATE_DESC);
                    break;
                case Emti_CtType_ROUTER_UPDATE_ROUTE_STATIC:
                    /*
                     * Update queueRouteTable    
                     */
                    memcpy (&Emti_fwRpsb[processIdx].staticQueueRouteTbl[lvRouteIdx],
                            &lvShadowRpsbPtr->staticQueueRouteTbl[lvRouteIdx], sizeof (Emti_QueueRoute_s));
                    /*
                     * Returns desc with 'NO ERROR'    
                     */
                    *((uint32_t *) lvProcessCdPtr + 2) = (uint32_t) Emti_CdQueueErrorCode_NO_ERROR;
                    Emti_mfence ();
                    Emti_pushEvent (lvRpQueueIdx, (uint32_t *) lvProcessCdPtr, TI_EM_MEM_PRIVATE_DESC);
                    break;
                case Emti_CtType_KILL:
                    /*
                     * Returns desc with 'NO ERROR'    
                     */
                    *((uint32_t *) lvProcessCdPtr + 2) = (uint32_t) Emti_CdQueueErrorCode_NO_ERROR;
                    Emti_mfence ();
                    Emti_pushEvent (lvRpQueueIdx, (uint32_t *) lvProcessCdPtr, TI_EM_MEM_PRIVATE_DESC);
                    return;
                default:
                    *((uint32_t *) lvProcessCdPtr + 2) = (uint32_t) Emti_CdQueueErrorCode_INVALID_CMD_ID;
                    Emti_mfence ();
                    Emti_pushEvent (lvRpQueueIdx, (uint32_t *) lvProcessCdPtr, TI_EM_MEM_PRIVATE_DESC);
                    break;
            }                   /* EndOf (switch(lvCtType))    */
        }                       /* EndOf (if((lvProcessCdPtr = (uint32_t)Emti_popEvent(Emti_fwRpcb[lvProcessIdx].cdQueueIdx) & 0xFFFFFFF0) != NULL))    */
        event_machine_sw_DmaReceive (processIdx);
        event_machine_sw_RioReceive (processIdx);
    }                           /* EndOf(if(Emti_fwRdsb.processMsk & (1<<lvProcessIdx))) */
    //}                       /* Endof (for(lvProcessIdx = 0; lvProcessIdx< TI_EM_PROCESS_NUM)) */
}
