/*
 *  ======== package.xs ========
 *
 */


/*
 *  ======== Package.getLibs ========
 *  This function is called when a program's configuration files are
 *  being generated and it returns the name of a library appropriate
 *  for the program's configuration.
 */

function getLibs(prog)
{
    var Settings = this.Settings;
    var suffix;
    var debugFlag;
    var targetFlag;
    var targetPath;

    /* find a compatible suffix */
    if ("findSuffix" in prog.build.target) 
    {
        suffix = prog.build.target.findSuffix(this);
    }
    else 
    {
        suffix = prog.build.target.suffix;
    }

    switch (Settings.debugFlag) 
    {
        case Settings.DEBUG:
            debugFlag = "debug_";
            break;
        case Settings.RELEASE:
            debugFlag = "release_";
            break;
        default:
            throw new Error("debugFlag not set");
    }
    
    switch (Settings.targetFlag) 
    {
        case Settings.C6670:
            targetFlag = "c6670_";
            targetPath = "lib/c6670/";
            break;
        case Settings.C6678:
            targetFlag = "c6678_";
            targetPath = "lib/c6678/";
            break;
        case Settings.C6634:
            targetFlag = "c6634_";
            targetPath = "lib/c6634/";
            break;
        default:
            throw new Error("targetFlag not set");
    }

    lib = targetPath + this.$name.replace(/\./g, "_") + "_" + targetFlag + debugFlag + suffix + ".a" + suffix
    if (java.io.File(this.packageBase + lib).exists()) 
    {
	return lib;
    }
    else
    {
         /* could not find any library, throw exception */
         throw new Error("Library not found: " + lib);
    }
}
 
/*
 *  ======== package.close ========
 */
function close()
{    
    if (xdc.om.$name != 'cfg') {
        return;
    }
}

