/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 * @file: event_machine_chaining.c
 *
 * @brief
 *    Open Event Machine HW TI specific functions implementation
 *    Functions related to chaining.
 *
 */

#include <stdio.h>
#include <string.h>
#include <ti/runtime/openem/event_machine_macros.h>
#include <ti/runtime/openem/event_machine_types.h>
#include <ti/runtime/openem/event_machine_group.h>
#include <ti/runtime/openem/event_machine_hw_macros.h>
#include <ti/runtime/openem/event_machine_hw_types.h>
#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti/runtime/openem/event_machine_hw_ti_types.h>
#include <ti/runtime/openem/src/event_machine_hwpform.h>
#include <ti/runtime/openem/src/event_machine_private_types.h>
#include <ti/runtime/openem/src/event_machine_private_macros.h>
#include <ti/runtime/openem/src/event_machine_private_variables.h>
#include <ti/runtime/openem/src/event_machine_private_functions.h>
#include <ti/runtime/openem/modules/event_machine_modules.h>

em_status_t
ti_em_process_add_route (
    ti_em_process_route_t process_route,
    ti_em_process_id_t process_idx
)
{
    return Emti_moduleChainAddRoute (TI_EM_CHAINING_PKTDMA, (ti_em_destination_id_t) process_idx,
                                     (void *) &process_route);
}

em_status_t
ti_em_device_add_xge_route (
    ti_em_device_id_t device_idx,
    ti_em_device_xge_route_t device_route
)
{
    return Emti_moduleChainAddRoute (TI_EM_CHAINING_XGE, (ti_em_destination_id_t) device_idx, (void *) &device_route);
}

em_status_t
ti_em_device_add_rio_route (
    ti_em_device_id_t device_idx,
    ti_em_device_rio_route_t device_route
)
{
    return Emti_moduleChainAddRoute (TI_EM_CHAINING_RIO, (ti_em_destination_id_t) device_idx, (void *) &device_route);
}
