/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  event_machine_error.c
 *
 *   @brief
 *      Open Event Machine Implementation for Keystone1 devices.
 *      Functions related to error management.
 *
 */

#include <stdio.h>
#include <inttypes.h>
#include <string.h>
#include <ti/runtime/openem/event_machine_macros.h>
#include <ti/runtime/openem/event_machine_hw_macros.h>
#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti/runtime/openem/event_machine_types.h>
#include <ti/runtime/openem/event_machine_hw_types.h>
#include <ti/runtime/openem/event_machine_hw_ti_types.h>
#include <ti/runtime/openem/event_machine_sw_ti_config.h>
#include <ti/runtime/openem/src/event_machine_hwpform.h>
#include <ti/runtime/openem/src/event_machine_private_macros.h>
#include <ti/runtime/openem/src/event_machine_private_types.h>
#include <ti/runtime/openem/src/event_machine_private_variables.h>
#include <ti/runtime/openem/src/event_machine_private_functions.h>
#include <ti/runtime/openem/event_machine_group.h>


em_status_t
em_register_error_handler (
    em_error_handler_t handler
)
{
    Emti_svErrorFunctionPtr = handler;

    /*
     * ensure data are written in memory 
     */
    Emti_mfence ();

    return EM_OK;
}

em_status_t
em_unregister_error_handler (
    void
)
{
    Emti_svErrorFunctionPtr = NULL;

    /*
     * ensure data are written in memory 
     */
    Emti_mfence ();

    return EM_OK;
}

void
em_error (
    em_status_t error,
    em_escope_t escope,
    ...
)
{
    Emti_EO_s *lvEoPtr = (Emti_EO_s *) (Emti_gvEoHandle);
    /*
     * call eo error function pointer 
     */
    if ((lvEoPtr != NULL) && (lvEoPtr->errorHandler != NULL))
    {
        lvEoPtr->errorHandler ((em_eo_t) (lvEoPtr), (em_status_t) (error), (em_escope_t) (escope), NULL);
    }
    else if (Emti_svErrorFunctionPtr != NULL)
    {
        Emti_svErrorFunctionPtr (EM_EO_UNDEF, (em_status_t) (error), (em_escope_t) (escope), NULL);
    }
    /*
     * else error? 
     */
    else
    {
        return;
    }

    return;
}

int
em_error_format_string (
    char *str,
    size_t size,
    em_eo_t eo,
    em_status_t error,
    em_escope_t escope,
    va_list args
)
{
    em_status_t ret = EM_ERR_NOT_IMPLEMENTED;
    return (int) (ret);
}
