/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  event_machine_private_macros.h
 *
 *   @brief
 *      Internal private constants used by the Open EM implementation.
 *      These constants are not available for use by the application.
 *
 */

#ifndef EVENT_MACHINE_PRIVATE_MACROS_H_
#define EVENT_MACHINE_PRIVATE_MACROS_H_

#include <ti/runtime/openem/event_machine_hw_ti_macros.h>

/**
 * @internal
 *
 * Miss mask associated with Ethernet type
 */
#define Emti_XGE_RX_MISS_MASK_ETHER_TYPE_ON (1u)

/**
 * @internal
 *
 * Miss mask associated with service type
 */
#define Emti_XGE_RX_MISS_MASK_SERVICE_TYPE_ON (2u)

/** 
 * @internal
 *
 * PA Tx queue index
 */
#define Emti_PA_TX_QUEUE_IDX            (640)

/** 
 * @internal
 *
 * QMSS xge queue base index
 */
#define Emti_QMSS_XGE_QUEUE_BASE        (810u)

/** 
 * @internal
 *
 * Eight bytes align boundary
 */
#define Emti_8_BYTES_ALIGN              (8)

/** 
 * @internal
 *
 * Cache L2 Maximum Size (bytes)
 */
#define CACHE_MAX_L2_SIZE               (256*1024)

/** 
 * @internal
 *
 * Cache L1D Maximum Size (bytes) 
 */
#define CACHE_MAX_L1D_SIZE              (32*1024)

/** 
 * @internal
 *
 * event buffer cache coherency : INV enabled
 */
#define Emti_CACHE_COH_INV_EN           (1)

/** 
 * @internal
 *
 * event buffer cache coherency : WBINV enabled
 */
#define Emti_CACHE_COH_WBINV_EN         (0)

/** 
 * @internal
 *
 * external memory radix
 */
#define Emti_EXT_MEM_LOCATION           (0x80000000)

/** 
 * @internal
 *
 * Minimum size of event descriptor
 */
#define Emti_EVENT_DSC_WORD_SIZE_MIN    (12)

/** 
 * @internal
 *
 * size of event descriptor with 4 ps data in descriptor
 */
#define Emti_EVENT_DSC_WORD_SIZE_4PSDATA (16)

/** 
 * @internal
 *
 * Mask of original buffer len field in descriptor
 */
#define Emti_EVENT_DSC_ORIG_BUFF_LEN_MASK (0x3fffff)

/** 
 * @internal
 *
 * Maximum number of scheduling queues
 */
#define Emti_SD_QUEUE_NUM               (TI_EM_CORE_NUM*TI_EM_PRIO_NUM)

/** 
 * @internal
 *
 * Maximum number of scheduling queues
 */
#define Emti_SD_QUEUE_NUM_UP            (TI_EM_CORE_NUM*TI_EM_PRIO_NUM)

/** 
 * @internal
 *
 * Maximum number of atomic queues
 */
#define Emti_AP_QUEUE_NUM               (TI_EM_CORE_NUM*TI_EM_PRIO_NUM*(1+TI_EM_PF_LEN))

/** 
 * @internal
 *
 * total number of CD queue events
 */
#define Emti_CD_QUEUE_EVENT_NUM         (1)

/** 
 * @internal
 *
 * register adress for command queue index on PDSP1 (LRAM1_BASE)
 */
#define Emti_CD_QUEUE_IDX_REG_ADD_PDSP1 (0x23A20000)

/** 
 * @internal
 *
 * register adress for command queue index on PDSP2 (LRAM2_BASE)
 */
#define Emti_CD_QUEUE_IDX_REG_ADD_PDSP2 (0x23A24000)

/**
 * @internal
 *
 * Iterator state : stopped
 */
#define Emti_ITERATOR_STATE_STOPPED     (0)

/**
 * @internal
 *
 * Iterator state : start
 */
#define Emti_ITERATOR_STATE_STARTED     (1)

/**
 * @internal
 *
 * Event free mode -
 */
#define Emti_EVENT_MODE_GLOBAL_HW       (0)

/**
 * @internal
 *
 * Event free mode -
 */
#define Emti_EVENT_MODE_GLOBAL_SW       (1)

/**
 * @internal
 *
 * Event free mode -
 */
#define Emti_EVENT_MODE_LOCAL           (2)

/**
 * @internal
 *
 * Event mode position in event handle
 */
#define Emti_EVENT_HDL_EVENT_MODE_SHIFT (0x0)

/**
 * @internal
 *
 * Event mode mask in event handle
 */
#define Emti_EVENT_HDL_EVENT_MODE_MASK  (0x3)

/**
 * @internal
 *
 * Position and shift of various fields in event descriptor
 */
#define Emti_COH_MODE_SHIFT             (0)
#define Emti_COH_MODE_MASK              (0x3)
#define Emti_COH_MODE_WORD              (0)

#define Emti_DIRTY_FLAG_SHIFT           (8)
#define Emti_DIRTY_FLAG_MASK            (0x1)
#define Emti_DIRTY_FLAG_WORD            (0)

#define Emti_PACKET_WSIZE_SHIFT         (16)
#define Emti_PACKET_WSIZE_MASK          (0xff)
#define Emti_PACKET_WSIZE_WORD          (0)

#define Emti_POOL_INDEX_SHIFT           (24)
#define Emti_POOL_INDEX_MASK            (0xff)
#define Emti_POOL_INDEX_WORD            (0)

#define Emti_FREE_INFO_SHIFT            (0)
#define Emti_FREE_INFO_MASK             (0xffff)
#define Emti_FREE_INFO_WORD             (1)

#define Emti_FREE_QUEUE_INDEX_SHIFT     (0)
#define Emti_FREE_QUEUE_INDEX_MASK      (0x3fff)
#define Emti_FREE_QUEUE_INDEX_WORD      (1)

#define Emti_FREE_PUSH_POLICY_SHIFT     (14)
#define Emti_FREE_PUSH_POLICY_MASK      (0x1)
#define Emti_FREE_PUSH_POLICY_WORD      (1)

#define Emti_FREE_POLICY_SHIFT          (15)
#define Emti_FREE_POLICY_MASK           (0x1)
#define Emti_FREE_POLICY_WORD           (1)

#define Emti_BUF_MODE_SHIFT             (16)
#define Emti_BUF_MODE_MASK              (0x3)
#define Emti_BUF_MODE_WORD              (1)

#define Emti_EVENT_MODE_SHIFT           (24)
#define Emti_EVENT_MODE_MASK            (0x3)
#define Emti_EVENT_MODE_WORD            (1)

/**
 * @internal
 *
 * Mask to check if the AP event is to be sent
 */
#define Emti_AP_COMMAND_TO_SEND         (0x4)

/**
 * @internal
 *
 * Mask to check if the AP event is to be sent
 */
#define Emti_AP_COMMAND_TO_CLEAR        (0xFFFFFFFB)

/**
 * @internal
 *
 * Mask to check if the global event needs to be freed
 */
#define Emti_LOCAL_EVENT_TO_CLAIM       (0x2)

/**
 * @internal
 *
 * Mask to clear the global event free message
 */
#define Emti_GLOBAL_EVENT_TO_CLEAR      (0xFFFFFFFD)

/**
 * @internal
 *
 * Mask to check if the PF command is to be sent
 */
#define Emti_PF_COMMAND_TO_SEND         (0x1)

/**
 * @internal
 *
 * Mask to clear the PF command sending message
 */
#define Emti_PF_COMMAND_TO_CLEAR        (0xFFFFFFFE)

/**
 * @internal
 *
 * CD command type AP
 */
#define Emti_CD_COMMAND_TYPE_AP         (0x1)

/**
 * @internal
 *
 * CD command type CT
 */
#define Emti_CD_COMMAND_TYPE_CT         (0x2)

/**
 * @internal
 *
 * CD command type PF
 */
#define Emti_CD_COMMAND_TYPE_PF         (0x0)

/**
 * @internal
 *
 * Tag for no event group
 */
#define Emti_NO_EVENT_GROUP_TAG         (0xFFFF0000)

/**
 * @internal
 *
 * Tag when an event group is present
 */
#define Emti_EVENT_GROUP_TAG            (0xFFFF)

/**
 * @internal
 *
 * Shift to retrieve the event group tag
 */
#define Emti_EVENT_GROUP_TAG_SHIFT      (0x10)

/**
 * @internal
 *
 * Mask for event group id: only the 16 LSB are conveyed
 */
#define Emti_EVENT_GROUP_ID_MASK        (0x0000FFFF)

/**
 * @internal
 *
 * Mask for event queue handle: only the 24 LSB are conveyed
 */
#define Emti_EQ_HANDLE_MASK             (0x00FFFFFF)

/**
 * @internal
 *
 * Mask for event type: only the 8 MSB are conveyed
 */
#define Emti_EVENT_TYPE_MASK            (0xFF000000)

/**
 * @internal
 *
 * Macros for accessing 64bit field in big and little endian
 */
#if defined(_BIG_ENDIAN)
#define Emti_LOW_LL(ll)                 (_hill(ll))
#define Emti_HIGH_LL(ll)                (_loll(ll))
#elif defined(_LITTLE_ENDIAN)
#define Emti_LOW_LL(ll)                 (_loll(ll))
#define Emti_HIGH_LL(ll)                (_hill(ll))
#else
#error "No endianness specified!"
#endif

/**
 * @internal
 *
 * Stream is not active.
 */
#define     Emti_STREAM_NOT_ACTIVE      (0u)

/**
 * @internal
 *
 * Stream is active.
 */
#define     Emti_STREAM_ACTIVE          (1u)

/** 
 * @internal
 *
 * Maximum number of chaining module
 */
#define     Emti_CHAIN_MODULE_NUM       (4u)

/**
 * @internal
 *
 * Descriptor hint for pushing to CH TX queue (192 bytes)
 */
#define     Emti_DSC_HINT_CH            (11u)

/**
 * @internal
 *
 * Descriptor hint for pushing to undefined TX queue (192 bytes)
 */
#define     Emti_DSC_HINT_MAX           (11u)

/**
 * @internal
 *
 * Descriptor hint for pushing to PL TX queue (192 bytes)
 */
#define     Emti_DSC_HINT_PL            (11u)

/**
 * @internal
 *
 * Descriptor hint for pushing to RIO TX queue (48 bytes)
 */
#define     Emti_DSC_HINT_RIO           (11u)

/**
 * @internal
 *
 * Descriptor hint for pushing to RS TX queue (32 bytes)
 */
#define     Emti_DSC_HINT_RS            (2u)

/**
 * @internal
 *
 * Descriptor hint for pushing to XGE TX queue (32 bytes)
 */
#define     Emti_DSC_HINT_XGE           (2u)

/**
 * @internal
 *
 * Number of QM queues.
 */
#define Emti_QM_QUEUE_NUM               (16384u)

#define Emti_QUEUE_HANDLE_QUEUE_IDX_MAX            0x00003FFF
#define Emti_QUEUE_HANDLE_SD_QUEUE_IDX_MAX         0x0000007F

/**
 * @internal
 *
 * Selects bits of queue index in set for hashing.
 */
#define Emti_HASH_QUEUE_IDX_IN_SET_MASK (TI_EM_CHAIN_TX_QUEUE_NUM-1)

/**
 * @internal
 *
 * Selects bits of queue set index for hashing.
 */
#define Emti_HASH_QUEUE_SET_IDX_MASK (TI_EM_CHAIN_TX_QUEUE_NUM-1)

/**
 * @internal
 *
 * Queue associated with HW queue.
 */
#define Emti_COMMAND_HANDLE_COMMAND_TYPE_MASK       0x00000003
#define Emti_COMMAND_HANDLE_COMMAND_TYPE_SHIFT      0x00000000
#define Emti_COMMAND_HANDLE_DSC_PTR_MASK            0xFFFFFFF0
#define Emti_COMMAND_HANDLE_DSC_PTR_SHIFT           0x00000004

#define Emti_DEVICE_ROUTE_INFO_ROUTE_TYPE_MASK      0x0000FFFE
#define Emti_DEVICE_ROUTE_INFO_ROUTE_TYPE_SHIFT     0x00000001
#define Emti_DEVICE_ROUTE_INFO_ENABLED_MASK         0x00000001
#define Emti_DEVICE_ROUTE_INFO_ENABLED_SHIFT        0x00000000

#define Emti_EVENT_HANDLE_EVENT_MODE_MASK           0x00000003
#define Emti_EVENT_HANDLE_EVENT_MODE_SHIFT          0x00000000
#define Emti_EVENT_HANDLE_DSC_PTR_MASK              0xFFFFFFF0
#define Emti_EVENT_HANDLE_DSC_PTR_SHIFT             0x00000004

#define Emti_FREE_INFO_QUEUE_IDX_MASK               0x00003FFF
#define Emti_FREE_INFO_QUEUE_IDX_SHIFT              0x00000000
#define Emti_FREE_INFO_PUSH_POLICY_MASK             0x00004000
#define Emti_FREE_INFO_PUSH_POLICY_SHIFT            0x0000000E
#define Emti_FREE_INFO_FREE_POLICY_MASK             0x00008000
#define Emti_FREE_INFO_FREE_POLICY_SHIFT            0x0000000F

#define Emti_QUEUE_HANDLE_DEVICE_IDX_MASK           0xFF000000
#define Emti_QUEUE_HANDLE_DEVICE_IDX_SHIFT          0x00000018
#define Emti_QUEUE_HANDLE_PROCESS_IDX_MASK          0x0000C000
#define Emti_QUEUE_HANDLE_PROCESS_IDX_SHIFT         0x0000000E
#define Emti_QUEUE_HANDLE_QUEUE_IDX_MASK            0x00003FFF
#define Emti_QUEUE_HANDLE_QUEUE_IDX_SHIFT           0x00000000
#define Emti_QUEUE_HANDLE_QUEUE_SET_IDX_MASK        0x000000FF  /*    sub field of queueIdx    */
#define Emti_QUEUE_HANDLE_QUEUE_SET_IDX_SHIFT       0x00000000  /*    sub field of queueIdx    */
#define Emti_QUEUE_HANDLE_QUEUE_IDX_IN_SET_MASK     0x00003F00  /*    sub field of queueIdx    */
#define Emti_QUEUE_HANDLE_QUEUE_IDX_IN_SET_SHIFT    0x00000008  /*    sub field of queueIdx    */

#define Emti_QUEUE_INFO_QUEUE_IDX_MASK              0x00003FFF
#define Emti_QUEUE_INFO_QUEUE_IDX_SHIFT             0x00000000
#define Emti_QUEUE_INFO_QUEUE_TYPE_MASK             0x00008000
#define Emti_QUEUE_INFO_QUEUE_TYPE_SHIFT            0x0000000F

#define Emti_QUEUE_ROUTE_INFO_ROUTE_ENABLED_MASK    0x00008000
#define Emti_QUEUE_ROUTE_INFO_ROUTE_ENABLED_SHIFT   0x0000000F
#define Emti_QUEUE_ROUTE_INFO_HW_QUEUE_IDX_MASK     0x00003FFF
#define Emti_QUEUE_ROUTE_INFO_HW_QUEUE_IDX_SHIFT    0x00000000
#define Emti_QUEUE_ROUTE_INFO_QUEUE_MODE_MASK       0x00004000
#define Emti_QUEUE_ROUTE_INFO_QUEUE_MODE_SHIFT      0x0000000E
#define Emti_QUEUE_ROUTE_INFO_QUEUE_TYPE_MASK       0x00000100
#define Emti_QUEUE_ROUTE_INFO_QUEUE_TYPE_SHIFT      0x00000008
#define Emti_QUEUE_ROUTE_INFO_SD_QUEUE_IDX_MASK     0x0000007F
#define Emti_QUEUE_ROUTE_INFO_SD_QUEUE_IDX_SHIFT    0x00000000

#define Emti_XGE_CHAIN_HDR_DST_INFO_QUEUE_IDX_MASK      0x00003FFF
#define Emti_XGE_CHAIN_HDR_DST_INFO_QUEUE_IDX_SHIFT     0x00000000
#define Emti_XGE_CHAIN_HDR_DST_INFO_PROCESS_IDX_MASK    0x0000C000
#define Emti_XGE_CHAIN_HDR_DST_INFO_PROCESS_IDX_SHIFT   0x0000000E

#define Emti_RIO_CHAIN_HDR_DST_INFO_QUEUE_IDX_MASK      0x00003FFF
#define Emti_RIO_CHAIN_HDR_DST_INFO_QUEUE_IDX_SHIFT     0x00000000
#define Emti_RIO_CHAIN_HDR_DST_INFO_PROCESS_IDX_MASK    0x0000C000
#define Emti_RIO_CHAIN_HDR_DST_INFO_PROCESS_IDX_SHIFT   0x0000000E
#define Emti_RIO_CHAIN_HDR_TPT_INFO_ERROR_CODE_MASK     0x00000003
#define Emti_RIO_CHAIN_HDR_TPT_INFO_ERROR_CODE_SHIFT    0x00000000
#define Emti_RIO_CHAIN_HDR_TPT_INFO_COS_MASK_MASK       0x00000001
#define Emti_RIO_CHAIN_HDR_TPT_INFO_COS_MASK_SHIFT      0x00000000
#define Emti_RIO_CHAIN_HDR_TPT_INFO_TPT_TYPE_MASK       0x00000002
#define Emti_RIO_CHAIN_HDR_TPT_INFO_TPT_TYPE_SHIFT      0x00000001

#define Emti_PROCESS_ROUTE_INFO_ROUTE_ENABLED_MASK  0x00000001
#define Emti_PROCESS_ROUTE_INFO_ROUTE_ENABLED_SHIFT 0x00000000
#define Emti_PROCESS_ROUTE_INFO_TX_QUEUE_IDX_MASK   0x0000FFFE
#define Emti_PROCESS_ROUTE_INFO_TX_QUEUE_IDX_SHIFT  0x00000001

#define Emti_EVENT_DSC_W1_SRC_TAG_HI_MASK           0xFF000000
#define Emti_EVENT_DSC_W1_SRC_TAG_HI_SHIFT          0x00000018
#define Emti_EVENT_DSC_W1_SRC_TAG_LO_MASK           0x00FF0000
#define Emti_EVENT_DSC_W1_SRC_TAG_LO_SHIFT          0x00000010
#define Emti_EVENT_DSC_W1_DST_TAG_HI_MASK           0x0000FF00
#define Emti_EVENT_DSC_W1_DST_TAG_HI_SHIFT          0x00000008
#define Emti_EVENT_DSC_W1_DST_TAG_LO_MASK           0x000000FF
#define Emti_EVENT_DSC_W1_DST_TAG_LO_SHIFT          0x00000000

#define Emti_EVENT_DSC_W2_PS_FLAG_MASK              0x000F0000
#define Emti_EVENT_DSC_W2_PS_FLAG_SHIFT             0x00000010

#define Emti_EVENT_DSC_W8_EVENT_TYPE_MASK           0xFF000000
#define Emti_EVENT_DSC_W8_EVENT_TYPE_SHIFT          0x00000018
#define Emti_EVENT_DSC_W8_QUEUE_TAG_MASK            0x00FF0000
#define Emti_EVENT_DSC_W8_QUEUE_TAG_SHIFT           0x00000010
#define Emti_EVENT_DSC_W8_QUEUE_TYPE_MASK           0x00010000
#define Emti_EVENT_DSC_W8_QUEUE_TYPE_SHIFT          0x00000010
#define Emti_EVENT_DSC_W8_QUEUE_IDX_MASK            0x00003FFF
#define Emti_EVENT_DSC_W8_QUEUE_IDX_SHIFT           0x00000000

#define Emti_EVENT_TAG_RIO                          30
#define Emti_EVENT_SIZE_EVENT_TAG_MASK              0x3E000000
#define Emti_EVENT_SIZE_EVENT_TAG_SHIFT             25

/* Field Make */
#define Emti_makeField(PER_REG_FIELD, val)                                         \
    (((val) << Emti_##PER_REG_FIELD##_SHIFT) & Emti_##PER_REG_FIELD##_MASK)

/* the Field Get macro */
#define Emti_getField(reg, PER_REG_FIELD)                                        \
    (((reg) & Emti_##PER_REG_FIELD##_MASK) >> Emti_##PER_REG_FIELD##_SHIFT)

/* the Field Set macro */
#define Emti_setField(reg, PER_REG_FIELD, val)                                   \
    ((reg) = ((reg) & ~Emti_##PER_REG_FIELD##_MASK)                          \
    | Emti_makeField(PER_REG_FIELD, val))

/**
 * Macros for defining per-core variables
 */
#ifdef TI_EM_THREAD_MODEL
#ifdef TI_EM_HAS___THREAD
#define Emti_LOCAL_VARIABLE(type, var)   __thread type var
#else /* TI_EM_HAS___THREAD */
#define Emti_LOCAL_VARIABLE(type, var)   type __attribute__ ((aligned(CACHE_L2_LINESIZE))) __##var##Array[TI_EM_CORE_NUM]
#endif /* TI_EM_HAS___THREAD */
#else /* TI_EM_THREAD_MODEL */
#define Emti_LOCAL_VARIABLE(type, var)   type var
#endif /* TI_EM_THREAD_MODEL */

/**
 * Macros for defining global variables shared in r/w mode
 */
#ifdef TI_EM_LINUX
#ifdef TI_EM_THREAD_MODEL
#define Emti_GLOBAL_VARIABLE(type, var) type var
#define Emti_GLOBAL_INIT(var, size)     &(var)
#else /* TI_EM_THREAD_MODEL */
#define Emti_GLOBAL_VARIABLE(type, var)
#define Emti_GLOBAL_INIT(var, size)     (typeof(var)*) Emti_sharedMemAlloc(size)
#endif /* TI_EM_THREAD_MODEL */
#else /* TI_EM_LINUX */
#define Emti_GLOBAL_VARIABLE(type, var)  type var
#define Emti_GLOBAL_INIT(var, size)      &(var)
#endif /* TI_EM_LINUX */

#endif /*EVENT_MACHINE_PRIVATE_MACROS_H_ */
