/*
 * Copyright (c) 2012, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 *   @file  event_machine_private_types.h
 *
 *   @brief
 *      Internal private types used by the Open EM implementation.
 *      These types are not available for use by the application.
 *
 */

#ifndef EVENT_MACHINE_PRIVATE_TYPES_H_
#define EVENT_MACHINE_PRIVATE_TYPES_H_

/* forward declarations */

typedef struct Emti_PaHdrRioFormat_s_ Emti_PaHdrRioFormat_s;
typedef struct Emti_EO_s_ Emti_EO_s;
typedef struct Emti_EventDsc_s_ Emti_EventDsc_s;
typedef struct Emti_DvEventDsc_s_ Emti_DvEventDsc_s;
typedef struct Emti_CdEventDsc_s_ Emti_CdEventDsc_s;
typedef struct Emti_PfEventDsc_s_ Emti_PfEventDsc_s;
typedef struct Emti_QueueSlow_s_ Emti_QueueSlow_s;
typedef struct Emti_QueueFast_s_ Emti_QueueFast_s;
typedef struct Emti_EventGroup_s_ Emti_EventGroup_s;
typedef struct Emti_QueueGroup_s_ Emti_QueueGroup_s;
typedef struct Emti_Eo64b_s_ Emti_Eo64b_s;
typedef struct Emti_Iterator_s_ Emti_Iterator_s;

typedef struct Emti_ApCoreState_s_ Emti_ApCoreState_s;
typedef struct Emti_DeviceRoute_s_ Emti_DeviceRoute_s;
typedef struct Emti_QueueRoute_s_ Emti_QueueRoute_s;
typedef struct Emti_ProcessRoute_s_ Emti_ProcessRoute_s;
typedef struct Emti_StreamState_s_ Emti_StreamState_s;
typedef struct Emti_SdQueueState_s_ Emti_SdQueueState_s;
typedef struct Emti_HwQueueState_s_ Emti_HwQueueState_s;

typedef struct Emti_Pcb_s_ Emti_Pcb_s;
typedef struct Emti_ChainPcb_s_ Emti_ChainPcb_s;
typedef struct Emti_ChainXgePcb_s_ Emti_ChainXgePcb_s;
typedef struct Emti_ChainRioPcb_s_ Emti_ChainRioPcb_s;
typedef struct Emti_ChainPoststorePcb_s_ Emti_ChainPoststorePcb_s;
typedef struct Emti_PreloadPcb_s_ Emti_PreloadPcb_s;

typedef struct Emti_Rpcb_s_ Emti_Rpcb_s;
typedef struct Emti_ChainRioRpcb_s_ Emti_ChainRioRpcb_s;
typedef struct Emti_Rdcb_s_ Emti_Rdcb_s;

typedef struct Emti_Spcb_s_ Emti_Spcb_s;
typedef struct Emti_PreloadSpcb_s_ Emti_PreloadSpcb_s;


typedef struct Emti_Tcb_s_ Emti_Tcb_s;
typedef struct Emti_ChainXgeTcb_s_ Emti_ChainXgeTcb_s;
typedef struct Emti_ChainRioTcb_s_ Emti_ChainRioTcb_s;
typedef struct Emti_ChainPoststoreTcb_s_ Emti_ChainPoststoreTcb_s;
typedef struct Emti_PreloadTcb_s_ Emti_PreloadTcb_s;

typedef struct Emti_Psb_s_ Emti_Psb_s;
typedef struct Emti_Psb_slow_s_ Emti_Psb_slow_s;

typedef struct Emti_Rpsb_s_ Emti_Rpsb_s;
typedef struct Emti_Rdsb_s_ Emti_Rdsb_s;

typedef struct Emti_Spsb_s_ Emti_Spsb_s;

typedef struct Emti_Tsb_s_ Emti_Tsb_s;
typedef struct Emti_ChainPoststoreTsb_s_ Emti_ChainPoststoreTsb_s;

typedef struct Emti_Esb_s_ Emti_Esb_s;

#include <inttypes.h>
#include <ti/runtime/openem/event_machine_group.h>
#include <ti/runtime/openem/event_machine_sw_config.h>
#include <ti/runtime/openem/event_machine_hw_ti_macros.h>
#include <ti/runtime/openem/event_machine_hw_ti_types.h>
#include <ti/runtime/openem/src/event_machine_hwpform.h>
#include <ti/runtime/openem/src/event_machine_private_macros.h>

#ifndef EM_CHECK_LEVEL
#error Missing error level definition. Define EM_CHECK_LEVEL value!
#endif

#ifndef TI_EM_TRACE_LEVEL
#error Missing trace level definition. Define TI_EM_TRACE_LEVEL value!
#endif

/**
 * @internal
 *
 * Queue mode.
 * Identifies the queue mode.
 *
 */
typedef uint32_t Emti_QueueMode_t;

/**
 * @internal
 *
 * Event mode.
 * Identifies the event mode.
 *
 */
typedef uint32_t Emti_event_mode_t;

/**
 * @internal
 *
 * @brief
 *  The structure describes the states of the Execution Object
 *
 * @details
 *  Execution Objects can be in either of the following states i.e. Free,
 *  Allocated or Started. Execution objects receive handlers can only be
 *  executed once the EO has been started.
 */
typedef enum Emti_EOState_e_
{
    /**
     * @brief   The event object is not being used.
     */
    Emti_EOState_FREE = 0x0,

    /**
     * @brief   The event object has been allocated but has not been started
     */
    Emti_EOState_ALLOCATED = 0x1,

    /**
     * @brief   The event object has been started and can be used
     */
    Emti_EOState_STARTED = 0x2
} Emti_EOState_e;

/**
 * @internal
 *
 * @brief
 *  The structure describes the types of the CD queue commands
 *
 * @details
 *  Command to be sent to the pDSP are of the types listed below
 */
typedef enum Emti_CdQueueCommandType_e_
{
    /**
     * @brief   NULL command type
     */
    Emti_CtType_NULL = -1,

    /**
     * @brief   pDSP global init
     */
    Emti_CtType_SCHEDULER_INIT = 0,

    /**
     * @brief   pDSP local init
     */
    Emti_CtType_LOCAL_START = 1,

    /**
     * @brief   maskUpdate
     */
    Emti_CtType_SCHEDULER_UPDATE_MASK = 2,

    /**
     * @brief   process init
     */
    Emti_CtType_ROUTER_CONNECT = 3,

    /**
     * @brief   maskUpdate
     */
    Emti_CtType_SCHEDULER_UPDATE_INTERRUPT = 4,

    /**
     * @brief   host process init
     */
    Emti_CtType_ROUTER_CONNECT_XGE = 5,

    /**
     * @brief   route update
     */
    Emti_CtType_ROUTER_UPDATE_ROUTE = 6,

    /**
     * @brief   static route update
     */
    Emti_CtType_ROUTER_UPDATE_ROUTE_STATIC = 7,

    /**
     * @brief   process init
     */
    Emti_CtType_ROUTER_CONNECT_RIO = 8,

    /**
     * @brief   init router
     */
    Emti_CtType_ROUTER_INIT = 9,

    /**
     * @brief   counter get
     */
    Emti_CtType_ROUTER_COUNTER_GET = 10,
    
    /**
     * @brief   rx miss enable
     */
    Emti_CtType_ROUTER_ENABLE_RX_MISS = 11,

    /**
     * @brief   rx miss disable
     */
    Emti_CtType_ROUTER_DISABLE_RX_MISS = 12,

    /**
     * @brief   scheduler/router kill
     */
    Emti_CtType_KILL = 0x66
} Emti_CdQueueCommandType_e;

typedef enum Emti_CdQueueErrorCode_e_
{
    /**
     * @brief   No Error Answer
     */
    Emti_CdQueueErrorCode_NO_ERROR = 0,

    /**
     * @brief   Firmware was expecting GLOBAL_INIT
     *          as the unique possible first command
     */
    Emti_CdQueueErrorCode_UNEXPECTED_CMD = 1,

    /**
     * @brief   Firmware decoded an undefined command type
     */
    Emti_CdQueueErrorCode_INVALID_CMD_ID = 2,

    /**
     * @brief   GLOBAL_INIT was already executed by Firmware
     */
    Emti_CdQueueErrorCode_ALREADY_EXECUTED = 3,

    /**
     * @brief   Default container value when DSP sends command
     *          to pDSP Firmware
     */
    Emti_CdQueueErrorCode_WAITING_CMD = 0x0BAD00CD
} Emti_CdQueueErrorCode_e;

/**
 * @internal
 *
 * @brief
 *  The structure describes the states of the  Queue
 *
 * @details
 *  queues can be either of the following states i.e.
 *  Free, Allocated or Mapped.  queues can only be used
 *  to send events once they are in the mapped state i.e. they
 *  have been associated with an EO.
 */
typedef enum Emti_QueueState_e_
{
    /**
     * @brief   The queue is not being used.
     */
    Emti_QueueState_FREE = 0x0,

    /**
     * @brief   The queue has been allocated but has not been mapped
     * to an event object
     */
    Emti_QueueState_ALLOCATED = 0x1,

    /**
     * @brief   The queue has been mapped to an event object.
     */
    Emti_QueueState_MAPPED = 0x2
} Emti_QueueState_e;

/**
 * @internal
 *
 * @brief
 *  The structure describes the states of the Event group
 *
 * @details
 *  Event Groups are either Free, Allocated or Applied.
 */
typedef enum Emti_EventGroupState_e_
{
    /**
     * @brief   Event group is available.
     */
    Emti_EventGroupState_FREE = 0x0,

    /**
     * @brief   Event group is allocated but it has not been associated
     * with the notification table.
     */
    Emti_EventGroupState_ALLOCATED = 0x1,

    /**
     * @brief   Event group is allocated & has been associated with the
     * notification table.
     */
    Emti_EventGroupState_APPLIED = 0x2
} Emti_EventGroupState_e;

/**
 * @internal
 *
 * @brief
 *  The structure describes the states of the Queue group
 *
 * @details
 *  Queue Groups are either Free, Allocated or Applied.
 */
typedef enum Emti_QueueGroupState_e_
{
    /**
     * @brief   Queue group is available.
     */
    Emti_QueueGroupState_FREE = 0x0,

    /**
     * @brief   Queue group is allocated but it has not been associated
     * with the notification table.
     */
    Emti_QueueGroupState_ALLOCATED = 0x1,

    /**
     * @brief   Queue group is enabled & has been associated with the
     * notification table.
     */
    Emti_QueueGroupState_ENABLED = 0x2
} Emti_QueueGroupState_e;

#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (Emti_Eo64b_s_, 8);
#pragma STRUCT_ALIGN (Emti_Eo64b_s, 8)
#endif
struct Emti_Eo64b_s_
{
    /**
     * @brief   Receive Function.
     */
    em_receive_func_t receive;

    /**
     * @brief   Pointer to the instance data.
     */
    void *contextPtr;
#ifdef TI_EM_GCC
} __attribute__ ((aligned (8)));
#else                           /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @internal
 *
 * @brief
 *  The structure describes the PA Header object for RIO format
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  PA header when the format mapped the RIO format.
 */
struct Emti_PaHdrRioFormat_s_
{
    uint16_t dstNodeIdx;
    uint16_t srcNodeIdx;
    uint8_t serviceClass;
    uint8_t transportInfo;
    uint16_t dstInfo;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the Execution Object
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  Execution object.
 */
struct Emti_EO_s_
{
    /**
     * @brief   Name of the event object
     */
    char name[EM_EO_NAME_LEN];

    /**
     * @brief   The state of the execution object
     */
    Emti_EOState_e state;

    Emti_Eo64b_s eo64b;

    /**
     * @brief   Start Function.
     */
    em_start_func_t start;

    /**
     * @brief   Local Start Function.
     */
    em_start_local_func_t localStart;

    /**
     * @brief   Stop Function.
     */
    em_stop_func_t stop;

    /**
     * @brief   Local Stop Function.
     */
    em_stop_local_func_t localStop;

    /**
     * @brief   EO error function pointer.
     */
    em_error_handler_t errorHandler;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the Atomic-Processing EventDsc
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  Atomic-Processing EventDsc.
 */
struct Emti_EventDsc_s_
{
    /**
     * @brief   Pointer to the associated queue
     */
    Emti_QueueSlow_s *queuePtr;

    /**
     * @brief   Pointer to the associated queue
     */
    uint32_t atomicQueueIdx;

    uint32_t pad[2];
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the DiVert (DV) EventDsc
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  Divert queue to use
 */
struct Emti_DvEventDsc_s_
{
    /**
     * @brief   Pointer to the associated queue
     */
    uint32_t txDivertQueueIdx;

    uint32_t pad[3];
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the iterator
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  iterator.
 */
struct Emti_Iterator_s_
{
    /**
     * @brief    Identifies the event with which the iterator is associated
     */
    uint32_t event_hdl;

    /**
     * @brief   Pointer to the second buffer descriptor of the associated event
     */
    uint32_t coh_desc_ptr;

    /**
     * @brief   current buffer descriptor pointer
     */
    uint32_t current_desc_ptr;

    /**
     * @brief   current buffer descriptor buffer pointer
     */
    uint32_t current_buf_ptr;

    /**
     * @brief   current buffer descriptor buffer size
     */
    uint32_t current_buf_size;

    /**
     * @brief   accumulated buffer size from packet descriptor to current descriptor
     */
    uint32_t current_size;

    /**
     * @brief   current buffer descriptor level
     */
    uint32_t current_level;

    /**
     * @brief   coherency level
     */
    uint32_t coh_level;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the CommanD queue Event Descriptor
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  CommanD queue Event Descriptor.
 */
struct Emti_CdEventDsc_s_
{
    /**
     * @brief   command type
     */
    Emti_CdQueueCommandType_e commandType;  /* Is splitted into ObjectIdx(16MSB) and ctType(16LSB) in case of router fw */

    /**
     * @brief   Command pointer
     */
    uint32_t commandPtr;

    /**
     * @brief   Error code returned by FW
     */
    Emti_CdQueueErrorCode_e errorCode;

    /**
     * @brief   Default return queue if not defined in FW database
     */
    uint32_t returnQueue;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the PreFetch queue Event Descriptor
 *
 * @details
 *  The structure is used to hold the information which describes the
 *  PreFetch queue Event Descriptor.
 */
struct Emti_PfEventDsc_s_
{
    /**
     * @brief   CPU core index
     */
    uint8_t coreIdx;

    /**
     * @brief   padding to align with PDSP structure for Big Endian
     */
    uint8_t pad1[3];

    /**
     * @brief   Pointer to the ECB
     */
    uint32_t ecbPtr;

    uint32_t pad[2];
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the queue in fast memory
 *
 * @details
 *  Each event queue in the event machine is mapped internally to this 
 *  queue data structure.
 */
#ifndef TI_EM_GCC
#pragma STRUCT_ALIGN (Emti_QueueFast_s_, 8);
#pragma STRUCT_ALIGN (Emti_QueueFast_s, 8)
#endif
struct Emti_QueueFast_s_
{
    /**
     * @brief   Receive Function.
     */
    em_receive_func_t receive;

    /**
     * @brief   Context of the queue.
     */
    void *contextPtr;
#ifdef TI_EM_GCC
} __attribute__ ((aligned (8)));
#else                           /* TI_EM_GCC */
};
#endif /* TI_EM_GCC */

/**
 * @internal
 *
 * @brief
 *  The structure describes the queue in slow memory
 *
 * @details
 *  Each event queue in the event machine is mapped internally to this 
 *  queue data structure.
 */
struct Emti_QueueSlow_s_
{
    /**
     * @brief   Name of the queue object
     */
    char name[EM_QUEUE_NAME_LEN];

    /**
     * @brief    Queue state
     */
    Emti_QueueState_e state;

    /**
     * @brief   Type of the Queue: Atomic, Parallel or Parallel Ordered.
     */
    em_queue_type_t type;

    /**
     * @brief   Hardware Scheduling Queue to which the queue is mapped.
     */
    Emti_QueueMode_t queueMode;

    /**
     * @brief   Pointer to the queue group
     */
    em_queue_group_t queueGroupPtr;

    /**
     * @brief   Priority of the queue.
     */
    em_queue_prio_t priority;

    /**
     * @brief   Execution Object to which this queue is mapped to.
     */
    Emti_EO_s *eoPtr;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the event group.
 *
 * @details
 *  Multiple events can be grouped together to form an event group and
 *  once all the grouped events are completed a notification event can
 *  be sent out.
 */
struct Emti_EventGroup_s_
{
    /**
     * @brief   Event Group state
     */
    Emti_EventGroupState_e state;

    /**
     * @brief   Number of events which have been grouped together.
     */
    uint32_t eventNum;

    /**
     * @brief   Number of notification events to be sent
     */
    int32_t notifNum;

    /**
     * @brief   Notification Table
     */
    em_notif_t notifTbl[EM_EVENT_GROUP_MAX_NOTIF];
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the event group.
 *
 * @details
 *  Multiple queues can be grouped together to form an queue group.
 *  A mask specified on which core(s), queue events can be executed.
 */
struct Emti_QueueGroup_s_
{
    /**
     * @brief   Name of the queue object
     */
    char name[EM_QUEUE_GROUP_NAME_LEN];

    /**
     * @brief   Event Group state
     */
    Emti_QueueGroupState_e state;

    /**
     * @brief   Number of events which have been grouped together.
     */
    em_core_mask_t coreMskTbl;

    /**
     * @brief   SD allocated queues.
     */
    Qmss_QueueHnd sdQueueTbl[TI_EM_PRIO_NUM];
};

struct Emti_SdQueueState_s_
{
    /**
     * @brief   .
     */
    int16_t atomicSetIdx;

    /**
     * @brief   .
     */
    int16_t nextFreeIdxInAtomicSet;

    /**
     * @brief   .
     */
    int16_t nextFreeIdxInParallelSet;

    /**
     * @brief   .
     */
    int16_t parallelSetIdx;
};

struct Emti_HwQueueState_s_
{
    /**
     * @brief   Identifies the current queue set. There is no current queue set if equal to "-1".
     */
    int16_t setIdx;
};

struct Emti_ApCoreState_s_
{
    /**
     * @brief   .
     * 0 - 31..16
     */
    uint16_t previousApEventPtr;

    /**
     * @brief   .
     * 0 - 15..0
     */
    uint16_t currentApEventPtr;

    /**
     * @brief   .
     * 1 - 31..24
     */
    uint8_t reserved0;

    /**
     * @brief   .
     * 1 - 23..16
     */
    uint8_t localityState;

    /**
     * @brief   .
     * 1 - 15..8
     */
    uint8_t previousSdOffset;

    /**
     * @brief   .
     * 1 - 7..0
     */
    uint8_t currentSdOffset;

    /**
     * @brief   .
     * 2 31..0
     */
    uint32_t currentQueueHdl;

    /**
     * @brief   .
     * 3 31..0
     */
    uint32_t reserved1;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the lookup table for process routing
 *
 * @details
 *
 */
struct Emti_ProcessRoute_s_
{
    /*
     * [   15 |   1   ]
     *
     * [txQueueBaseIdx|enabled]    
     */
    uint16_t routeInfo;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the lookup table for device routing (using XGE)
 *
 * @details
 *
 */
struct Emti_DeviceRoute_s_
{
    union
    {
        /*
         * [   15 |   1   ] 
         *
         * [chainingType|enabled]    
         */
        uint8_t routeInfo;

        /*
         * chaining specific-data
         */
        uint8_t privateData[16];
    } u;
};

/**
 * @internal
 *
 * @brief
 *  The structure describes the lookup table for queue routing
 *
 * @details
 *
 */
struct Emti_QueueRoute_s_
{
    /*
     * queueMode == TI_EM_QUEUE_MODE_SD                                      
     */
    /*
     * [      1     |    1    |    5   |    1    |   1    |     7    ]      
     */
    /*
     * [routeEnabled|queueMode|reserved|queueType|reserved|sdQueueIdx]      
     */

    /*
     * queueMode == Emti_QUEUE_MODE_QM                                      
     */
    /*
     * [      1     |    1    |               14                     ]      
     */
    /*
     * [routeEnabled|queueMode|          hwQueueIdx                  ]      
     */
    uint16_t routeInfo;
};

struct Emti_StreamState_s_
{
    /*
     * [ 7 |   1   ]  
     */
    /*
     * [Rsd|enabled]  
     */
    uint8_t stateInfo;
    uint8_t active;
    /*
     * [ 4 |     4     ]    
     */
    /*
     * [Rsd|eventSeqIdx]    
     */
    uint8_t eventSeqIdx;
    uint8_t fragmentSeqIdx;
    uint16_t eventSize;
    uint16_t reserved;
    uint32_t firstFragmentPtr;
    uint32_t lastFragmentPtr;
};

struct Emti_ChainPcb_s_
{
    /**
     * @brief   identifies the CT command free queue.
     */
    ti_em_queue_id_t ctCommandFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_pdsp_id_t pdspIdx;

    /**
     * @brief   .
     */
    em_pool_id_t poolIdxSizeA;

    /**
     * @brief   .
     */
    em_pool_id_t poolIdxSizeB;

    /**
     * @brief   .
     */
    em_pool_id_t poolIdxSizeC;

    /**
     * @brief   .
     */
    em_pool_id_t poolIdxSizeD;

    /**
     * @brief   .
     */
    em_pool_id_t poolIdxOverflow;

    /**
     * @brief   .
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rpQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txQueueBaseIdx;

    /**
     * @brief   .
     */
    uint8_t moduleChainEnabled[Emti_CHAIN_MODULE_NUM];

    /**
     * @brief   . 
     */
    int32_t dma_idx;

    /**
     * @brief   . 
     */
    ti_em_queue_id_t dma_queue_base_idx;

    /**
     * @brief   .
     */
    uint8_t poststoreEnabled;
};

struct Emti_ChainXgePcb_s_
{
    /**
     * @brief   identifies the router device command queue.
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   identifies the CT command free queue.
     */
    ti_em_queue_id_t ctCommandFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_dma_id_t dmaIdx;

    /**
     * @brief   Base Address for the DV event dscs.
     */
    uint32_t dvEventDscBasePtr;

    /**
     * @brief   Identifies the free queue for the DV events
     */
    ti_em_queue_id_t dvFreeQueueIdx;

    /**
     * @brief   Specifies the Ethernet type. Must be the same for all processes on a single device.
     */
    uint8_t etherType[2];

    /**
     * @brief   Specifies the MAC address of the device on which this OpenEM instance is running. Must be the same for all processes on a single device
     */
    uint8_t myMacAddress[6];

    /**
     * @brief   Identifies a PDSP running chaining firmware. Must be the same for all processes on a single device
     */
    ti_em_pdsp_id_t pdspIdx;

    /**
     * @brief   Identifies the RP queue for the router
     */
    ti_em_queue_id_t rpQueueIdx;

    /**
     * @brief   Identifies the free queue for RX fragments. Must be the same for all processes on a single device
     */
    ti_em_queue_id_t rxFragmentFreeQueueIdx;

    /**
     * @brief   Identifies the free queue for RX headers. Must be the same for all processes on a single device
     */
    ti_em_queue_id_t rxHeaderFreeQueueIdx;

    /**
     * @brief   Identifies the first of TI_EM_XGE_VLAN_PRIO_NUM XGE RX miss queues
     */
    ti_em_queue_id_t rxMissQueueBaseIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueBaseIdx;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM XGE TX divert queues
     */
    ti_em_queue_id_t txDivertQueueBaseIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txFragmentFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txHeaderFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txQueueBaseIdx;

    /**
     * @brief   Selects the VLAN priorities that are used for receiving events from XGE. Must be the same for all processes on a single device
     */
    uint8_t vlanPrioMask;
    
    /**
     * @brief   Specifies the service type. Must be the same for all processes on a single device
     */
    uint8_t serviceType;
    
};

struct Emti_ChainXgeTcb_s_
{
    /**
     * @brief   .
     */
    uint8_t etherType[2];

    /**
     * @brief   .
     */
    uint8_t myMacAddress[6];

    /**
     * @brief   Identifies the free queue for the DV events
     */
    ti_em_queue_id_t dvFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txFragmentFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txHeaderFreeQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txQueueBaseIdx;
    
    /**
     * @brief   Specifies the service type. Must be the same for all processes on a single device
     */
    uint8_t serviceType;
    
};

struct Emti_ChainRioPcb_s_
{
    /**
     * @brief   .
     */
    ti_em_dma_id_t dmaIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t dmaTxQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueIdx;

    /**
     * @brief   .
     */
    uint8_t myNodeIdx[2];

    /**
     * @brief   .
     */
    uint8_t serviceClass;
};

struct Emti_ChainRioTcb_s_
{
    /**
     * @brief   .
     */
    ti_em_queue_id_t dmaTxQueueIdx;

    /**
     * @brief   .
     */
    uint8_t myNodeIdx[2];

    /**
     * @brief   .
     */
    uint8_t serviceClass;
};

struct Emti_ChainPoststorePcb_s_
{
    /**
     * @brief   .
     */
    uint32_t localEventNum;

    /**
     * @brief   .
     */
    ti_em_queue_id_t localFreeQueueIdxTbl[2 * TI_EM_CORE_NUM];

    /**
     * @brief   .
     */
    void *localHeapPtrTbl[2 * TI_EM_CORE_NUM];

    /**
     * @brief   .
     */
    size_t poststoreSizeMax;
};

struct Emti_ChainPoststoreTcb_s_
{
    /**
     * @brief   .
     */
    uint32_t localEventNum;

    /**
     * @brief   .
     */
    ti_em_queue_id_t localFreeQueueIdxTbl[2];

    /**
     * @brief   .
     */
    void *localHeapPtrTbl[2];

    /**
     * @brief   .
     */
    size_t poststoreSizeMax;
};

struct Emti_ChainPoststoreTsb_s_
{
    /**
     * @brief   .
     */
    size_t poststoreSizeTbl[2];
};

struct Emti_PreloadPcb_s_
{
    /**
      * Preload free queues.
      */
    ti_em_queue_id_t localFreeQueueIdxTbl[TI_EM_CORE_NUM];

    /**
     * @brief   .
     */
    uint8_t preloadLineNumA;

    /**
     * @brief   .
     */
    uint8_t preloadLineNumB;

    /**
     * @brief   .
     */
    uint16_t preloadLineNumC;

    /**
     * @brief   identifies the preloading size A value.
     */
    size_t preloadSizeA;

    /**
     * @brief   identifies the preloading size B value.
     */
    size_t preloadSizeB;

    /**
     * @brief   identifies the preloading size C value.
     */
    size_t preloadSizeC;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL return queues.
     */
    ti_em_queue_id_t returnQueueBaseIdx;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL TX queues.
     */
    ti_em_queue_id_t txQueueBaseIdx;
};

struct Emti_PreloadTcb_s_
{
    /**
     * @brief.
     */
    ti_em_queue_id_t returnQueueIdx;

    /**
     * @brief.
     */
    ti_em_queue_id_t localFreeQueueIdx;

    /**
     * @brief   Pointer to the preload return queue register D (returnQueueIdx)
     */
    uint32_t returnQueueRegDPtr;

    /**
     * @brief   .
     */
    em_event_t localEventHdl;
};

struct Emti_Spcb_s_
{
    /**
     * @brief   This is a general purpose free queue which holds all the
     * atomic-processing event dsc. At init, Nprio x Ncore events are available.
     */
    ti_em_queue_id_t apFreeQueueBaseIdx;

    /**
     * @brief   Identifies the first AP queue.
     */
    ti_em_queue_id_t apQueueBaseIdx;

    /**
     * @brief   Identifies the first AP SD queue.
     */
    ti_em_queue_id_t apSdQueueBaseIdx;

    /**
     * @brief   identifies the first SD queue.
     */
    ti_em_queue_id_t sdQueueBaseIdx;

    /**
     * @brief   identifies the first prefetch (PF) queue.
     */
    ti_em_queue_id_t pfQueueBaseIdx;

    /**
     * @brief   identifies the command queue.
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL return queues.
     */
    ti_em_queue_id_t returnQueueBaseIdx;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL TX queues.
     */
    ti_em_queue_id_t txQueueBaseIdx;

    /**
     * @brief   identifies the preloading size C value.
     *          temporarly added
     */
    uint16_t preloadSizeC;

    /**
     * @brief   identifies the preloading size B value.
     *          temporarly added
     */
    uint8_t preloadSizeB;

    /**
     * @brief   identifies the preloading size A value.
     *          temporarly added
     */
    uint8_t preloadSizeA;

    /**
     * @brief   Identifies the AP mutex queue index
     */
    ti_em_queue_id_t apMutexQueueIdx;

    /**
     * @brief   pad to align on word
     */
    uint16_t processIdx;
};

struct Emti_PreloadSpcb_s_
{
    /**
     * @brief   .
     */
    uint8_t preloadLineNumA;

    /**
     * @brief   .
     */
    uint8_t preloadLineNumB;

    /**
     * @brief   .
     */
    uint16_t preloadLineNumC;

    /**
     * @brief   identifies the preloading size A value.
     */
    size_t preloadSizeA;

    /**
     * @brief   identifies the preloading size B value.
     */
    size_t preloadSizeB;

    /**
     * @brief   identifies the preloading size C value.
     */
    size_t preloadSizeC;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL return queues.
     */
    ti_em_queue_id_t returnQueueBaseIdx;

    /**
     * @brief   Identifies the first of TI_EM_CORE_NUM PL TX queues.
     */
    ti_em_queue_id_t txQueueBaseIdx;
};

struct Emti_Rdcb_s_
{
    /**
     * @brief   .
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   .
     */
    uint8_t myMacAddress[6];

    /**
     * @brief   .
     */
    uint8_t etherType[2];

     /**
     * @brief   .
     */
    ti_em_queue_id_t rxFragmentFreeQueueIdx;

     /**
     * @brief   .
     */
    ti_em_queue_id_t rxHeaderFreeQueueIdx;

     /**
     * @brief   .
     */
    ti_em_queue_id_t rxMissQueueBaseIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueBaseIdx;

     /**
     * @brief   .
     */
    ti_em_queue_id_t txQueueBaseIdx;

     /**
     * @brief   .
     */
    uint8_t vlanPrioMask;
    
    /**
     * @brief   Specifies the service type. Must be the same for all processes on a single device
     */
    uint8_t serviceType;
};

struct Emti_ChainRioRpcb_s_
{
   /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueIdx;
};


struct Emti_Rpcb_s_
{
    /**
     * @brief   .
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   identifies the release DMA Tx queue.
     */
    ti_em_queue_id_t rsTxQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rxQueueIdx;

    /**
     * @brief   identifies the first SD queue.
     */
    ti_em_queue_id_t sdQueueBaseIdx;

    /**
     * @brief   identifies the first SD queue.
     */
    uint16_t staticQueueNum;

    /**
     * @brief   .
     */
    ti_em_queue_id_t txQueueBaseIdx;

    /**
     * @brief   .
     */
    Emti_ChainRioRpcb_s chainRioDsc;

    /**
     * @brief   .
     */
    uint8_t rioEnabled;

    /**
     * @brief   .
     */
    uint8_t xgeEnabled;

    /**
     * @brief   .
     */
    ti_em_queue_id_t paTxQueueIdx;

    /**
     * @brief   .
     */
    uint8_t staticQueueNumInSet;
};

struct Emti_Esb_s_
{
    /**
     * @brief   Stores the free information of the current global event (event descriptor: word 12).
     */
    uint32_t globalFreeInfo;

    /**
     * @brief   Pointer to the current AP event descriptor. to be removed from SW AND FW structure
     */
    uint32_t apEventDscPtr;

    /**
     * @brief   Pointer to the current global event handle.
     */
    uint32_t globalEventHdl;

    /**
     * @brief   Specifies the size of the current global event (event descriptor: word 0: bits 0..21).
     */
    uint32_t globalEventSize;

    /**
     * @brief   Indicates if the scheduler succeeded in prefetching an atomic event to achieve locality.
     */
    uint32_t atomicLocalityHint;

    /**
     * @brief   .
     */
    uint8_t apLocalityHint;

    /**
     * @brief   .
     */
    void *globalEventDscPtr;

    /**
     * @brief   .
     */
    uint32_t globalEventInfo;
};

struct Emti_Spsb_s_
{
    /**
     * @brief   Priority masks for each core.
     */
    uint32_t sdQueueMaskTbl[TI_EM_CORE_NUM][Emti_SD_QUEUE_NUM >> 5];

    /**
     * @brief   .
     */
    Emti_ApCoreState_s apCoreStateTbl[TI_EM_CORE_NUM];

    /**
     * @brief   .
     */
    uint16_t apQueueLookupTbl[TI_EM_PRIO_NUM][2 * TI_EM_CORE_NUM];
};

struct Emti_Rdsb_s_
{
    /**
     * @brief   .
     */
    uint32_t wrongEtherTypeCnt;

    /**
     * @brief   .
     */
    uint32_t wrongServiceTypeCnt;

    /**
     * @brief   .
     */
    uint32_t receiveCnt;

    /**
     * @brief .
     */
    uint32_t wrongMacCnt;

    /**
     * @brief .
     */
    uint32_t wrongProcessCnt;

    /**
     * @brief   .
     */
    uint8_t processMsk;

    /**
     * @brief   .
     */
    uint8_t deviceInitDone;

    /**
     * @brief   .
     */
    uint8_t rxMissMask;

    /**
     * @brief   .
     */
    uint8_t pad;

    /**
     * @brief   .
     */
    Emti_StreamState_s streamStateTbl[TI_EM_STREAM_NUM];
};

struct Emti_Rpsb_s_
{
    /**
     * @brief   .
     */
    uint32_t errorCnt;

    /**
     * @brief   .
     */
    Emti_QueueRoute_s queueRouteTbl[TI_EM_QUEUE_SET_NUM];

    /**
     * @brief   .
     */
    Emti_QueueRoute_s staticQueueRouteTbl[EM_QUEUE_STATIC_MAX + 1];
};

struct Emti_Pcb_s_
{
    /**
     * @brief   identifies the AP command free queue.
     */
    ti_em_queue_id_t apCommandFreeQueueIdx;

    /**
     * @brief   This is a general purpose free queue which holds all the
     * atomic-processing event dsc. At init, Nprio x Ncore events are available.
     */
    ti_em_queue_id_t apFreeQueueBaseIdx;

    /**
     * @brief   Identifies the first AP queue.
     */
    ti_em_queue_id_t apQueueBaseIdx;

    /**
     * @brief   Identifies the first AP SD queue.
     */
    ti_em_queue_id_t apSdQueueBaseIdx;

    /**
     * @brief   identifies the command queue.
     */
    ti_em_queue_id_t cdQueueIdx[TI_EM_SCHEDULER_THREAD_NUM];

    /**
     * @brief   identifies the AP mutex queue for multi-threaded scheduler.
     */
    ti_em_queue_id_t apMutexQueueIdx;

    /**
     * @brief
     */
    uint8_t chainEnabled;

    /**
     * @brief   identifies the CT command free queue.
     */
    ti_em_queue_id_t ctCommandFreeQueueIdx;

    /**
     * @brief
     */
    ti_em_queue_id_t hwQueueBaseIdx;

    /**
      * Hardware Semaphore used to protect internal data structures.
      */
    ti_em_sem_id_t hwSemIdx;

    /**
      * @brief
      */
    int32_t dma_idx;

    /**
      * @brief 
      */
    ti_em_queue_id_t dma_queue_base_idx;

    /**
     * @brief   .
     */
    ti_em_device_id_t myDeviceIdx;

    /**
     * @brief   .
     */
    ti_em_process_id_t myProcessIdx;

    /**
     * @brief   .
     */
    ti_em_pdsp_id_t pdspIdxTbl[TI_EM_SCHEDULER_THREAD_NUM];

    /**
     * @brief   .
     */
    uint8_t pdspNum;

    /**
     * @brief   identifies the PF command free queue.
     */
    ti_em_queue_id_t pfCommandFreeQueueIdx;

    /**
     * @brief   identifies the first prefetch (PF) queue.
     */
    ti_em_queue_id_t pfQueueBaseIdx;

    /**
      * pool configuration descriptors.
      */
    ti_em_pool_config_t poolConfigTbl[TI_EM_POOL_NUM];

    /**
     * @brief   .
     */
    uint32_t poolNum;

    /**
     * @brief   .
     */
    uint8_t preloadEnabled;

    /**
     * @brief   .
     */
    uint8_t hwInterruptBaseIdx;

    /**
     * @brief   identifies the response queue.
     */
    ti_em_queue_id_t rpQueueIdx;

    /**
     * @brief   identifies the release DMA free queue.
     */
    ti_em_queue_id_t rsFreeQueueIdx;

    /**
     * @brief   identifies the release DMA Tx queue.
     */
    ti_em_queue_id_t rsTxQueueIdx;

    /**
     * @brief   identifies the first SD queue.
     */
    ti_em_queue_id_t sdQueueBaseIdx;

    /**
     * @brief   .
     */
    Emti_ChainPcb_s chainDsc;

    /**
     * @brief   .
     */
    Emti_ChainXgePcb_s chainXgeDsc;

    /**
     * @brief   .
     */
    Emti_ChainRioPcb_s chainRioDsc;

    /**
     * @brief   .
     */
    Emti_ChainPoststorePcb_s chainPoststoreDsc;

    /**
     * @brief   .
     */
    Emti_PreloadPcb_s preloadDsc;

    /**
     * @brief   .
     */
    Emti_Spcb_s shadowSDsc;

    /**
     * @brief   .
     */
    Emti_Spcb_s shadowPreloadSDsc;

    /**
     * @brief   .
     */
    Emti_Rpcb_s shadowRDsc;

    /**
     * @brief   .
     */
    Emti_Rdcb_s shadowRdcbDsc;

    /**
     * @brief   .
     */
    ti_em_queue_id_t shadowRChainTxQueueBaseIdx;

    /**
     * @brief   .
     */
    uint32_t shadowRCounterCommand;
    
    /**
     * @brief   array of public free queue register D.
     */
    uint32_t freeQueueRegDPtrTbl[TI_EM_POOL_NUM];

    /**
     * @brief   Base Address for the ap event dscs.
     */
    uint32_t apEventDscBasePtr;

    /**
     * @brief   Base Address for the prefetch event dscs.
     */
    uint32_t pfEventDscBasePtr;

    /**
     * @brief   set to 1 if the core is not part of an instance
     */
    uint16_t externalCore;
    
};

struct Emti_Tcb_s_
{
    /**
     * @brief   Array of AP command handles.
     */
    uint32_t apCommandHdlTbl[1 + TI_EM_PF_LEN];

    /**
     * @brief   .
     */
    ti_em_queue_id_t cdQueueIdx;

    /**
     * @brief   Pointer to the command (CD) queue register D (cdQueueIdx)
     */
    uint32_t cdQueueRegDPtr;

    /**
      * Hardware Semaphore used to protect internal data structures.
      */
    ti_em_sem_id_t hwSemIdx;

    /**
     * @brief   .
     */
    ti_em_device_id_t myDeviceIdx;

    /**
     * @brief   .
     */
    ti_em_process_id_t myProcessIdx;

    /**
     * @brief   prefetch command handle.
     */
    uint32_t pfCommandHdl;

    /**
     * @brief   .
     */
    ti_em_queue_id_t pfQueueIdx;

    /**
     * @brief   Pointer to the PF queue register D (pfQueueIdx)
     */
    uint32_t pfQueueRegDPtr;

    /**
     * pool configuration descriptors.
     */
    ti_em_pool_config_t poolConfigTbl[TI_EM_POOL_NUM];

    /**
     * @brief   .
     */
    uint32_t poolNum;

    /**
     * @brief   .
     */
    uint8_t preloadEnabled;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rpQueueIdx;

    /**
     * @brief   .
     */
    ti_em_queue_id_t rsTxQueueIdx;

    /**
     * @brief   Pointer to the release Tx queue register D (rsTxQueueIdx)
     */
    uint32_t rsTxQueueRegDPtr;

    /**
     * @brief   identifies the first SD queue.
     */
    ti_em_queue_id_t sdQueueBaseIdx;

    /**
     * @brief   .
     */
    Emti_ChainXgeTcb_s chainXgeDsc;

    /**
     * @brief   .
     */
    Emti_ChainRioTcb_s chainRioDsc;

    /**
     * @brief   .
     */
    Emti_ChainPoststoreTcb_s chainPoststoreDsc;

    /**
     * @brief   .
     */
    Emti_PreloadTcb_s preloadDsc;

     /**
     * @brief   array of public free queue register D.
     */
    uint32_t freeQueueRegDPtrTbl[TI_EM_POOL_NUM];

    /**
     * @brief   Base Address for the ap event dscs.
     */
    uint32_t apEventDscBasePtr;

    /**
     * @brief   Base Address for the prefetch event dscs.
     */
    uint32_t pfEventDscBasePtr;

    /**
     * @brief   Pointer to the AP SD queue register D
     */
    uint32_t apSdQueueRegDBasePtr;

};

struct Emti_Psb_s_
{
    /**
     * @brief   Priority masks for each core.
     */
    uint32_t coreMskTbl[TI_EM_PRIO_NUM];

    /**
     * @brief   .
     */
    Emti_DeviceRoute_s deviceRouteTbl[TI_EM_DEVICE_NUM];

    /**
     * @brief   .
     */
    em_error_handler_t errorHandler;

    /**
     * @brief   Index of the first free execution object in the database.
     */
    uint32_t nextFreeEoIdx;

    /**
     * @brief   Index of the first free event group in the database.
     */
    uint32_t nextFreeEventGroupIdx;

    /**
     * @brief   Index of the first free queue group in the database.
     */
    uint32_t nextFreeQueueGroupIdx;

    /**
     * @brief.
     */
    uint16_t nextFreeQueueSetIdx;

    /**
     * @brief   .
     */
    int8_t nextFreeSdQueueIdxTbl[TI_EM_PRIO_NUM];

    /**
     * @brief   .
     */
    Emti_ProcessRoute_s processRouteTbl[TI_EM_PROCESS_NUM];

    /**
     * @brief   .
     */
    Emti_QueueRoute_s queueRouteTbl[TI_EM_QUEUE_SET_NUM];

    /**
     * @brief   Priority masks for each core.
     */
    uint32_t sdQueueMaskTbl[TI_EM_CORE_NUM][Emti_SD_QUEUE_NUM >> 5];

    /**
     * @brief   .
     */
    Emti_QueueRoute_s staticQueueRouteTbl[EM_QUEUE_STATIC_MAX + 1];

    /**
     * @brief   .
     */
    ti_em_trace_handler_t traceHandler;

    /**
     * @brief   shadow SPSB per OpenEM process.
     */
    Emti_Spsb_s shadowSDsc;

    /**
     * @brief   shadow RPSB per OpenEM process.
     */
    Emti_Rpsb_s shadowRDsc;

    /**
     * @brief   Priority masks for each core.
     */
    uint8_t coreNum;

};

struct Emti_Psb_slow_s_
{
    /**
     * @brief   .
     */
    Emti_SdQueueState_s sdQueueStateTbl[Emti_SD_QUEUE_NUM];

    /**
     * @brief   .
     */
    Emti_HwQueueState_s hwQueueStateTbl[Emti_QM_QUEUE_NUM];
};

struct Emti_Tsb_s_
{
    /**
     * @brief   .
     */
    uint8_t actionFlags;

    /**
     * @brief index in apCommandHdlTbl of the next AP command handle to push/pop to/from the CD queue, also used for postore ping pong mechanism.
     */
    uint8_t pingPong;

    /**
     * @brief   .
     */
    em_eo_t eoHdl;

    /**
     * @brief   .
     */
    em_event_group_t eventGroupHdl;

    /**
     * @brief   .
     */
    uint32_t mFence;

    /**
     * @brief   Event State Block (ESB) per dispatcher.
     */
    Emti_Esb_s esbDsc;

    /**
     * @brief   pointer to the ESB per dispatcher.
     */
    Emti_Esb_s *esbDscPtr;

    /**
     * @brief   shadow ESB per dispatcher.
     * legacy code
     */
    ti_em_queue_id_t apSdQueueIdx;

    /**
     * @brief .
     */
    Emti_ChainPoststoreTsb_s chainPoststoreDsc;
};

#endif /*EVENT_MACHINE_PRIVATE_TYPES_H_ */
